#ifndef _CHANNEL4_MODULE_HARDWARE_H_
#define _CHANNEL4_MODULE_HARDWARE_H_

// Basisklasse einbinden.
#include "Channel4Protocol.h"

/** In diesem Module sind alle Hardwarespezifischen Funktionen enthalten.
 *
 * Die Aufteilung basiert Hauptsaechlich auf Typen und Komponenten.
 *
 */
#define IV_MODULE_HARDWARE                       0x0000

/** Prueft die Erreichbarkeit eines Geraets. Es wird nur ein einzelnes Paket versendet.

  -> {}
  <- {}
 */
#define IV_FUNCTION_PING                         0x0000

/** Setzen des Geraetetyps.
 
  -> {[]}
  <- {[]}

@arg p1[in, byte] Typangabe
                - 0 PZE Master IV
				- 1 ZK Master IV
				- 2 BDE Master IV
@arg p2[in, short] Generierter Sicherheitsschluessel.

@retval IV_ERROR_SUCCESS Erfolgreiche Durchfuehrung.
@retval <IV_ERROR_SUCCESS Modul oder Funktionsfehler.
*/
#define IV_FUNCTION_SET_TYPE_DEVICE              0x0001
// (unsigned char)(UINT16)

// Ermitteln des Geraetetyp
#define IV_FUNCTION_GET_TYPE_DEVICE              0x0002
// (unsigned char)

#define IV_VALUE_TYPE_DEVICE_PZE                 0 // PZE-MasterIV
#define IV_VALUE_TYPE_DEVICE_ZK                  1 // ZK-MasterIV
#define IV_VALUE_TYPE_DEVICE_BDE                 2 // BDE-MasterIV
#define IV_VALUE_TYPE_DEVICE_MOBIL               3 // Mobil-MasterIV
#define IV_VALUE_TYPE_DEVICE_MDE                 4 // MDE-BoxIV
#define IV_VALUE_TYPE_DEVICE_AE                  5 // AE-MasterIV
#define IV_VALUE_TYPE_DEVICE_TIMEBOY             6 // Timeboy-MasterIV

// Setzen/Ermitteln des Boardtyp (Redesignversion z.B.: 0x13) (in Nibbles)
#define IV_FUNCTION_SET_TYPE_BOARD               0x0003
#define IV_FUNCTION_GET_TYPE_BOARD               0x0004

#define IV_VALUE_TYPE_BOARD_1_0                  0x10
#define IV_VALUE_TYPE_BOARD_1_1                  0x11
#define IV_VALUE_TYPE_BOARD_1_2                  0x12
#define IV_VALUE_TYPE_BOARD_1_3                  0x13
#define IV_VALUE_TYPE_BOARD_1_4                  0x14
#define IV_VALUE_TYPE_BOARD_2_0                  0x20
#define IV_VALUE_TYPE_BOARD_2_1                  0x21


// Setzen/Ermitteln der Boardkomponente: RS485 fuer Kommunikation mit PC
#define IV_FUNCTION_SET_COMPONENT_RS485_0        0x0005
#define IV_FUNCTION_GET_COMPONENT_RS485_0        0x0006

// Setzen/Ermitteln der Boardkomponente: TCP/IP
#define IV_FUNCTION_SET_COMPONENT_TCPIP          0x0007
#define IV_FUNCTION_GET_COMPONENT_TCPIP          0x0008

// Setzen/Ermitteln der Boardkomponente: Modem MC35i/MC55
#define IV_FUNCTION_SET_TYPE_MODEM_MC            0x0009
#define IV_FUNCTION_GET_TYPE_MODEM_MC            0x000A

#define IV_VALUE_TYPE_MODEM_MC_NONE              0 // Kein GSM/GPRS
#define IV_VALUE_TYPE_MODEM_MC_39                1 // Siemens MC35i/MC39
#define IV_VALUE_TYPE_MODEM_MC_55                2 // Siemens MC55


// Setzen/Ermitteln der Boardkomponente: RS485 fuer Zutrittskontrolle
#define IV_FUNCTION_SET_COMPONENT_RS485_1        0x000B
#define IV_FUNCTION_GET_COMPONENT_RS485_1        0x000C

// Setzen/Ermitteln des Transpondertyps
#define IV_FUNCTION_SET_TYPE_TRANSPONDER         0x000D
#define IV_FUNCTION_GET_TYPE_TRANSPONDER         0x000E

#define IV_VALUE_TYPE_TRANSPONDER_NONE           0 // Kein Transponderleser
#define IV_VALUE_TYPE_TRANSPONDER_TSR30          1 // GIS TSR30
#define IV_VALUE_TYPE_TRANSPONDER_TSR32          2 // GIS TSR32
#define IV_VALUE_TYPE_TRANSPONDER_PRIMO100       3 // PHG Primo 100
#define IV_VALUE_TYPE_TRANSPONDER_MIFARE         4 // Cavitec Mifare
#define IV_VALUE_TYPE_TRANSPONDER_IBUTTON_EXTERN 5 // Dallas iButton (als separates Modul angebunden)
#define IV_VALUE_TYPE_TRANSPONDER_SMARTRELAIS    6 // SimonsVoss SmartRelais
#define IV_VALUE_TYPE_TRANSPONDER_NEDAP          7 // Nedap
#define IV_VALUE_TYPE_TRANSPONDER_IBUTTON_INTERN 8 // Dallas iButton (Fest auf Platine bestueckt)

// Setzen/Ermitteln des Displaytyp
#define IV_FUNCTION_SET_TYPE_DISPLAY             0x000F
#define IV_FUNCTION_GET_TYPE_DISPLAY             0x0010

#define IV_VALUE_TYPE_DISPLAY_NONE               0 // Kein Display
#define IV_VALUE_TYPE_DISPLAY_SMALL              1 // Lehner Dabitros (320 x 240) 4 Zoll Monochrom
#define IV_VALUE_TYPE_DISPLAY_BIG                2 // Lehner Dabitros (320 x 240) 6 Zoll Monochrom
#define IV_VALUE_TYPE_DISPLAY_SMALLER            3 // 1,9 Zoll Color

// Setzen/Ermitteln des Tastaturtyps
#define IV_FUNCTION_SET_TYPE_KEYPAD              0x0011
#define IV_FUNCTION_GET_TYPE_KEYPAD              0x0012

#define IV_VALUE_TYPE_KEYPAD_NONE                0 // Keine Tastatur
#define IV_VALUE_TYPE_KEYPAD_9                   1 // 9 Tasten Tastatur (PZE-MasterIV usw.)
#define IV_VALUE_TYPE_KEYPAD_58                  2 // 56+2 Tasten Tastatur (BDE-MasterIV usw.)
#define IV_VALUE_TYPE_KEYPAD_29                  3 // 28+1 Tasten Tastatur (AE-MasterIV usw.)
#define IV_VALUE_TYPE_KEYPAD_7                   4 // 7 Tasten Tastatur (PZE-MasterIV Basic)
#define IV_VALUE_TYPE_KEYPAD_25                  5 // 25 Tasten Tastatur (Timeboy-MasterIV usw.)

// Setzen/Ermitteln der Multimediakarte
#define IV_FUNCTION_SET_TYPE_MMC                 0x0013
#define IV_FUNCTION_GET_TYPE_MMC                 0x0014

#define IV_VALUE_TYPE_MMC_NONE                   0 // Keine MMC eingebaut
#define IV_VALUE_TYPE_MMC_EXTENSION              1 // Als Speichererweiterung
#define IV_VALUE_TYPE_MMC_DISK                   2 // Als Wechseldatentraeger
#define IV_VALUE_TYPE_MMC_USB                    3 // USB-Host als Speichererweiterung

// Setzen/Ermitteln des GPS - Typs
#define IV_FUNCTION_SET_TYPE_GPS                 0x0015
#define IV_FUNCTION_GET_TYPE_GPS                 0x0016

#define IV_VALUE_TYPE_GPS_NONE                   0 // Kein GPS
#define IV_VALUE_TYPE_GPS_GN80                   1 // Furuno GN-80


// Setzen/Ermitteln der USB - Komponente
#define IV_FUNCTION_SET_COMPONENT_USB            0x0017
#define IV_FUNCTION_GET_COMPONENT_USB            0x0018

// Setzen/Ermitteln der Hardwareconfigurationsnummer
#define IV_FUNCTION_SET_HW_CONFIG                0x0019
#define IV_FUNCTION_GET_HW_CONFIG                0x001A

// Setzen/Ermitteln des Funkmodultyps
#define IV_FUNCTION_SET_TYPE_RADIO               0x001B
#define IV_FUNCTION_GET_TYPE_RADIO               0x001C

#define IV_VALUE_TYPE_RADIO_NONE                 0 // Kein Funk
#define IV_VALUE_TYPE_RADIO_WRAPTHOR             1 // Bluegiga Wrap Thor


// Setzen/Ermitteln des Funkuhrmodultyps
#define IV_FUNCTION_SET_TYPE_DCF77               0x001D
#define IV_FUNCTION_GET_TYPE_DCF77               0x001E

#define IV_VALUE_TYPE_DCF77_NONE                 0 // Keine Funkuhr
#define IV_VALUE_TYPE_DCF77_HKW                  1 // HKW


// Setzen/Ermitteln der Option fuer analoge Eingaenge (32bit)
#define IV_FUNCTION_SET_COMPONENT_IN_ANALOG      0x001F
#define IV_FUNCTION_GET_COMPONENT_IN_ANALOG      0x0020

#define IV_VALUE_COMPONENT_IN_ANALOG_NONE        0x00000000
#define IV_VALUE_COMPONENT_IN_ANALOG_1           0x00000001
#define IV_VALUE_COMPONENT_IN_ANALOG_2           0x00000003


// Setzen/Ermitteln der Option fuer digitale Eingaenge (32bit)
#define IV_FUNCTION_SET_COMPONENT_IN_DIGITAL     0x0021
#define IV_FUNCTION_GET_COMPONENT_IN_DIGITAL     0x0022

#define IV_VALUE_COMPONENT_IN_DIGITAL_NONE       0x00000000
#define IV_VALUE_COMPONENT_IN_DIGITAL_1          0x00000001
#define IV_VALUE_COMPONENT_IN_DIGITAL_2          0x00000003
#define IV_VALUE_COMPONENT_IN_DIGITAL_3          0x00000007
#define IV_VALUE_COMPONENT_IN_DIGITAL_4          0x0000000F
#define IV_VALUE_COMPONENT_IN_DIGITAL_5          0x0000001F
#define IV_VALUE_COMPONENT_IN_DIGITAL_6          0x0000003F
#define IV_VALUE_COMPONENT_IN_DIGITAL_7          0x0000007F
#define IV_VALUE_COMPONENT_IN_DIGITAL_8          0x000000FF


// Setzen/Ermitteln der Option fuer digitale Ausgaenge (32bit)
#define IV_FUNCTION_SET_COMPONENT_OUT_DIGITAL    0x0023
#define IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL    0x0024

#define IV_VALUE_COMPONENT_OUT_DIGITAL_NONE      0x00000000
#define IV_VALUE_COMPONENT_OUT_DIGITAL_1         0x00000001
#define IV_VALUE_COMPONENT_OUT_DIGITAL_2         0x00000003
#define IV_VALUE_COMPONENT_OUT_DIGITAL_3         0x00000007
#define IV_VALUE_COMPONENT_OUT_DIGITAL_4         0x0000000F


// Setzen/Ermitteln des Fingerprinttyps
#define IV_FUNCTION_SET_TYPE_FINGERPRINT         0x0025
#define IV_FUNCTION_GET_TYPE_FINGERPRINT         0x0026

#define IV_VALUE_TYPE_FINGERPRINT_NONE           0
#define IV_VALUE_TYPE_FINGERPRINT_BIOKEY_3000    1 // Idencom (BioKey 3000 - Modul)

// Setzen/Ermitteln der Typen der Externen Lesermodule
#define IV_FUNCTION_SET_TYPE_EXTERNAL_READER     0x0027
#define IV_FUNCTION_GET_TYPE_EXTERNAL_READER     0x0028

#define IV_VALUE_TYPE_EXTERNAL_READER_UNDEFINED  0x00 // Undefined
#define IV_VALUE_TYPE_EXTERNAL_READER_GIS        0x01 // GIS
#define IV_VALUE_TYPE_EXTERNAL_READER_PHG        0x02 // PHG
#define IV_VALUE_TYPE_EXTERNAL_READER_PROTECT    0x04 // Protect

// Setzen/Ermitteln der Optionen fuer die Firmware
#define IV_FUNCTION_SET_OPTION_IOMODULE          0x0029
#define IV_FUNCTION_GET_OPTION_IOMODULE          0x002A

#define IV_VALUE_OPTION_IOMODULE_UNDEFINED       0xFF // Undefiniert
#define IV_VALUE_OPTION_IOMODULE_NONE            0x00 // Keines
#define IV_VALUE_OPTION_IOMODULE                 0x01 // IOModule Anbindung benoetigt

// Setzen/Ermitteln der Optionen fuer die Firmware
#define IV_FUNCTION_SET_OPTION_COMSERVER         0x002B
#define IV_FUNCTION_GET_OPTION_COMSERVER         0x002C

#define IV_VALUE_OPTION_COMSERVER_UNDEFINED      0xFF // Undefiniert
#define IV_VALUE_OPTION_COMSERVER_NONE           0x00 // Keines
#define IV_VALUE_OPTION_COMSERVER                0x01 // ComServerMode wird benoetigt

// Setzen/Ermitteln der Optionen fuer die Firmware
#define IV_FUNCTION_SET_OPTION_TBADAPTION        0x002D
#define IV_FUNCTION_GET_OPTION_TBADAPTION        0x002E

#define IV_VALUE_OPTION_TBADAPTION_UNDEFINED     0xFF // Undefiniert
#define IV_VALUE_OPTION_TBADAPTION_NONE          0x00 // Keines
#define IV_VALUE_OPTION_TBADAPTION               0x01 // ComServerMode wird benoetigt


class CChannel4ModuleHardware : public CChannel4Protocol
{
public:
	/// Konstruktor fuer Initialisierung der Klasse.
	CChannel4ModuleHardware()  : CChannel4Protocol(IV_DEFAULT_CHANNELID, IV_DEFAULT_DEVICEID)
	{ InitClass(); }
	CChannel4ModuleHardware(short channelID, short deviceID) : CChannel4Protocol(channelID, deviceID) 
	{ InitClass(channelID, deviceID); }
	virtual void InitClass(short channelID = IV_DEFAULT_CHANNELID, short deviceID = IV_DEFAULT_DEVICEID);
	/// Destruktor
	virtual ~CChannel4ModuleHardware();

public:
	// Gibt Modulfehler zurueck.
	virtual void GetErrorModule(short module, short error, char *text);
	// Gibt Funktionsfehlertext zurueck.
	virtual void GetErrorFunction(short module, short function, short error, char *text);

	// Einfache Anfrage ob Geraet mit Protokollversion 4 erreichbar.
	int Request(short deviceIDRemote);

	// MODULE_HARDWARE

	// Einfache Anfrage ob das Geraet antwortet.
	int Ping(short deviceIDRemote);
	int PingRandom(short deviceIDRemote);

	// BUG:
	// Im Bootloader 00.04.00.00 && 00.04.00.01 liefert die Funktion
	// - TypeTransponder den Wert fuer das Display
	// - TypeDisplay den Wert fuer das Keypad
	// - TypeKeypad den Wert fuer den Transponder
	
	// Typ schreiben/lesen
	int SetTypeDevice(short deviceIDRemote, unsigned char value);
	int GetTypeDevice(short deviceIDRemote, unsigned char &value);
	int SetTypeBoard(short deviceIDRemote, unsigned char value);
	int GetTypeBoard(short deviceIDRemote, unsigned char &value);
	int SetTypeModemMC(short deviceIDRemote, unsigned char value);
	int GetTypeModemMC(short deviceIDRemote, unsigned char &value);
	int SetTypeDisplay(short deviceIDRemote, unsigned char value, int checked = FALSE);
	int GetTypeDisplay(short deviceIDRemote, unsigned char &value, int checked = FALSE);
	int SetTypeKeypad(short deviceIDRemote, unsigned char value, int checked = FALSE);
	int GetTypeKeypad(short deviceIDRemote, unsigned char &value, int checked = FALSE);
	int SetTypeTransponder(short deviceIDRemote, unsigned char value, int checked = FALSE);
	int GetTypeTransponder(short deviceIDRemote, unsigned char &value, int checked = FALSE);
	int SetTypeMMC(short deviceIDRemote, unsigned char value);
	int GetTypeMMC(short deviceIDRemote, unsigned char &value);
	int SetTypeRadio(short deviceIDRemote, unsigned char value);
	int GetTypeRadio(short deviceIDRemote, unsigned char &value);
	int SetTypeGPS(short deviceIDRemote, unsigned char value);
	int GetTypeGPS(short deviceIDRemote, unsigned char &value);
	int SetTypeDCF77(short deviceIDRemote, unsigned char value);
	int GetTypeDCF77(short deviceIDRemote, unsigned char &value);
	int SetTypeFingerprint(short deviceIDRemote, unsigned char value);
	int GetTypeFingerprint(short deviceIDRemote, unsigned char &value);
	int SetTypeExternalReader(short deviceIDRemote, unsigned char value);
	int GetTypeExternalReader(short deviceIDRemote, unsigned char &value);

	// Komponente schreiben/lesen
	int SetComponentRS485_0(short deviceIDRemote, unsigned char value);
	int GetComponentRS485_0(short deviceIDRemote, unsigned char &value);
	int SetComponentTCPIP(short deviceIDRemote, unsigned char value);
	int GetComponentTCPIP(short deviceIDRemote, unsigned char &value);
	int SetComponentRS485_1(short deviceIDRemote, unsigned char value);
	int GetComponentRS485_1(short deviceIDRemote, unsigned char &value);
	int SetComponentUSB(short deviceIDRemote, unsigned char value);
	int GetComponentUSB(short deviceIDRemote, unsigned char &value);
	int SetComponentInAnalog(short deviceIDRemote, unsigned long value);
	int GetComponentInAnalog(short deviceIDRemote, unsigned long &value);
	int SetComponentInDigital(short deviceIDRemote, unsigned long value);
	int GetComponentInDigital(short deviceIDRemote, unsigned long &value);
	int SetComponentOutDigital(short deviceIDRemote, unsigned long value);
	int GetComponentOutDigital(short deviceIDRemote, unsigned long &value);

	// Optionen schreiben/lesen
	int SetOptionIOModule(short deviceIDRemote, unsigned char value);
	int GetOptionIOModule(short deviceIDRemote, unsigned char &value);
	int SetOptionComServer(short deviceIDRemote, unsigned char value);
	int GetOptionComServer(short deviceIDRemote, unsigned char &value);
	int SetOptionTBAdaption(short deviceIDRemote, unsigned char value);
	int GetOptionTBAdaption(short deviceIDRemote, unsigned char &value);
	

	// HWConfig lesen
	int SetHWConfig(short deviceIDRemote, unsigned char value);
	int GetHWConfig(short deviceIDRemote, unsigned char &value);

	//#################################
	// CallRoutinen

	// Ausfuehren von Befehlen
	virtual int CallFunction(short module, short function, unsigned char *data, int &count);

	// Ping ausfuehren
	int CallPing(unsigned char *data, int &count);

	// Typ lesen
//	int CallSetTypeDevice(unsigned char *data, int &count);
	int CallGetTypeDevice(unsigned char *data, int &count);
//	int CallSetTypeBoard(unsigned char *data, int &count);
	int CallGetTypeBoard(unsigned char *data, int &count);
//	int CallSetTypeModemMC(unsigned char *data, int &count);
	int CallGetTypeModemMC(unsigned char *data, int &count);
//	int CallSetTypeDisplay(unsigned char *data, int &count);
	int CallGetTypeDisplay(unsigned char *data, int &count);
//	int CallSetTypeKeypad(unsigned char *data, int &count);
	int CallGetTypeKeypad(unsigned char *data, int &count);
//	int CallSetTypeTransponder(unsigned char *data, int &count);
	int CallGetTypeTransponder(unsigned char *data, int &count);
//	int CallSetTypeMMC(unsigned char *data, int &count);
	int CallGetTypeMMC(unsigned char *data, int &count);
//	int CallSetTypeRadio(unsigned char *data, int &count);
	int CallGetTypeRadio(unsigned char *data, int &count);
//	int CallSetTypeGPS(unsigned char *data, int &count);
	int CallGetTypeGPS(unsigned char *data, int &count);
//	int CallSetTypeDCF77(unsigned char *data, int &count);
	int CallGetTypeDCF77(unsigned char *data, int &count);
//	int CallSetTypeFingerprint(unsigned char *data, int &count);
	int CallGetTypeFingerprint(unsigned char *data, int &count);
//	int CallSetTypeExternalReader(unsigned char *data, int &count);
	int CallGetTypeExternalReader(unsigned char *data, int &count);

	// Komponente lesen
//	int CallSetComponentRS485_0(unsigned char *data, int &count);
	int CallGetComponentRS485_0(unsigned char *data, int &count);
//	int CallSetComponentTCPIP(unsigned char *data, int &count);
	int CallGetComponentTCPIP(unsigned char *data, int &count);
//	int CallSetComponentRS485_1(unsigned char *data, int &count);
	int CallGetComponentRS485_1(unsigned char *data, int &count);
//	int CallSetComponentUSB(unsigned char *data, int &count);
	int CallGetComponentUSB(unsigned char *data, int &count);
//	int CallSetComponentInAnalog(unsigned char *data, int &count);
	int CallGetComponentInAnalog(unsigned char *data, int &count);
//	int CallSetComponentInDigital(unsigned char *data, int &count);
	int CallGetComponentInDigital(unsigned char *data, int &count);
//	int CallSetComponentOutDigital(unsigned char *data, int &count);
	int CallGetComponentOutDigital(unsigned char *data, int &count);

	// HWConfig lesen
//	int CallSetHWConfig(unsigned char *data, int &count);
	int CallGetHWConfig(unsigned char *data, int &count);
};

#endif
