/**
 * @file  Channel4ModuleSystem.cpp
 * @brief Implementierung der Klasse CChannel4ModuleSystem.
 */
//////////////////////////////////////////////////////////////////////

//#include "StdAfx.h"
#include "Channel4ModuleSystem.h"
#include <stdio.h>

// ##################################################################
/**
 * @brief Initialisieren der Membervariablen.
 *
 * @param id Nummer der Kanalklasse fuer z.B. Trace.
 */
void CChannel4ModuleSystem::InitClass(short channelID, short deviceID)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::InitClass [ %i, %i ]\n", __FILE__, __LINE__, channelID, deviceID);
}

// ##################################################################
/**
 * @brief Destruktor.
 *
 */
CChannel4ModuleSystem::~CChannel4ModuleSystem() 
{
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void CChannel4ModuleSystem::GetErrorModule(short module, short error, char *text)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::GetErrorModule [ %i, %i, 0x%08X ]\n", __FILE__, __LINE__, module, error, text);

	// Ist IV_MODULE_SYSTEM ?
	if (module == IV_MODULE_SYSTEM) {

		switch(error) {
		case 1: sprintf(text, "( %i ): Das geforderte Modul wird nicht untersttzt.", error); break;
		case 2: sprintf(text, "( %i ): Die geforderte Funktion wird nicht untersttzt.", error); break;
		case 3: sprintf(text, "( %i ): Es wurden bei der Funktion zu wenige Paramteter angegeben.", error); break;
		case 4: sprintf(text, "( %i ): Es wurden bei der Funktion zu viele Paramteter angegeben.", error); break;
		default: sprintf(text, "( %i ): Nicht eingetragener Modulfehler!", error);
		}

		return;
	}

	// Ermittlung des Fehlertextes an Basisklasse abgeben.
	CChannel4ModuleHardware::GetErrorModule(module, error, text);
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void CChannel4ModuleSystem::GetErrorFunction(short module, short function, short error, char *text)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::GetErrorFunction [ %i, %i, %i, 0x%08X ]\n", __FILE__, __LINE__, module, function, error, text);

	if (module == IV_MODULE_SYSTEM) {

		switch(function) {
		case IV_FUNCTION_SYSTEM_RESET:
			switch(error) {
			case 1: sprintf(text, "( %i ): Resetmodus unbekannt.", error); break;
			}
			break;
		case IV_FUNCTION_SET_SERIAL:
			switch(error) {
			case 1: sprintf(text, "( %i ): Pruefung der Seriennummer - Verschluesselung schlug fehl.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_GET_VERSION:
		case IV_FUNCTION_GET_VERSION_BOOTLOADER:
		case IV_FUNCTION_GET_SERIAL:
		case IV_FUNCTION_SET_RTC:
		case IV_FUNCTION_GET_RTC:
			switch(error) {
			case 1:sprintf(text, "( %i ): ", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_SET_SYSTEM_VARIABLE:
		case IV_FUNCTION_GET_SYSTEM_VARIABLE:
			switch(error) {
			case 1:sprintf(text, "( %i ): Der Majorbestandteil des Variablennamens ist unbekannt.", error); break;
			case 2:sprintf(text, "( %i ): Der Minorbestandteil des Variablennamens ist unbekannt.", error); break;
			case 3:sprintf(text, "( %i ): Der zu setzende Variablenwert ist zu lang.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_SET_RELAY:
		case IV_FUNCTION_GET_RELAY:
			switch(error) {
			case 1:sprintf(text, "( %i ): Bereichsname unbekannt. (INTERN, TMR, IOM ...)", error); break;
			case 2:sprintf(text, "( %i ): Busnummer (z.B. TMR) nicht bekannt oder erreichbar.", error); break;
			case 3:sprintf(text, "( %i ): Relaisnummer unbekannt.", error); break;
			case 4:sprintf(text, "( %i ): Zu setzender Zustand (Status) unbekannt oder nicht untersttzt.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_SET_FLASH:
			switch(error) {
			case 1:sprintf(text, "( %i ): Speicherseite konnte nicht geschrieben werden.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_GET_FLASH:
		case IV_FUNCTION_SET_EEPROM:
		case IV_FUNCTION_GET_EEPROM:
		case IV_FUNCTION_SET_PROGRAM:
		case IV_FUNCTION_GET_PROGRAM:
			switch(error) {
			case 1:sprintf(text, "( %i ): ", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_BEGIN_SET_SEGMENT:
		case IV_FUNCTION_END_SET_SEGMENT:
			switch(error) {
			case 1:sprintf(text, "( %i ): Der Typ ist unbekannt.", error); break;
			case 2:sprintf(text, "( %i ): Die Updatesegmente sind erschpft.", error); break;
			case 3:sprintf(text, "( %i ): Der Angeforderte Speicherplatz steht nicht zur Verfgung.", error); break;
			case 4:sprintf(text, "( %i ): Es wurde kein STOP vorm START durchgefhrt.", error); break;
			case 5:sprintf(text, "( %i ): Verschachtelungsfehler von STOP und START.", error); break;
			case 6:sprintf(text, "( %i ): Prfsumme (CRC) des Updatesegmentes stimmt nicht mit gegebener berein.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_BEGIN_SET_SEGMENT_OK:
		case IV_FUNCTION_END_SET_SEGMENT_OK:
			break;
		case IV_FUNCTION_BEGIN_GET_SEGMENT:
		case IV_FUNCTION_END_GET_SEGMENT:
			switch(error) {
			case 1:sprintf(text, "( %i ): Der Typ ist unbekannt.", error); break;
			case 2:sprintf(text, "( %i ): Die Updatesegmente sind erschpft.", error); break;
			case 3:sprintf(text, "( %i ): Der Bentigte Speicherplatz steht nicht zur Verfgung.", error); break;
			case 4:sprintf(text, "( %i ): Es wurde kein STOP vorm START durchgefhrt.", error); break;
			case 5:sprintf(text, "( %i ): Verschachtelungsfehler von STOP und START.", error); break;
			case 6:sprintf(text, "( %i ): Prfsumme (CRC) des Updatesegmentes stimmt nicht mit gegebener berein.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_BEGIN_GET_SEGMENT_OK:
		case IV_FUNCTION_END_GET_SEGMENT_OK:
			break;
		case IV_FUNCTION_ENTER_BIOS:
		case IV_FUNCTION_EXIT_BIOS:
			switch(error) {
			case 1:sprintf(text, "( %i ): ", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_ENTERED_BIOS:
		case IV_FUNCTION_EXITED_BIOS:
			break;
		case IV_FUNCTION_GET_INFO:
			switch(error) {
			case 1:sprintf(text, "( %i ): Der Majorbestandteil des Modulnamens ist unbekannt.", error); break;
			case 2:sprintf(text, "( %i ): Der Minorbestandteil des Modulnamens ist unbekannt.", error); break;
			case 3: sprintf(text, "( %i ): Es wurden zu wenige Paramteter angegeben.", error); break;
			case 4: sprintf(text, "( %i ): Es wurden zu viele Paramteter angegeben.", error); break;
			case 5: sprintf(text, "( %i ): Der Wert eines Parameters liegt auserhalb seines Gltigkeitsbereichs.", error); break;
			case 6: sprintf(text, "( %i ): Die Information zur Komponente steht nicht zur Verfgung.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_OPEN_COM_SERVER_MODE:
			switch(error) {
			case 1: sprintf(text, "( %i ): Es wurden zu wenige Paramteter angegeben.", error); break;
			case 2: sprintf(text, "( %i ): Es wurden zu viele Paramteter angegeben.", error); break;
			case 3: sprintf(text, "( %i ): Die Angabe des comm Parameters wird nicht untersttzt.", error); break;
			case 4: sprintf(text, "( %i ): Die Angabe des baudRate Parameters wird nicht untersttzt.", error); break;
			case 5: sprintf(text, "( %i ): Die Angabe des byteSize Parameters wird nicht untersttzt.", error); break;
			case 6: sprintf(text, "( %i ): Die Angabe des parity Paramters wird nicht untersttzt.", error); break;
			case 7: sprintf(text, "( %i ): Die Angabe des stopBits Paramters wird nicht untersttzt.", error); break;
			case 8: sprintf(text, "( %i ): Die Angabe des flags Parameters wird nicht untersttzt.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_CLOSE_COM_SERVER_MODE:
			switch(error) {
			case 1: sprintf(text, "( %i ): Bei der bertragung ist ein Overflow eingetreten.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_OPEN_COM_SERVER_MODE_OK:
			break;

		default: sprintf(text, "( %i ): Nicht eingetragener Fehlerblock fr Funktion!", error); break;
		}
		
		return;
	}

	// Ermittlung des Fehlertextes an Basisklasse abgeben.
	CChannel4ModuleHardware::GetErrorFunction(module, function, error, text);
}

//#########################################################
//################# HARDWARE BUGFIX #######################
//#########################################################

// ##################################################################
/**
 * @brief berschreibt eine gleichnamige Funktion aus dem zugrundeliegenden Module
 *        und stellt einen Bugfix da.
 *
 */
int CChannel4ModuleSystem::GetTypeDisplay(short deviceIDRemote, unsigned char &value, int checked)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::GetTypeDisplay [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	// Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
	// - Funktionen TypeTransponder liefern <-> Displaywert
	// - Funktionen TypeDisplay liefern <-> Keypadwert
	// - Funktionen TypeKeypad liefern <-> Transponderwert
	int res;
	char version[64];
	memset(version, 0, sizeof(version));
	// Befindet sich im Bootloader?
	if (!checked && (res = GetVersion(deviceIDRemote, version)) != IV_ERROR_SUCCESS) {
		if ((res > 0) && (((res >> 8) & 0xFF) == 2)) {

			// Befehl wird nicht unterstuetzt
			// Geraet befindet sich im Bootloadermodus
			if ((res = GetVersionBootloader(deviceIDRemote, version)) != IV_ERROR_SUCCESS) {

				return res;
			}

			// Ist eine von den gesuchten Versionen?
			if (strstr(version, "00.04.00.00") || strstr(version, "00.04.00.01")) {
			
				// Auf entsprechende Funktion verzweigen
				return CChannel4ModuleHardware::GetTypeTransponder(deviceIDRemote, value, TRUE);
			}
		}
	}

	return CChannel4ModuleHardware::GetTypeDisplay(deviceIDRemote, value, TRUE);
}

// ##################################################################
/**
 * @brief berschreibt eine gleichnamige Funktion aus dem zugrundeliegenden Module
 *        und stellt einen Bugfix da.
 *
 */
int CChannel4ModuleSystem::GetTypeKeypad(short deviceIDRemote, unsigned char &value, int checked)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::GetTypeKeypad [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	// Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
	// - Funktionen TypeTransponder liefern <-> Displaywert
	// - Funktionen TypeDisplay liefern <-> Keypadwert
	// - Funktionen TypeKeypad liefern <-> Transponderwert
	int res;
	char version[64];
	memset(version, 0, sizeof(version));
	// Befindet sich im Bootloader?
	if (!checked && (res = GetVersion(deviceIDRemote, version)) != IV_ERROR_SUCCESS) {
		if ((res > 0) && (((res >> 8) & 0xFF) == 2)) {

			// Befehl wird nicht unterstuetzt
			// Geraet befindet sich im Bootloadermodus
			if ((res = GetVersionBootloader(deviceIDRemote, version)) != IV_ERROR_SUCCESS) {

				return res;
			}

			// Ist eine von den gesuchten Versionen?
			if (strstr(version, "00.04.00.00") || strstr(version, "00.04.00.01")) {
			
				// Auf entsprechende Funktion verzweigen
				return CChannel4ModuleHardware::GetTypeDisplay(deviceIDRemote, value, TRUE);
			}
		}
	}

	return CChannel4ModuleHardware::GetTypeKeypad(deviceIDRemote, value, TRUE);
}

// ##################################################################
/**
 * @brief berschreibt eine gleichnamige Funktion aus dem zugrundeliegenden Module
 *        und stellt einen Bugfix da.
 *
 */
int CChannel4ModuleSystem::GetTypeTransponder(short deviceIDRemote, unsigned char &value, int checked)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::GetTypeTransponder [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	// Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
	// - Funktionen TypeTransponder liefern <-> Displaywert
	// - Funktionen TypeDisplay liefern <-> Keypadwert
	// - Funktionen TypeKeypad liefern <-> Transponderwert
	int res;
	char version[64];
	memset(version, 0, sizeof(version));
	// Befindet sich im Bootloader?
	if (!checked && (res = GetVersion(deviceIDRemote, version)) != IV_ERROR_SUCCESS) {
		if ((res > 0) && (((res >> 8) & 0xFF) == 2)) {

			// Befehl wird nicht unterstuetzt
			// Geraet befindet sich im Bootloadermodus
			if ((res = GetVersionBootloader(deviceIDRemote, version)) != IV_ERROR_SUCCESS) {

				return res;
			}

			// Ist eine von den gesuchten Versionen?
			if (strstr(version, "00.04.00.00") || strstr(version, "00.04.00.01")) {
			
				// Auf entsprechende Funktion verzweigen
				return CChannel4ModuleHardware::GetTypeKeypad(deviceIDRemote, value, TRUE);
			}
		}
	}

	return CChannel4ModuleHardware::GetTypeTransponder(deviceIDRemote, value, TRUE);
}

int CChannel4ModuleSystem::GetHardwareInfo(short deviceIDRemote, TDFHardwareInfo &hwInfo)
{
	int res = IV_ERROR_SUCCESS;

	// Struktur initialisieren
	memset(&hwInfo, 0, sizeof(TDFHardwareInfo));

	// Geraetetyp ermitteln
	if ((res = GetTypeDevice(deviceIDRemote, hwInfo.typeDevice)) != IV_ERROR_SUCCESS) {
		
		// Wenn der Modulfehler -Funktion unbekannt- ist,
		// dann liegt eine FW 04.01.01.<k vor.
		// Nicht der Modulfehler -Funktion unbekannt-
		if (!(((res >> 8) & 0xFF) == 2)) {

			return res;
		}
		// Fehlerobjekt zurueck setzen
		res = IV_ERROR_SUCCESS;
	}

	// Boardtyp ermitteln
	if ((res = GetTypeBoard(deviceIDRemote, hwInfo.typeBoard)) != IV_ERROR_SUCCESS) {

		// Nicht der Modulfehler -Funktion unbekannt-
		if (!(((res >> 8) & 0xFF) == 2)) {

			return res;
		}
		// Fehlerobjekt zurueck setzen
		res = IV_ERROR_SUCCESS;
	}
	
	// RS485 ermitteln
	if ((res = GetComponentRS485_0(deviceIDRemote, hwInfo.componentRS485_0)) != IV_ERROR_SUCCESS) {
		
		// Nicht der Modulfehler -Funktion unbekannt-
		if (!(((res >> 8) & 0xFF) == 2)) {

			return res;
		}
		// Fehlerobjekt zurueck setzen
		res = IV_ERROR_SUCCESS;
	}
	
	// TCPIP ermitteln
	if ((res = GetComponentTCPIP(deviceIDRemote, hwInfo.componentTCPIP)) != IV_ERROR_SUCCESS) {		
		
		// Nicht der Modulfehler -Funktion unbekannt-
		if (!(((res >> 8) & 0xFF) == 2)) {

			return res;
		}
		// Fehlerobjekt zurueck setzen
		res = IV_ERROR_SUCCESS;
	}
	
	// ModemMC35i ermitteln
	if ((res = GetTypeModemMC(deviceIDRemote, hwInfo.typeModemMC)) != IV_ERROR_SUCCESS) {
		
		// Nicht der Modulfehler -Funktion unbekannt-
		if (!(((res >> 8) & 0xFF) == 2)) {

			return res;
		}
		// Fehlerobjekt zurueck setzen
		res = IV_ERROR_SUCCESS;
	}
	
	// RS485 (intern) ermitteln
	if ((res = GetComponentRS485_1(deviceIDRemote, hwInfo.componentRS485_1)) != IV_ERROR_SUCCESS) {
		
		// Nicht der Modulfehler -Funktion unbekannt-
		if (!(((res >> 8) & 0xFF) == 2)) {

			return res;
		}
		// Fehlerobjekt zurueck setzen
		res = IV_ERROR_SUCCESS;
	}
	
	// Typ ermitteln (erst ab 04.01.01.p / Bootloader 4.0)
	if ((res = GetTypeDisplay(deviceIDRemote, hwInfo.typeDisplay)) != IV_ERROR_SUCCESS) {
		
		// Wenn der Modulfehler -Funktion unbekannt- ist,
		// dann liegt eine FW 04.01.01.<p vor.
		// Nicht der Modulfehler -Funktion unbekannt-
		if (!(((res >> 8) & 0xFF) == 2)) {

			return res;
		}
		// Fehlerobjekt zurueck setzen
		res = IV_ERROR_SUCCESS;
	}
	
	// Typ ermitteln
	if ((res = GetTypeKeypad(deviceIDRemote, hwInfo.typeKeypad)) != IV_ERROR_SUCCESS) {

		// Nicht der Modulfehler -Funktion unbekannt-
		if (!(((res >> 8) & 0xFF) == 2)) {

			return res;
		}
		// Fehlerobjekt zurueck setzen
		res = IV_ERROR_SUCCESS;
	}
	
	// Typ ermitteln
	if ((res = GetTypeTransponder(deviceIDRemote, hwInfo.typeTransponder)) != IV_ERROR_SUCCESS) {
		
		// Nicht der Modulfehler -Funktion unbekannt-
		if (!(((res >> 8) & 0xFF) == 2)) {

			return res;
		}
		// Fehlerobjekt zurueck setzen
		res = IV_ERROR_SUCCESS;
	}
	
	// Typ ermitteln
	if ((res = GetTypeMMC(deviceIDRemote, hwInfo.typeMMC)) != IV_ERROR_SUCCESS) {
		
		// Nicht der Modulfehler -Funktion unbekannt-
		if (!(((res >> 8) & 0xFF) == 2)) {

			return res;
		}
		// Fehlerobjekt zurueck setzen
		res = IV_ERROR_SUCCESS;
	}
	
	// Typ ermitteln
	if ((res = GetTypeRadio(deviceIDRemote, hwInfo.typeRadio)) != IV_ERROR_SUCCESS) {
		
		// Nicht der Modulfehler -Funktion unbekannt-
		if (!(((res >> 8) & 0xFF) == 2)) {

			return res;
		}
		// Fehlerobjekt zurueck setzen
		res = IV_ERROR_SUCCESS;
	}
	
	// Typ ermitteln
	if ((res = GetTypeGPS(deviceIDRemote, hwInfo.typeGPS)) != IV_ERROR_SUCCESS) {
		
		// Nicht der Modulfehler -Funktion unbekannt-
		if (!(((res >> 8) & 0xFF) == 2)) {

			return res;
		}
		// Fehlerobjekt zurueck setzen
		res = IV_ERROR_SUCCESS;
	}
	
	// Komponende ermitteln
	if ((res = GetComponentUSB(deviceIDRemote, hwInfo.componentUSB)) != IV_ERROR_SUCCESS) {
		
		// Nicht der Modulfehler -Funktion unbekannt-
		if (!(((res >> 8) & 0xFF) == 2)) {

			return res;
		}
		// Fehlerobjekt zurueck setzen
		res = IV_ERROR_SUCCESS;
	}
	
	// Typ ermitteln
	if ((res = GetTypeDCF77(deviceIDRemote, hwInfo.typeDCF77)) != IV_ERROR_SUCCESS) {
		
		// Nicht der Modulfehler -Funktion unbekannt-
		if (!(((res >> 8) & 0xFF) == 2)) {

			return res;
		}
		// Fehlerobjekt zurueck setzen
		res = IV_ERROR_SUCCESS;
	}
	
	// Komponende ermitteln
	if ((res = GetComponentInAnalog(deviceIDRemote, hwInfo.componentInAnalog)) != IV_ERROR_SUCCESS) {
		
		// Nicht der Modulfehler -Funktion unbekannt-
		if (!(((res >> 8) & 0xFF) == 2)) {

			return res;
		}
		// Fehlerobjekt zurueck setzen
		res = IV_ERROR_SUCCESS;
	}
	
	// Komponende ermitteln
	if ((res = GetComponentInDigital(deviceIDRemote, hwInfo.componentInDigital)) != IV_ERROR_SUCCESS) {
		
		// Nicht der Modulfehler -Funktion unbekannt-
		if (!(((res >> 8) & 0xFF) == 2)) {

			return res;
		}
		// Fehlerobjekt zurueck setzen
		res = IV_ERROR_SUCCESS;
	}
	
	// Komponende ermitteln
	if ((res = GetComponentOutDigital(deviceIDRemote, hwInfo.componentOutDigital)) != IV_ERROR_SUCCESS) {
		
		// Nicht der Modulfehler -Funktion unbekannt-
		if (!(((res >> 8) & 0xFF) == 2)) {

			return res;
		}
		// Fehlerobjekt zurueck setzen
		res = IV_ERROR_SUCCESS;
	}
	
	// Typ ermitteln
	if ((res = GetTypeFingerprint(deviceIDRemote, hwInfo.typeFingerprint)) != IV_ERROR_SUCCESS) {
		
		// Nicht der Modulfehler -Funktion unbekannt-
		if (!(((res >> 8) & 0xFF) == 2)) {

			return res;
		}
		// Fehlerobjekt zurueck setzen
		res = IV_ERROR_SUCCESS;
	}

	hwInfo.typeExternalReader = 0xFF;
	// Typ ermitteln
	if ((res = GetTypeExternalReader(deviceIDRemote, hwInfo.typeExternalReader)) != IV_ERROR_SUCCESS) {
		
		// Nicht der Modulfehler -Funktion unbekannt-
		if (!(((res >> 8) & 0xFF) == 2)) {

			return res;
		}
		// Fehlerobjekt zurueck setzen
		res = IV_ERROR_SUCCESS;
	}

	// Wenn ZK Option vorhanden
	if (hwInfo.componentRS485_1) {
		// aber der Externe Lesertyp undefiniert ist,
		// dann wird von TS-Lesern ausgegangen.
		if (hwInfo.typeExternalReader == 0xFF) {

			hwInfo.typeExternalReader = IV_VALUE_TYPE_EXTERNAL_READER_GIS;
		}
	} else {

		// Keine Externen leser
		hwInfo.typeExternalReader = 0;
	}
	// BUGFIX
	// Eine Firmware die die Serien TS und Protect untersttzt setzt als untersttzte Serien
	// TS und Protect obwohl es so sein sollte, dass die angesprochene Serie eingetragen wird,
	// so dass der Wert der Hardwareinfo eindeutig ist.
	// Werden beide Flags vorgefunden oder Protect alleine wird von TS alleine ausgegangen.
	if (hwInfo.typeExternalReader & 0x04) {

		hwInfo.typeExternalReader = 0x01;
	}

	// Option ermitteln
	if ((res = GetOptionIOModule(deviceIDRemote, hwInfo.optionIOModule)) != IV_ERROR_SUCCESS) {
		
		// Nicht der Modulfehler -Funktion unbekannt-
		if (!(((res >> 8) & 0xFF) == 2)) {

			return res;
		}
		// Fehlerobjekt zurueck setzen
		res = IV_ERROR_SUCCESS;
	}

	// Option ermitteln
	if ((res = GetOptionTBAdaption(deviceIDRemote, hwInfo.optionTBAdaption)) != IV_ERROR_SUCCESS) {
		
		// Nicht der Modulfehler -Funktion unbekannt-
		if (!(((res >> 8) & 0xFF) == 2)) {

			return res;
		}
		// Fehlerobjekt zurueck setzen
		res = IV_ERROR_SUCCESS;
	}
	
	return res;
}

int CChannel4ModuleSystem::GetInfos(TDFHardwareInfo hwInfo, char *info)
{
	char tmp[0x100];
	unsigned long bit;
	int res = 1;

	if (hwInfo.typeBoard == 0) {

		memset(tmp, 0, sizeof(tmp));
		strcpy(tmp, "Es liegen keine Informationen vor.");
		strcat(info, tmp);
		return 0;
	}

	memset(tmp, 0, sizeof(tmp));
	strcpy(tmp, "Enthaltene Optionen/Module\r\n");
	strcat(tmp, "------------------------------------------\r\n");
	strcat(info, tmp);

	memset(tmp, 0, sizeof(tmp));
	strcat(tmp, "\r\nBasis\r\n----------\r\n");
	strcat(info, tmp);

	strcat(info, "Geraete: ");
	memset(tmp, 0, sizeof(tmp));
	// Moegliche Geraetetypen
	switch(hwInfo.typeDevice) {
	case IV_VALUE_TYPE_DEVICE_PZE:     strcat(tmp, "PZE-MasterIV"); break;
	case IV_VALUE_TYPE_DEVICE_ZK:      strcat(tmp, "ZK-MasterIV");  break;
	case IV_VALUE_TYPE_DEVICE_BDE:     strcat(tmp, "BDE-MasterIV"); break;
	case IV_VALUE_TYPE_DEVICE_MOBIL:   strcat(tmp, "Mobil-MasterIV"); break;
	case IV_VALUE_TYPE_DEVICE_MDE:     strcat(tmp, "MDE-BoxIV"); break;
	case IV_VALUE_TYPE_DEVICE_AE:      strcat(tmp, "AE-MasterIV"); break;
	case IV_VALUE_TYPE_DEVICE_TIMEBOY: strcat(tmp, "Timeboy-MasterIV"); break;
	default:                           strcat(tmp, "Undefiniert"); break;
	}
	strcat(tmp, "\r\n");
	strcat(info, tmp);

	strcat(info, "Platinen: ");
	memset(tmp, 0, sizeof(tmp));
	// Boardtyp pruefen
	switch(hwInfo.typeBoard) {
	case IV_VALUE_TYPE_BOARD_1_0: strcat(tmp, "1.0"); break;
	case IV_VALUE_TYPE_BOARD_1_1: strcat(tmp, "1.1"); break;
	case IV_VALUE_TYPE_BOARD_1_2: strcat(tmp, "1.2"); break;
	case IV_VALUE_TYPE_BOARD_1_3: strcat(tmp, "1.3"); break;
	case IV_VALUE_TYPE_BOARD_1_4: strcat(tmp, "1.4"); break;
	case IV_VALUE_TYPE_BOARD_2_0: strcat(tmp, "2.0"); break;
	case IV_VALUE_TYPE_BOARD_2_1: strcat(tmp, "2.1"); break;
	default:                      strcat(tmp, "Undefiniert"); break;
	}
	strcat(tmp, "\r\n");
	strcat(info, tmp);

	/////////////////////////////////////////////
	// Hauptkommunikationen
	memset(tmp, 0, sizeof(tmp));
	strcat(tmp, "\r\nHauptkommunikationen\r\n------------------------------\r\n");
	strcat(info, tmp);
	
	// Funk pruefen
	strcat(info, "Bluetooth: ");
	memset(tmp, 0, sizeof(tmp));
	if (hwInfo.typeRadio) {

		switch(hwInfo.typeRadio) {
		case IV_VALUE_TYPE_RADIO_WRAPTHOR: strcat(tmp, "Ja"); break;
		default:                           strcat(tmp, "Undefiniert"); break;
		}
	} else {
		strcat(tmp, "Nein");
	}
	strcat(tmp, "\r\n");
	strcat(info, tmp);

	// ModemMCxx pruefen
	strcat(info, "Mobil-Funkmodem: ");
	memset(tmp, 0, sizeof(tmp));
	if (hwInfo.typeModemMC) {

		switch(hwInfo.typeModemMC) {
		case IV_VALUE_TYPE_MODEM_MC_39: strcat(tmp, "Siemens MC39"); break;
		case IV_VALUE_TYPE_MODEM_MC_55: strcat(tmp, "Siemens MC55"); break;
		default:         strcat(tmp, "Undefiniert"); break;
		}
	} else {
		strcat(tmp, "Nein");
	}
	strcat(tmp, "\r\n");
	strcat(info, tmp);

	// RS485 pruefen
	sprintf(tmp, "RS485 Kommunikation: %s\r\n", hwInfo.componentRS485_0 ? "Ja" : "Nein");
	strcat(info, tmp);

	// TCPIP pruefen
	sprintf(tmp, "TCP/IP Kommunikation: %s\r\n", hwInfo.componentTCPIP ? "Ja" : "Nein");
	strcat(info, tmp);

	// USB pruefen
	sprintf(tmp, "USB Kommunikation: %s\r\n", hwInfo.componentUSB ? "Ja" : "Nein");
	strcat(info, tmp);

	/////////////////////////////////////////////
	// Zusatzoptionen
	memset(tmp, 0, sizeof(tmp));
	strcat(tmp, "\r\nZustzliche Optionen der Firmware\r\n------------------------------\r\n");
	strcat(info, tmp);

	// USB pruefen
	sprintf(tmp, "IO-Modul: %s\r\n", hwInfo.optionIOModule ? "Ja" : "Nein");
	strcat(info, tmp);

	// USB pruefen
	sprintf(tmp, "Timeboyanbindung: %s\r\n", hwInfo.optionTBAdaption ? "Ja" : "Nein");
	strcat(info, tmp);

	/////////////////////////////////////////////
	// Zusatzbaugruppen
	memset(tmp, 0, sizeof(tmp));
	strcat(tmp, "\r\nZusatzbaugruppen\r\n------------------------------\r\n");
	strcat(info, tmp);

	// Display pruefen
	strcat(info, "Anzeige: ");
	memset(tmp, 0, sizeof(tmp));
	if (hwInfo.typeDisplay) {

		switch(hwInfo.typeDisplay) {
		case IV_VALUE_TYPE_DISPLAY_SMALL:   strcat(tmp, "4 Zoll Monochrome"); break;
		case IV_VALUE_TYPE_DISPLAY_BIG:     strcat(tmp, "6 Zoll Monochrome"); break;
		case IV_VALUE_TYPE_DISPLAY_SMALLER: strcat(tmp, "1,9 Zoll Color"); break;
		default:                            strcat(tmp, "Undefiniert"); break;
		}
	} else {
		strcat(tmp, "Nein");
	}
	strcat(tmp, "\r\n");
	strcat(info, tmp);

	// DigitalOut pruefen
	strcat(info, "Ausgaenge Digital: ");
	memset(tmp, 0, sizeof(tmp));
	if (hwInfo.componentOutDigital) {

		for (bit=1;; bit <<= 1) {

			if (bit & hwInfo.componentOutDigital) {

				if (strlen(tmp)) {
					strcat(tmp, ", ");
				}

				switch(bit) { // Digitale Ausgaenge
				case 0x01: strcat(tmp, "dO1"); break;
				case 0x02: strcat(tmp, "dO2"); break;
				case 0x04: strcat(tmp, "dO3"); break;
				case 0x08: strcat(tmp, "dO4"); break;
				default:   strcat(tmp, "Undefiniert"); break;
				}
			}

			if (bit == 0x8000000) break;
		}
	} else {
		strcat(tmp, "Nein");
	}
	strcat(tmp, "\r\n");
	strcat(info, tmp);

	// AnalogIn pruefen
	strcat(info, "Eingaenge Analog: ");
	memset(tmp, 0, sizeof(tmp));
	if (hwInfo.componentInAnalog) {

		for (bit=1;; bit <<= 1) {

			if (bit & hwInfo.componentInAnalog) {

				if (strlen(tmp)) {
					strcat(tmp, ", ");
				}

				switch(bit) { // Analoge Eingaenge
				case 0x01: strcat(tmp, "aI1"); break;
				case 0x02: strcat(tmp, "aI2"); break;
				default:   strcat(tmp, "Undefiniert"); break;
				}
			}

			if (bit == 0x8000000) break;
		}
	} else {
		strcat(tmp, "Nein");
	}
	strcat(tmp, "\r\n");
	strcat(info, tmp);

	// DigitalIn pruefen
	strcat(info, "Eingaenge Digital: ");
	memset(tmp, 0, sizeof(tmp));
	if (hwInfo.componentInDigital) {

		for (bit=1;; bit <<= 1) {

			if (bit & hwInfo.componentInDigital) {

				if (strlen(tmp)) {
					strcat(tmp, ", ");
				}

				switch(bit) { // Digitale Eingaenge
				case 0x01: strcat(tmp, "dI1"); break;
				case 0x02: strcat(tmp, "dI2"); break;
				case 0x04: strcat(tmp, "dI3"); break;
				case 0x08: strcat(tmp, "dI4"); break;
				case 0x10: strcat(tmp, "dI5"); break;
				case 0x20: strcat(tmp, "dI6"); break;
				case 0x40: strcat(tmp, "dI7"); break;
				case 0x80: strcat(tmp, "dI8"); break;
				default:   strcat(tmp, "Undefiniert"); break;
				}
			}

			if (bit == 0x8000000) break;
		}
	} else {
		strcat(tmp, "Nein");
	}
	strcat(tmp, "\r\n");
	strcat(info, tmp);

	// Fingerprint pruefen
	strcat(info, "Fingerprint: ");
	memset(tmp, 0, sizeof(tmp));
	if (hwInfo.typeFingerprint) {

		switch(hwInfo.typeFingerprint) {
		case IV_VALUE_TYPE_FINGERPRINT_BIOKEY_3000: strcat(tmp, "Ja"); break;
		default:                                    strcat(tmp, "Undefiniert"); break;
		}
	} else {
		strcat(tmp, "Nein");
	}
	strcat(tmp, "\r\n");
	strcat(info, tmp);
	
	// DCF77 pruefen
	strcat(info, "Funkuhr: ");
	memset(tmp, 0, sizeof(tmp));
	if (hwInfo.typeDCF77) {

		switch(hwInfo.typeDCF77) {
		case IV_VALUE_TYPE_DCF77_HKW:  strcat(tmp, "Ja"); break;
		default:                       strcat(tmp, "Undefiniert"); break;
		}
	} else {
		strcat(tmp, "Nein");
	}
	strcat(tmp, "\r\n");
	strcat(info, tmp);

	// GPS pruefen
	strcat(info, "GPS: ");
	memset(tmp, 0, sizeof(tmp));
	if (hwInfo.typeGPS) {

		switch(hwInfo.typeGPS) {
		case IV_VALUE_TYPE_GPS_GN80: strcat(tmp, "Ja"); break;
		default:                     strcat(tmp, "Undefiniert"); break;
		}
	} else {
		strcat(tmp, "Nein");
	}
	strcat(tmp, "\r\n");
	strcat(info, tmp);

	// MMC pruefen
	strcat(info, "MM Card: ");
	memset(tmp, 0, sizeof(tmp));
	if (hwInfo.typeMMC) {

		switch(hwInfo.typeMMC) {
		case IV_VALUE_TYPE_MMC_EXTENSION: strcat(tmp, "Speichererweiterung"); break;
		case IV_VALUE_TYPE_MMC_DISK:      strcat(tmp, "Wechseldatentrger"); break;
		default:                          strcat(tmp, "Undefiniert"); break;
		}
	} else {
		strcat(tmp, "Nein");
	}
	strcat(tmp, "\r\n");
	strcat(info, tmp);

	// Tastatur pruefen
	strcat(info, "Tastatur: ");
	memset(tmp, 0, sizeof(tmp));
	if (hwInfo.typeKeypad) {

		switch(hwInfo.typeKeypad) {
		case IV_VALUE_TYPE_KEYPAD_9:  strcat(tmp, "9 Tasten"); break;
		case IV_VALUE_TYPE_KEYPAD_58: strcat(tmp, "58 Tasten"); break;
		case IV_VALUE_TYPE_KEYPAD_29: strcat(tmp, "29 Tasten"); break;
		case IV_VALUE_TYPE_KEYPAD_7:  strcat(tmp, "7 Tasten"); break;
		case IV_VALUE_TYPE_KEYPAD_25: strcat(tmp, "25 Tasten"); break;
		default:                      strcat(tmp, "Undefiniert"); break;
		}

	} else {
		strcat(tmp, "Nein");
	}
	strcat(tmp, "\r\n");
	strcat(info, tmp);

	// Transponder pruefen
	strcat(info, "Transponderleser: ");
	memset(tmp, 0, sizeof(tmp));
	if (hwInfo.typeTransponder) {

		switch(hwInfo.typeTransponder) {
		case IV_VALUE_TYPE_TRANSPONDER_TSR30:          strcat(tmp, "[EM4102, Hitag2]"); break;
		case IV_VALUE_TYPE_TRANSPONDER_TSR32:          strcat(tmp, "[EM4102, EM4450, Hitag]"); break;
		case IV_VALUE_TYPE_TRANSPONDER_PRIMO100:       strcat(tmp, "[Legic]"); break;
		case IV_VALUE_TYPE_TRANSPONDER_MIFARE:         strcat(tmp, "[Mifare]"); break;
		case IV_VALUE_TYPE_TRANSPONDER_IBUTTON_EXTERN: strcat(tmp, "[IButton (Extern)]"); break;
		case IV_VALUE_TYPE_TRANSPONDER_SMARTRELAIS:    strcat(tmp, "[SimonsVoss]"); break;
		case IV_VALUE_TYPE_TRANSPONDER_NEDAP:          strcat(tmp, "[Nedap]"); break;
		case IV_VALUE_TYPE_TRANSPONDER_IBUTTON_INTERN: strcat(tmp, "[IButton (Intern)]"); break;
		default:                                       strcat(tmp, "[Undefiniert]"); break;
		}
	} else {
		strcat(tmp, "Nein");
	}
	strcat(tmp, "\r\n");
	strcat(info, tmp);
	
	// RS485 (intern) ermitteln
	strcat(info, "Zutrittskontrolle: ");
	memset(tmp, 0, sizeof(tmp));
	if (hwInfo.componentRS485_1) {

		switch(hwInfo.typeExternalReader) {
		case IV_VALUE_TYPE_EXTERNAL_READER_GIS:     strcat(tmp, "TS-Serie"); break;
		case IV_VALUE_TYPE_EXTERNAL_READER_PHG:     strcat(tmp, "PHG-Serie"); break;
		case IV_VALUE_TYPE_EXTERNAL_READER_PROTECT: strcat(tmp, "Protect-Serie"); break;
		case 0xFF:                                  strcat(tmp, "Unbekannt"); break;
		default:                                    strcat(tmp, "Undefiniert"); break;
		}
	} else {
		strcat(tmp, "Nein");
	}
	strcat(tmp, "\r\n");
	strcat(info, tmp);

	return res;
}

//#########################################################
//####################### SYSTEM ##########################
//#########################################################

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::CallFunction(short module, short function, unsigned char *data, int &count)
{
	if (module == IV_MODULE_SYSTEM) {
		
		int res = IV_ERROR_SUCCESS;
		
		switch(function) {
		// Ermitteln der Firmwareversion
		case IV_FUNCTION_GET_VERSION: return CallGetVersion(data, count);

		// Setzen/Ermitteln von: SerialNum
//		case IV_FUNCTION_SET_SERIAL: return CallSetSerial(data, count);
		case IV_FUNCTION_GET_SERIAL: return CallGetSerial(data, count);
	
		// Setzen/Ermitteln von: DateTime (YYYY-MM-DD HH:MM:SS\0)
		case IV_FUNCTION_SET_RTC: return CallSetRTC(data, count);
		case IV_FUNCTION_GET_RTC: return CallGetRTC(data, count);

		// Setzen/Ermitteln von: Variable (Systemvariablen GPRS) 
		case IV_FUNCTION_SET_SYSTEM_VARIABLE: return CallSetSystemVariable(data, count);
		case IV_FUNCTION_GET_SYSTEM_VARIABLE: return CallGetSystemVariable(data, count);

		// Setzen/Ermitteln von: Relaiszustand 
		case IV_FUNCTION_SET_RELAY: return CallSetRelay(data, count);
		case IV_FUNCTION_GET_RELAY: return CallGetRelay(data, count);

		// Schreiben/Lesen von: Externen Flash
		case IV_FUNCTION_SET_FLASH: return CallSetFlash(data, count);
		case IV_FUNCTION_GET_FLASH: return CallGetFlash(data, count);

		// Schreiben/Lesen von: EEPROM
		case IV_FUNCTION_SET_EEPROM: return CallSetEEPROM(data, count);
		case IV_FUNCTION_GET_EEPROM: return CallGetEEPROM(data, count);

		// Schreiben/Lesen von: Program-Flash schreiben/lesen
//		case IV_FUNCTION_SET_PROGRAM: return CallSetProgram(data, count);
//		case IV_FUNCTION_GET_PROGRAM: return CallGetProgram(data, count);

		// Starten/Beenden von: Segmentuebertragung (inklusive aktiver Antwort)
		case IV_FUNCTION_BEGIN_SET_SEGMENT: {
			if ((res = CallBeginSetSegment(data, count)) == IV_ERROR_SUCCESS) {

				// Bei (den einzelnen innenliegenden) Systemstopps,
				// werden die erfolgreich berechneten CRC - Pruefungen zurueckgemeldet.
				if (((*(unsigned short*)data) == IV_VALUE_SEGMENT_UPDATE) || 
					((*(unsigned short*)data) == IV_VALUE_SEGMENT_UPDATE_BG) || 
					((*(unsigned short*)data) == IV_VALUE_SEGMENT_LIST_ACCESS)) {

					count = sizeof(unsigned short);
					return Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM , IV_FUNCTION_BEGIN_SET_SEGMENT_OK, m_data, count);
				}

			}

			return res;}
		case IV_FUNCTION_BEGIN_SET_SEGMENT_OK: return CallBeginSetSegmentOK(data, count);
		
		case IV_FUNCTION_END_SET_SEGMENT: return CallEndSetSegment(data, count);
		case IV_FUNCTION_END_SET_SEGMENT_OK: return CallEndSetSegmentOK(data, count);

		// Ermitteln von: Bootloaderversion
		case IV_FUNCTION_GET_VERSION_BOOTLOADER: return CallGetVersionBootloader(data, count);

		// Durchfuehren eines Resets der Speicherbereiche
//		case IV_FUNCTION_SYSTEM_RESET: return CallSystemReset(data, count);

		// Ins Bios wechseln
		case IV_FUNCTION_ENTER_BIOS: return CallEnterBios(data, count);
		// Bioswechsel erfolgt
//		case IV_FUNCTION_ENTERED_BIOS: return CallEnteredBios(data, count);

		// Bios verlassen (aenderungen uebernehmen
		case IV_FUNCTION_EXIT_BIOS: return CallExitBios(data, count);
		// Biosverlassen erfolgt
//		case IV_FUNCTION_EXITED_BIOS: return CallExitedBios(data, count);

		// Setzt/liest das zu verwendende Interface
		case IV_FUNCTION_SET_INTERFACE: return CallSetInterface(data, count);
		case IV_FUNCTION_GET_INTERFACE: return CallGetInterface(data, count);

		// Setzt/liest die zu verwendende BaudRate
		case IV_FUNCTION_SET_BAUD: return CallSetBaud(data, count);
		case IV_FUNCTION_GET_BAUD: return CallGetBaud(data, count);

		// Setzt/liest die zu verwendende DeviceID
		case IV_FUNCTION_SET_DEVICE_ID: return CallSetDeviceIDBios(data, count);
		case IV_FUNCTION_GET_DEVICE_ID: return CallGetDeviceIDBios(data, count);

		// Befehl der beim Geraetestart den Bootloader des ZK - Master IV wiederherstellt.
		case IV_FUNCTION_ENTER_BOOTLOADER: return CallEnterBootloader(data, count);

		// In Monitor Mode von XPort wechselt
		case IV_FUNCTION_ENTER_XPORT_MONITOR: return CallEnterXPortMonitor(data, count);
		// Antwort in Monitor Mode gewechselt
		case IV_FUNCTION_ENTER_XPORT_MONITOR_OK: return CallEnterXPortMonitor(data, count);

		// Monitorbefehl an XPort und Antwort erhalten.
		case IV_FUNCTION_CMD_XPORT: return CallCommandXPort(data, count);

		// In Monitor Mode von XPort wechselt
		case IV_FUNCTION_EXIT_XPORT_MONITOR: return CallExitXPortMonitor(data, count);
		// Antwort in Monitor Mode gewechselt
//		case IV_FUNCTION_EXIT_XPORT_MONITOR_OK: return CallExitXPortMonitorOK(data, count);


		case IV_FUNCTION_BEGIN_GET_SEGMENT: return CallBeginGetSegment(data, count);
//		case IV_FUNCTION_BEGIN_GET_SEGMENT_OK: return CallBeginGetSegmentOK(data, count);

		case IV_FUNCTION_END_GET_SEGMENT: return CallEndGetSegment(data, count);

		default: return 0x0002; // Funktion nicht definiert
		}
	}

	return CChannel4ModuleHardware::CallFunction(module, function, data, count);
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::SystemReset(short deviceIDRemote, unsigned short modus)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::SystemReset [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, modus);

	int count = sizeof(unsigned short);

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Den Resetmodus uebernehmen.
	*(unsigned short*)m_data = modus;

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM , IV_FUNCTION_SYSTEM_RESET, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_RESET);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::SystemReset [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	// Aktive Antwort abwarten
	if ((m_result = Execute(IV_CONTEXT_PENDING, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_RESET_OK, m_data, count)) != IV_ERROR_SUCCESS ) {

		// Fehler ausgeben
		TraceError(GetDeviceIDRemote(), IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_RESET_OK);
		return m_result;
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Leited einen zu bertragenden Datenblock ein. Z. B. Listen.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::BeginSetSegment(short deviceIDRemote, unsigned short art, unsigned long &address, unsigned long size, unsigned long crc)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::BeginSetSegment [ %i, %i, %i, %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, art, address, size, crc);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Meberpuffer aufbereiten
	*(unsigned short*)m_data = art;
	count = sizeof(unsigned short);

	// addresse wegen wohlgeformtem Paket mitgeben
	*(unsigned long*)&m_data[count] = address;
	count += sizeof(unsigned long);

	// geforderte Speichergroesze mitgeben
	*(unsigned long*)&m_data[count] = size;
	count += sizeof(unsigned long);

	// CRC32 fuer Datenblock mitgeben
	*(unsigned long*)&m_data[count] = crc;
	count += sizeof(unsigned long);

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_SET_SEGMENT, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_SET_SEGMENT);
		return m_result;
	}

	// OK
	
	// Ausgeben der empfangenen Werte

	if (count == 14) {

		if (art != *(unsigned short*)m_data) {
			Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::BeginSetSegment [ Art ist (%i) soll (%i) ]\n", __FILE__, __LINE__, *(unsigned short*)m_data, art);
		}
		
		address = *(unsigned long*)&m_data[sizeof(unsigned short)];

		if (size != *(unsigned long*)&m_data[sizeof(unsigned short) + sizeof(unsigned long)]) {
			Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::BeginSetSegment [ Size ist (%i) soll (%i) ]\n", __FILE__, __LINE__, *(unsigned long*)&m_data[sizeof(unsigned short) + sizeof(unsigned long)], size);
		}

		if (crc != *(unsigned long*)&m_data[sizeof(unsigned short) + sizeof(unsigned long)*2]) {
			Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::BeginSetSegment [ CRC ist (%i) soll (%i) ]\n", __FILE__, __LINE__, *(unsigned long*)&m_data[sizeof(unsigned short) + sizeof(unsigned long)*2], crc);
		}

		Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::BeginSetSegment (0x%04X)(0x%08X)(0x%08X)(0x%08X)\n", __FILE__, __LINE__, art, address, size, crc);
	} else {

		Trace("%s:(%i):ERROR(1): CChannel4ModuleSystem::BeginSetSegment [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
	}

	// Bei (den einzelnen innenliegenden) Systemstopps,
	// werden die erfolgreich berechneten CRC - Pruefungen zurueckgemeldet.
	if ((art == IV_VALUE_SEGMENT_UPDATE) || (art == IV_VALUE_SEGMENT_UPDATE_BG) ||
		(art == IV_VALUE_SEGMENT_LIST_ACCESS)) {
/*
		short module, function, error = IV_ERROR_SUCCESS;
		// nachdem der CRC geprueft wurde, kommt ein Bestaetigungspaket,
		// dieses muss hier seperat abgegriffen werden.
		if ((m_result = Pending(deviceIDRemote, module, function, error, m_data, count)) != IV_ERROR_SUCCESS) {

			Trace("%s:(%i):ERROR(1): CChannel4ModuleSystem::BeginSetSegment [ Quittierungsprfantwort - CRC schlug fehl. ]\n", __FILE__, __LINE__);
			return m_result;
		}

		if ((module == IV_MODULE_SYSTEM) && (function == IV_FUNCTION_BEGIN_SET_SEGMENT_OK)) {

			Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::BeginSetSegment [ Antwort von CRC OK. ]\n", __FILE__, __LINE__);
		}

		// evntl. aktiven Fehler auswerten
		if (error != IV_ERROR_SUCCESS) {

			m_result = error;
			// Fehler ausgeben
			TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_SET_SEGMENT_OK);
			return m_result;
		}

		if (*(unsigned short*)m_data != art) {

			Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::BeginSetSegment [ Die Art der Quittierung ist ungleich. ]\n", __FILE__, __LINE__);
		} 
		
		if (count > sizeof(unsigned short)) {

			Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::BeginSetSegment [ Datenmenge ist mit (%i) grsser als erwartet. ]\n", __FILE__, __LINE__, count);
		}
*/
		if ((m_result = Execute(IV_CONTEXT_PENDING, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_SET_SEGMENT_OK, m_data, count)) != IV_ERROR_SUCCESS ) {

			Trace("%s:(%i):ERROR(1): CChannel4ModuleSystem::BeginSetSegment [ Quittierungsprfantwort - CRC schlug fehl. ]\n", __FILE__, __LINE__);
			return m_result;
		}

		if (*(unsigned short*)m_data != art) {

			Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::BeginSetSegment [ Die Art der Quittierung ist ungleich. ]\n", __FILE__, __LINE__);
		} 
	}

	return m_result;
}

int CChannel4ModuleSystem::CallBeginSetSegment(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallBeginSetSegment [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);
/*
// Die folgenden IV_VALUE_SEGMENT_.. gelten fuer _BEGIN_SEGMENT und _END_SEGMENT
#define IV_VALUE_SEGMENT_UPDATE         0 // Es werden Daten gesetzt die nachdem Uebernommen werden.
#define IV_VALUE_SEGMENT_UPDATE_BG      1 // Es werden Daten gesetzt die nachdem im Hintergrund uebernommen werden.
#define IV_VALUE_SEGMENT_FIRMWARE       2
#define IV_VALUE_SEGMENT_FONTS          3
#define IV_VALUE_SEGMENT_GRAFICS        4
#define IV_VALUE_SEGMENT_SETUP          5
#define IV_VALUE_SEGMENT_LIST           6
#define IV_VALUE_SEGMENT_SETUP_EXTENDED 7
#define IV_VALUE_SEGMENT_LIST_ACCESS    8 // Listen fuer ZK II werden uebertragen, dadurch wird die Zutrittskontrolle blockiert.

 */

	return IV_ERROR_SUCCESS;
}

int CChannel4ModuleSystem::CallBeginSetSegmentOK(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallBeginSetSegmentOK [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	if (count > sizeof(unsigned short)) {
		
		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::CallBeginSetSegmentOK [ Datenmenge ist mit (%i) grsser als erwartet. ]\n", __FILE__, __LINE__, count);
	}

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::EndSetSegment(short deviceIDRemote, unsigned short art, unsigned long crc)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::EndSetSegment [ %i, %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, art, crc);

	int count = sizeof(unsigned short);

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Meberpuffer aufbereiten
	*(unsigned short*)m_data = art;

	// Beendigung einer Uebertragung eines Datanblocks
	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT);
		return m_result;
	}

	// OK
	if (art != *(unsigned short*)m_data) {
		Trace("%s:(%i):ERROR(1): CChannel4ModuleSystem::BeginSetSegment [ Art ist (%i) soll (%i) ]\n", __FILE__, __LINE__, *(unsigned short*)m_data, art);
	}
	
	// Bei (den einzelnen innenliegenden) Systemstopps,
	// werden die erfolgreich berechneten CRC - Pruefungen zurueckgemeldet.
/*	
	short module, function, error = IV_ERROR_SUCCESS;
	// nachdem der CRC geprueft wurde, kommt ein Bestaetigungspaket,
	// dieses musz hier seperat abgegriffen werden.
	if ((m_result = Pending(deviceIDRemote, module, function, error, m_data, count)) != IV_ERROR_SUCCESS) {
		
		Trace("%s:(%i):ERROR(1): CChannel4ModuleSystem::BeginSetSegment [ Quittierungsprfantwort - CRC schlug fehl (%i) ]\n", __FILE__, __LINE__, m_result);
		return m_result;
	}
	
	if ((module == IV_MODULE_SYSTEM) && (function == IV_FUNCTION_END_SET_SEGMENT_OK)) {
		
		Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::BeginSetSegment [ Antwort von CRC OK ]\n", __FILE__, __LINE__);
	}

	// evntl. aktiven Fehler auswerten
	if (error != IV_ERROR_SUCCESS) {

		m_result = error;
		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT_OK);
		return m_result;
	}
*/
	if ((m_result = Execute(IV_CONTEXT_PENDING, IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT_OK, m_data, count)) != IV_ERROR_SUCCESS ) {

		// Fehler ausgeben
		TraceError(GetDeviceIDRemote(), IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT_OK);
		return m_result;
	}

	if (*(unsigned short*)m_data != art) {
		
		Trace("%s:(%i):ERROR(1): CChannel4ModuleSystem::BeginSetSegment [ Art ist (%i) soll (%i) ]\n", __FILE__, __LINE__, *(unsigned short*)m_data, art);
	} else if (*(unsigned long*)&m_data[sizeof(unsigned short)] != crc) {
		
		Trace("%s:(%i):ERROR(1): CChannel4ModuleSystem::BeginSetSegment [ CRC ist (0x%08X) soll (0x%08X) ]\n", __FILE__, __LINE__, *(unsigned long*)&m_data[sizeof(unsigned short)], crc);
		return -IV_ERROR_CRC_SEGMENT;
	}
	
	return m_result;
}

int CChannel4ModuleSystem::CallEndSetSegment(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallEndSetSegment [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	return IV_ERROR_SUCCESS;
}

int CChannel4ModuleSystem::CallEndSetSegmentOK(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallEndSetSegmentOK [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Leited einen zu bertragenden Datenblock ein. Z. B. Listen.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::BeginGetSegment(short deviceIDRemote, unsigned short art, unsigned long &address, unsigned long &size, unsigned long &crc)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::BeginGetSegment [ %i, %i, %i, %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, art, address, size, crc);

	int count = sizeof(unsigned short);

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Meberpuffer aufbereiten
	*(unsigned short*)m_data = art;

	// addresse wegen wohlgeformtem Paket mitgeben
	*(unsigned long*)&m_data[sizeof(unsigned short)] = address;
	count += sizeof(unsigned long);

	// geforderte Speichergroesze mitgeben
	*(unsigned long*)&m_data[sizeof(unsigned short) + sizeof(unsigned long)] = size;
	count += sizeof(unsigned long);

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_GET_SEGMENT, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_GET_SEGMENT);
		return m_result;
	}

	// OK
	
	// Ausgeben der empfangenen Werte

	if (count == 10) {

		if (art != *(unsigned short*)m_data) {
			Trace("%s:(%i):ERROR(1): CChannel4ModuleSystem::BeginSetSegment [ Art ist (%i) soll (%i) ]\n", __FILE__, __LINE__, *(unsigned short*)m_data, art);
		}
		
		address = *(unsigned long*)&m_data[sizeof(unsigned short)];
		size    = *(unsigned long*)&m_data[sizeof(unsigned short) + sizeof(unsigned long)];

		Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::BeginSetSegment [ (0x%04X)(0x%08X)(0x%08X) ]\n", __FILE__, __LINE__, art, address, size);
	} else {

		Trace("%s:(%i):ERROR(1): CChannel4ModuleSystem::BeginGetSegment [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
	}
/*
	short module, function, error = IV_ERROR_SUCCESS;
	// nachdem der CRC geprueft wurde, kommt ein Bestaetigungspaket,
	// dieses musz hier seperat abgegriffen werden.
	if ((m_result = Pending(deviceIDRemote, module, function, error, m_data, count)) != IV_ERROR_SUCCESS) {

		Trace("%s:(%i):ERROR(1): CChannel4ModuleSystem::BeginGetSegment.Pending [ Quittierungsprfantwort - CRC schlug fehl (%i) ]\n", __FILE__, __LINE__, m_result);
		return m_result;
	}

	if ((module == IV_MODULE_SYSTEM) && (function == IV_FUNCTION_BEGIN_SET_SEGMENT_OK)) {

		Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::BeginGetSegment.Pending [ Antwort von CRC OK ]\n", __FILE__, __LINE__);
	}

	// evntl. aktiven Fehler auswerten
	if (error != IV_ERROR_SUCCESS) {

		m_result = error;
		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_SET_SEGMENT_OK);
		return m_result;
	}
*/
	if ((m_result = Execute(IV_CONTEXT_PENDING, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_SET_SEGMENT_OK, m_data, count)) != IV_ERROR_SUCCESS ) {

		// Fehler ausgeben
		TraceError(GetDeviceIDRemote(), IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_SET_SEGMENT_OK);
		return m_result;
	}

	if (count == (sizeof(unsigned short) + sizeof(unsigned long))) {

		if (*(unsigned short*)m_data != art) {

			Trace("%s:(%i):ERROR(1): CChannel4ModuleSystem::BeginGetSegment.Pending [ Art ist (%i) soll (%i) ]\n", __FILE__, __LINE__, *(unsigned short*)m_data, art);

		}
		
		// CRC uebernehmen
		crc = *(unsigned long*)&m_data[sizeof(unsigned short)];

	} else {

		Trace("%s:(%i):ERROR(1): CChannel4ModuleSystem::BeginGetSegment.Pending [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
	}
	
	return m_result;
}

int CChannel4ModuleSystem::CallBeginGetSegment(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallBeginGetSegment [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	return IV_ERROR_SUCCESS;
}

int CChannel4ModuleSystem::CallBeginGetSegmentOK(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallBeginGetSegmentOK [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::EndGetSegment(short deviceIDRemote, unsigned short art)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::EndGetSegment [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, art);

	int count = sizeof(unsigned short);

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Meberpuffer aufbereiten
	*(unsigned short*)m_data = art;

	// Beendigung einer Uebertragung eines Datanblocks
	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_END_GET_SEGMENT, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_END_GET_SEGMENT);
		return m_result;
	}

	// OK
	if (art != *(unsigned short*)m_data) {
		Trace("%s:(%i):ERROR: BeginSetSegment [ Art ist (%i) soll (%i) ]\n", __FILE__, __LINE__, *(unsigned short*)m_data, art);
	}
	
	// Bei (den einzelnen innenliegenden) Systemstopps,
	// werden die erfolgreich berechneten CRC - Pruefungen zurueckgemeldet.
/*
	short module, function, error = IV_ERROR_SUCCESS;
	// nachdem der CRC geprueft wurde, kommt ein Bestaetigungspaket,
	// dieses musz hier seperat abgegriffen werden.
	if ((m_result = Pending(deviceIDRemote, module, function, error, m_data, count)) != IV_ERROR_SUCCESS) {
		
		Trace("%s:(%i):ERROR: EndGetSegment.Pending [ Quittierungsprfantwort - CRC schlug fehl (%i) ]\n", __FILE__, __LINE__, m_result);
		return m_result;
	}
	
	if ((module == IV_MODULE_SYSTEM) && (function == IV_FUNCTION_END_GET_SEGMENT_OK)) {
		
		Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::EndGetSegment.Pending [ Antwort von CRC OK ]\n", __FILE__, __LINE__);
	}
	
	// evntl. aktiven Fehler auswerten
	if (error != IV_ERROR_SUCCESS) {

		m_result = error;
		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_END_GET_SEGMENT_OK);
		return m_result;
	}
*/
	if ((m_result = Execute(IV_CONTEXT_PENDING, IV_MODULE_SYSTEM, IV_FUNCTION_END_GET_SEGMENT_OK, m_data, count)) != IV_ERROR_SUCCESS ) {

		// Fehler ausgeben
		TraceError(GetDeviceIDRemote(), IV_MODULE_SYSTEM, IV_FUNCTION_END_GET_SEGMENT_OK);
		return m_result;
	}

	if (*(unsigned short*)m_data != art) {
		
		Trace("%s:(%i):ERROR: EndGetSegment.Pending [ Art ist (%i) soll (%i) ]\n", __FILE__, __LINE__, *(unsigned short*)m_data, art);
	}

	if (count != sizeof(unsigned short)) {
	
		Trace("%s:(%i):ERROR: EndGetSegment.Pending [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
	}
	
	return m_result;
}

int CChannel4ModuleSystem::CallEndGetSegment(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallEndGetSegment [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	return IV_ERROR_SUCCESS;
}

int CChannel4ModuleSystem::CallEndGetSegmentOK(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallEndGetSegmentOK [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	return IV_ERROR_SUCCESS;
}


#define IV_FLASH_SIZE_DATA      512

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::SetFlash(short deviceIDRemote, unsigned long address, unsigned char *buf, int count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::SetFlash [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, deviceIDRemote, address, buf, count);

	int addressOffset;
	int i, sizeData;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Gesamten Datenblock aufteilen
	for (addressOffset=0; addressOffset < count; addressOffset += IV_FLASH_SIZE_DATA) {

		// Datengroesze 
		sizeData = ((addressOffset + IV_FLASH_SIZE_DATA) > count) ? count - addressOffset : IV_FLASH_SIZE_DATA;

		// Memberpuffer aufbereiten
		*(unsigned long*)m_data = address + addressOffset;
		// Zu setzende Daten uebernehmen
		for (i=0; i < sizeData; i++) {

			m_data[sizeof(unsigned long) + i] = buf[addressOffset + i];
		}

		// Variable aufbereiten
		sizeData += sizeof(unsigned long);

		if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_SET_FLASH, m_data, sizeData)) != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_SET_FLASH);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (sizeData) {

			Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::SetFlash [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, sizeData);
		}
		
		//Trace("%s:(%i):Address: 0x%08X, wrote: 0x%04X Byte\n", __FILE__, __LINE__, *(unsigned long*)m_data, *(unsigned short*)&m_data[sizeof(unsigned long)]);
		/*if (m_callBack1) {
			if (!m_callBack1()) {
				
				m_result = -IV_ERROR_BREAK;
				// Fehlernummern setzen
				m_errorCommunicationNum = IV_ERROR_BREAK;
				// Fehlertext ermitteln und eintragen.
				GetErrorCommunication(m_errorCommunicationNum, m_errorCommunication);
				
				Trace("%s:(%i):ERROR(1): CChannel4ModuleSystem::SetFlash ( %04X )[%s]\n", __FILE__, __LINE__, -m_result & 0xFFFF, m_errorCommunication);
			}
		} else*/ if (m_callBack2) {
			if (!m_callBack2(count, addressOffset + sizeData)) {
				
				m_result = -IV_ERROR_BREAK;
				// Fehlernummern setzen
				m_errorCommunicationNum = IV_ERROR_BREAK;
				// Fehlertext ermitteln und eintragen.
				GetErrorCommunication(m_errorCommunicationNum, m_errorCommunication);
				
				Trace("%s:(%i):ERROR(1): CChannel4ModuleSystem::SetFlash ( %04X )[%s]\n", __FILE__, __LINE__, -m_result & 0xFFFF, m_errorCommunication);
				break;
			}
		}
	}

	return m_result;
}

int CChannel4ModuleSystem::CallSetFlash(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallSetFlash [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	count = 0;

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::GetFlash(short deviceIDRemote, unsigned long address, unsigned char *buf, int count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::GetFlash [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, deviceIDRemote, address, buf, count);

	int addressOffset;
	int i, sizeData;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Gesamten Datenblock aufteilen
	for (addressOffset=0; addressOffset < count; addressOffset += IV_FLASH_SIZE_DATA) {

		// Datengroesze 
		sizeData = ((addressOffset + IV_FLASH_SIZE_DATA) > count) ? count - addressOffset : IV_FLASH_SIZE_DATA;

		// Meberpuffer aufbereiten
		*(unsigned long*)m_data = address + addressOffset;
		// Angeforderte Laenge eintragen
		*(unsigned short*)&m_data[sizeof(unsigned long)] = sizeData;

		// Variable aufbereiten
		sizeData = sizeof(unsigned long) + sizeof(unsigned short);

		if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH, m_data, sizeData)) != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH);
			return m_result;
		}

		// OK
		
		// Empfangene Daten uebernehmen
		for (i=0; i < (sizeData - (signed)sizeof(unsigned long)); i++) {
			
			buf[addressOffset + i] = m_data[sizeof(unsigned long) + i];
		}
		
	}

	return m_result;
}

int CChannel4ModuleSystem::CallGetFlash(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallGetFlash [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	return IV_ERROR_SUCCESS;
}

#define IV_EEPROM_SIZE_DATA   256

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::SetEEPROM(short deviceIDRemote, unsigned long address, unsigned char *buf, int count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::SetEEPROM [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, deviceIDRemote, address, buf, count);

	int addressOffset;
	int i, sizeData;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Gesamten Datenblock aufteilen
	for (addressOffset=0; addressOffset < count; addressOffset += IV_EEPROM_SIZE_DATA) {

		// Datengroesze 
		sizeData = ((addressOffset + IV_EEPROM_SIZE_DATA) > count) ? count - addressOffset : IV_EEPROM_SIZE_DATA;

		// Memberpuffer aufbereiten
		*(unsigned long*)m_data = address + addressOffset;
		// Zu setzende Daten uebernehmen
		for (i=0; i < sizeData; i++) {

			m_data[sizeof(unsigned long) + i] = buf[addressOffset + i];
		}

		// Variable aufbereiten
		sizeData += sizeof(unsigned long);

		if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_SET_EEPROM, m_data, sizeData)) != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_SET_EEPROM);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (sizeData) {

			Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::SetEEPROM [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, sizeData);
		}
		
		//Trace("%s:(%i):Address: 0x%08X, wrote: 0x%04X Byte\n", __FILE__, __LINE__, *(unsigned long*)m_data, *(unsigned short*)&m_data[sizeof(unsigned long)]);
		/*if (m_callBack1) {
			if (!m_callBack1()) {
				
				// Fehlernummern setzen
				m_errorCommunicationNum = IV_ERROR_BREAK;
				// Fehlertext ermitteln und eintragen.
				GetErrorCommunication(m_errorCommunicationNum, m_errorCommunication);
				
				Trace("%s:(%i):ERROR(1): CChannel4ModuleSystem::SetEEPROM ( %04X )[%s]\n", __FILE__, __LINE__, -m_result & 0xFFFF, m_errorCommunication);
			}
		} else*/ if (m_callBack2) {
			if (!m_callBack2(count, addressOffset + sizeData)) {
				
				// Fehlernummern setzen
				m_errorCommunicationNum = IV_ERROR_BREAK;
				// Fehlertext ermitteln und eintragen.
				GetErrorCommunication(m_errorCommunicationNum, m_errorCommunication);
				
				Trace("%s:(%i):ERROR(1): CChannel4ModuleSystem::SetEEPROM ( %04X )[%s]\n", __FILE__, __LINE__, -m_result & 0xFFFF, m_errorCommunication);
			}
		}
	}

	return m_result;
}

int CChannel4ModuleSystem::CallSetEEPROM(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallSetEEPROM [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::GetEEPROM(short deviceIDRemote, unsigned long address, unsigned char *buf, int count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::GetEEPROM [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, deviceIDRemote, address, buf, count);

	int addressOffset;
	int i, sizeData;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Gesamten Datenblock aufteilen
	for (addressOffset=0; addressOffset < count; addressOffset += IV_EEPROM_SIZE_DATA) {

		// Datengroesze 
		sizeData = ((addressOffset + IV_EEPROM_SIZE_DATA) > count) ? count - addressOffset : IV_EEPROM_SIZE_DATA;

		// Meberpuffer aufbereiten
		*(unsigned long*)m_data = address + addressOffset;
		// Angeforderte Laenge eintragen
		*(unsigned short*)&m_data[sizeof(unsigned long)] = sizeData;

		// Variable aufbereiten
		sizeData = sizeof(unsigned long) + sizeof(unsigned short);

		if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_GET_EEPROM, m_data, sizeData)) != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_GET_EEPROM);
			return m_result;
		}

		// OK
		
		// Empfangene Daten uebernehmen
		for (i=0; i < (sizeData - (signed)sizeof(unsigned long)); i++) {
			
			buf[addressOffset + i] = m_data[sizeof(unsigned long) + i];
		}
	}

	return m_result;
}

int CChannel4ModuleSystem::CallGetEEPROM(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallGetEEPROM [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::GetFlashPage(short deviceIDRemote, unsigned short pageNum, unsigned char *buf, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::GetFlashPage [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, deviceIDRemote, pageNum, buf, count);

	int i;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);

	// Meberpuffer aufbereiten
	*(unsigned short*)m_data = pageNum;
	// Offset in die Seite
	*(unsigned short*)&m_data[sizeof(unsigned short)] = 0;
	// Angeforderte Laenge eintragen
	*(unsigned short*)&m_data[sizeof(unsigned short) * 2] = 528;

	// Variable aufbereiten
	count = sizeof(unsigned short) * 3;

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH_PAGE, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH_PAGE);
		return m_result;
	}

	// OK
	if (count != 532) {

		Trace("%s:(%i):ERROR(1): CChannel4ModuleSystem::SetFlashPage [ Antwort ist ungleich 532. ]\n", __FILE__, __LINE__);
	}
	
	// Empfangene Daten uebernehmen
	for (i=0; i < (count - (signed)(sizeof(unsigned short) * 2)); i++) {
		
		buf[i] = m_data[sizeof(unsigned short) * 2 + i];
	}
	count -= sizeof(unsigned short) * 2;

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt die Firmwareversion.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param version Zeiger Variable fuer den Versionsstring.
 *                Format: XX.XX.XX.XX ?...
 */
int CChannel4ModuleSystem::GetVersion(short deviceIDRemote, char *version)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::GetVersion [ %i, 0x%08X ]\n", __FILE__, __LINE__, deviceIDRemote, version);

	int i, count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_GET_VERSION, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_GET_VERSION);
		return m_result;
	}

	// OK
	for (i=0; i < count; i++) {
		
		version[i] = m_data[i];
	}

	return m_result;
}

int CChannel4ModuleSystem::CallGetVersion(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallGetVersion [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	strcpy((char*)data, "04.01.02.DLL");
	count = strlen((char*)data) + 1;

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Ermittelt die Bootloaderversion.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param version Zeiger auf Variable fuer Versionsstring. 
 *                Format: XX.XX.XX.XX ?...
 */
int CChannel4ModuleSystem::GetVersionBootloader(short deviceIDRemote, char *version)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::GetVersionBootloader [ %i, 0x%08X ]\n", __FILE__, __LINE__, deviceIDRemote, version);

	int i, count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_GET_VERSION_BOOTLOADER, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_GET_VERSION_BOOTLOADER);
		return m_result;
	}

	// OK
	for (i=0; i < count; i++) {

		version[i] = m_data[i];
	}

	return m_result;
}

int CChannel4ModuleSystem::CallGetVersionBootloader(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallGetVersionBootloader [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	strcpy((char*)data, "04.00.01.00.DLL");
	count = strlen((char*)data) + 1;

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Ermittelt die Seriennummer des Geraets.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param serial Variable die die ermittelte Seriennummer erhaehlt.
 */
int CChannel4ModuleSystem::GetSerial(short deviceIDRemote, unsigned long &serial)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::GetSerial [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, serial);

	int count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SERIAL, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SERIAL);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count > sizeof(unsigned long)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::GetSerial [ Seriennummer mit %i Byte, laenger als erwartete 32bit ]\n", __FILE__, __LINE__, count - 4);
	}

	serial = *(unsigned long*)m_data;

	return m_result;
}

int CChannel4ModuleSystem::CallGetSerial(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallGetSerial [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	// Seriennummer uebernehmen
	*(unsigned long*)data = 12345;
	count = sizeof(unsigned long);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Setzten der RTC (RealTimeClock) im Geraet.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param time Zeitstring im Format: "JJJJ-MM-DD hh:mm:ss"
 */
int CChannel4ModuleSystem::SetRTC(short deviceIDRemote, char *time)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::SetRTC [ %i, 0x%08X ]\n", __FILE__, __LINE__, deviceIDRemote, time);

	int i, count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	count = 20;
	// Puffer loeschen
	memset(m_data, 0, sizeof(m_data));
	for (i=0; i < (count - 1); i++) {

		m_data[i] = time[i];
	}

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_SET_RTC, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_SET_RTC);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::SetRTC [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	return m_result;
}

int CChannel4ModuleSystem::CallSetRTC(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallSetRTC [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	Trace("CallSetRTC( %s )\n", data);
	count = 0;

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Lesen der RTC (RealTimeClock) aus Geraet.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param time Zeiger auf Zeichenkette fuer Zeitstring. 
 *             Format: "JJJJ-MM-DD hh:mm:ss"
 */
int CChannel4ModuleSystem::GetRTC(short deviceIDRemote, char *time)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::GetRTC [ %i, 0x%08X ]\n", __FILE__, __LINE__, deviceIDRemote, time);

	int i, count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_GET_RTC, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_GET_RTC);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count > 20) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::GetRTC [ Uhrzeitstring ist mit %i Byte laenger als erwartet. ]\n", __FILE__, __LINE__, count - 20);
	}

	// OK
	for (i=0; i < count; i++) {

		time[i] = m_data[i];
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Lesen der RTC (RealTimeClock) aus Geraet.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param time Zeiger auf Zeichenkette fuer Zeitstring. 
 *             Format: "JJJJ-MM-DD hh:mm:ss"
 */
int CChannel4ModuleSystem::CallGetRTC(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallGetRTC [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	strcpy((char*)data, "2006-11-01 12:13:14");
	count = strlen((char*)data) + 1;

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::SetSystemVariable(short deviceIDRemote, char *name, char *value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::SetSystemVariable [ %i, 0x%08X, 0x%08X ]\n", __FILE__, __LINE__, deviceIDRemote, name, value);

	int i, j, count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Puffer loeschen
	memset(m_data, 0, sizeof(m_data));
	// Namen uebernehmen
	count = strlen(name) + 1;
	for (i=0; i < count; i++) {

		m_data[i] = name[i];
	}
	// Wert uebernehmen
	count += strlen(value) + 1;
	for (j=i; i < count; i++) {

		m_data[i] = value[i - j];
	}

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_SET_SYSTEM_VARIABLE, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_SET_SYSTEM_VARIABLE);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::SetSystemVariable [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	return m_result;
}

int CChannel4ModuleSystem::CallSetSystemVariable(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallSetSystemVariable [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	Trace("CallSetSystemVariable( %s = %s )\n", data, &data[strlen((char*)data) + 1]);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::GetSystemVariable(short deviceIDRemote, char *name, char *value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::GetSystemVariable [ %i, 0x%08X, 0x%08X ]\n", __FILE__, __LINE__, deviceIDRemote, name, value);

	int i, count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Puffer loeschen
	memset(m_data, 0, sizeof(m_data));
	// Namen uebernehmen
	count = strlen(name) + 1;
	for (i=0; i < count; i++) {

		m_data[i] = name[i];
	}

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SYSTEM_VARIABLE, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SYSTEM_VARIABLE);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count > 80) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::GetSystemVariable [ Laenge der Variablen mit %i Byte ist fragwuerdig ]\n", __FILE__, __LINE__, count);
	}

	// Namen uebergehen
	count = strlen(name) + 1;

	// Wert uebernehmen
	strcpy(value, (char*)&m_data[count]);

	return m_result;
}

int CChannel4ModuleSystem::CallGetSystemVariable(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallGetSystemVariable [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	Trace("CallGetSystemVariable( %s = %s )\n", data, &data[strlen((char*)data) + 1]);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::SetRelay(short deviceIDRemote, char *name, unsigned char busNum, unsigned char relayNum, unsigned char status, unsigned short time)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::SetRelay [ %i, 0x%08X, %i, %i, %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, name, busNum, relayNum, status, time);

	int count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Parameter uebernehmen
	count = sprintf((char*)m_data, "%s", name) + 1;

	// Weitere Parameter als String uebergehen
	count += sprintf((char*)&m_data[count], "%i", busNum) + 1;
	count += sprintf((char*)&m_data[count], "%i", relayNum) + 1;
	count += sprintf((char*)&m_data[count], "%i", status) + 1;
	count += sprintf((char*)&m_data[count], "%i", time) + 1;
/*
	// Busnummer
	m_data[count] = busNum;
	count += sizeof(unsigned char);

	// Relaisnummer
	m_data[count] = relayNum;
	count += sizeof(unsigned char);

	// Status
	m_data[count] = status;
	count += sizeof(unsigned char);

	// Time
	*(unsigned short*)&m_data[count] = status;
	count += sizeof(unsigned short);
*/

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_SET_RELAY, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_SET_RELAY);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::SetRelay [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	return m_result;
}

int CChannel4ModuleSystem::CallSetRelay(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallSetRelay [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	char *name, *busNum, *relayNum, *status, *time;
	name = (char*)data;
	busNum = name + strlen((char*)data) + 1;
	relayNum = busNum + strlen(busNum) + 1;
	status = relayNum + strlen(relayNum) + 1;
	time = status + strlen(status) + 1;

	Trace("CallSetRelay( name = %s, busNum = %s, relayNum = %s, status = %s, time = %s )\n",
		name, busNum, relayNum, status, time);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::GetRelay(short deviceIDRemote, char *name, unsigned char busNum, unsigned char relayNum, unsigned char &status, unsigned short &time)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::GetRelay [ %i, 0x%08X, %i, %i, %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, name, busNum, relayNum, status, time);

	int count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Namen uebernehmen
	count = sprintf((char*)m_data, "%s", name) + 1;

	// Weitere Parameter als String uebernehmen
	count += sprintf((char*)&m_data[count], "%i", busNum) + 1;
	count += sprintf((char*)&m_data[count], "%i", relayNum) + 1;
/*
	// Busnummer
	m_data[count] = busNum;
	count += sizeof(unsigned char);

	// Relaisnummer
	m_data[count] = relayNum;
	count += sizeof(unsigned char);
*/	
	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_GET_RELAY, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_GET_RELAY);
		return m_result;
	}

	// Name
	count = strlen((char*)m_data) + 1;

	// Weitere Parameter als String uebergehen
	// busNum, relayNum
	count += strlen((char*)&m_data[count]) + 1;
	count += strlen((char*)&m_data[count]) + 1;

	// Status uebernehmen
	status = atoi((char*)&m_data[count]);
	count += strlen((char*)&m_data[count]) + 1;

	// Zeit uebernehmen
	time = atoi((char*)&m_data[count]);
/*
	// Keine Antwortdaten erwartet
	if (count != (sizeof(unsigned char) + sizeof(unsigned short))) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::GetRelay [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
	}

	// Status
	status = m_data[0];
	// Time
	time = *(unsigned short*)&m_data[sizeof(unsigned char)];
*/

	return m_result;
}

int CChannel4ModuleSystem::CallGetRelay(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::GetRelay [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	// Parameter uebernehmen
	count = sprintf((char*)data, "%s", data) + 1;

	// Weitere Parameter als String uebergehen
	count += sprintf((char*)&data[count], "%i", 0) + 1;
	count += sprintf((char*)&data[count], "%i", 0) + 1;
	count += sprintf((char*)&data[count], "%i", 0) + 1;
	count += sprintf((char*)&data[count], "%i", 0) + 1;

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::EnterBios(short deviceIDRemote)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::EnterBios [ %i ]\n", __FILE__, __LINE__, deviceIDRemote);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_BIOS, m_data, count, 1)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_BIOS);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::EnterBios [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}
/*	
	// Nach dem Wechsel ins Bios wird eine Bestaetigung gesendet.
	short module, function, error = IV_ERROR_SUCCESS;
	if ((m_result = Pending(deviceIDRemote, module, function, error, m_data, count)) != IV_ERROR_SUCCESS) {
		
		Trace("%s:(%i):Quittierungsantwort - schlug fehl.\n", __FILE__, __LINE__);
		return m_result;
	}
	
	if ((module == IV_MODULE_SYSTEM) && (function == IV_FUNCTION_ENTERED_BIOS)) {
		
		Trace("%s:(%i):Antwort OK.\n", __FILE__, __LINE__);
	}
	
	// evntl. aktiven Fehler auswerten
	if (error != IV_ERROR_SUCCESS) {

		m_result = error;
		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_ENTERED_BIOS);
		return m_result;
	}
*/
	if ((m_result = Execute(IV_CONTEXT_PENDING, IV_MODULE_SYSTEM, IV_FUNCTION_ENTERED_BIOS, m_data, count)) != IV_ERROR_SUCCESS ) {

		// Fehler ausgeben
		TraceError(GetDeviceIDRemote(), IV_MODULE_SYSTEM, IV_FUNCTION_ENTERED_BIOS);
		return m_result;
	}

	return m_result;
}

int CChannel4ModuleSystem::CallEnterBios(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallEnterBios [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::ExitBios(short deviceIDRemote)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::ExitBios [ %i ]\n", __FILE__, __LINE__, deviceIDRemote);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Beendigung einer Uebertragung eines Datanblocks
	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_BIOS, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_BIOS);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::ExitBios [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

/*	
	// Nach dem Verlassen des Bios kommt eine Bestaetigungsanfrage.
	short module, function, error = IV_ERROR_SUCCESS;
	if ((m_result = Pending(deviceIDRemote, module, function, error, m_data, count)) != IV_ERROR_SUCCESS) {
		
		Trace("%s:(%i):ERROR(1): CChannel4ModuleSystem::ExitBios [ Quittierungsantwort - schlug fehl (%i) ]\n", __FILE__, __LINE__, m_result);
		return m_result;
	}
	
	if ((module == IV_MODULE_SYSTEM) && (function == IV_FUNCTION_EXITED_BIOS)) {
		
		Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::ExitBios [ Antwort OK ]\n", __FILE__, __LINE__);
	}
	
	// evntl. aktiven Fehler auswerten
	if (error != IV_ERROR_SUCCESS) {

		m_result = error;
		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_EXITED_BIOS);
		return m_result;
	}
*/
	if ((m_result = Execute(IV_CONTEXT_PENDING, IV_MODULE_SYSTEM, IV_FUNCTION_EXITED_BIOS, m_data, count)) != IV_ERROR_SUCCESS ) {

		// Fehler ausgeben
		TraceError(GetDeviceIDRemote(), IV_MODULE_SYSTEM, IV_FUNCTION_EXITED_BIOS);
		return m_result;
	}

	return m_result;
}

int CChannel4ModuleSystem::CallExitBios(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallExitBios [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::SetInterface(short deviceIDRemote, unsigned char type)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::SetInterface [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, type);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Den Geraetetyp uebernehmen.
	m_data[0] = type;
	count += sizeof(unsigned char);

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_SET_INTERFACE, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_SET_INTERFACE);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::SetInterface [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	return m_result;
}

int CChannel4ModuleSystem::CallSetInterface(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallSetInterface [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::GetInterface(short deviceIDRemote, unsigned char &type)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::GetInterface [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, type);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_GET_INTERFACE, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_GET_INTERFACE);
		return m_result;
	}

	// OK
	if (count != sizeof(unsigned char)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::GetInterface [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
	}

	type = (m_data[0] != 0xFF) ? m_data[0] : 0;

	return m_result;
}

int CChannel4ModuleSystem::CallGetInterface(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallGetInterface [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);
/*
#define IV_VALUE_INTERFACE_RS232 0
#define IV_VALUE_INTERFACE_TCPIP 1
#define IV_VALUE_INTERFACE_GSM   2
#define IV_VALUE_INTERFACE_GPRS  3
#define IV_VALUE_INTERFACE_RS485 4
*/

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::SetBaud(short deviceIDRemote, unsigned long br)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::SetBaud [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, br);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Den Geraetetyp uebernehmen.
	*(unsigned long*)&m_data[0] = br;
	count += sizeof(unsigned long);

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_SET_BAUD, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_SET_BAUD);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::SetBaud [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	return m_result;
}

int CChannel4ModuleSystem::CallSetBaud(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallSetBaud [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::GetBaud(short deviceIDRemote, unsigned long &br)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::GetBaud [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, br);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_GET_BAUD, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_GET_BAUD);
		return m_result;
	}

	// OK
	if (count != sizeof(unsigned long)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::GetBaud [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
	}

	br = *(unsigned long*)m_data;
	switch(br) { // Bereich pruefen
	case 9600:
	case 14400:
	case 19200:
	case 38400:
	case 57600:
	case 115200:
		break;
	default:
		br = 38400;
	}

	return m_result;
}

int CChannel4ModuleSystem::CallGetBaud(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallGetBaud [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::SetDeviceIDBios(short deviceIDRemote, unsigned short id)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::SetDeviceIDBios [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, id);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Den Geraetetyp uebernehmen.
	*(unsigned short*)&m_data[0] = id;
	count += sizeof(unsigned short);

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_SET_DEVICE_ID, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_SET_DEVICE_ID);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::SetDeviceIDBios [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	return m_result;
}

int CChannel4ModuleSystem::CallSetDeviceIDBios(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallSetDeviceIDBios [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	return IV_ERROR_SUCCESS;
}


// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::GetDeviceIDBios(short deviceIDRemote, unsigned short &id)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::GetDeviceIDBios [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, id);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_GET_DEVICE_ID, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_GET_DEVICE_ID);
		return m_result;
	}

	// OK
	if (count != sizeof(unsigned short)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::GetDeviceIDBios [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
	}

	id = *(unsigned short*)m_data;
	if (!((id >= 1 && id <= 31) && id != 254)) {

		id = 254;
	}

	return m_result;
}

int CChannel4ModuleSystem::CallGetDeviceIDBios(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallGetDeviceIDBios [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::EnterBootloader(short deviceIDRemote)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::EnterBootloader [ %i ]\n", __FILE__, __LINE__, deviceIDRemote);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_BOOTLOADER, m_data, count, 1)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_BOOTLOADER);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::EnterBootloader [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	return m_result;
}

int CChannel4ModuleSystem::CallEnterBootloader(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallEnterBootloader [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::EnterXPortMonitor(short deviceIDRemote, char *data)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::EnterXPortMonitor [ %i, 0x%08X ]\n", __FILE__, __LINE__, deviceIDRemote, data);

	int count = 0, i;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::EnterXPortMonitor [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}
	
	// Bei (den einzelnen innenliegenden) Systemstopps,
	// werden die erfolgreich berechneten CRC - Pruefungen zurueckgemeldet.
/*
	short module, function, error = IV_ERROR_SUCCESS;
	// nachdem der CRC geprueft wurde, kommt ein Bestaetigungspaket,
	// dieses musz hier seperat abgegriffen werden.
	if ((m_result = Pending(deviceIDRemote, module, function, error, m_data, count)) != IV_ERROR_SUCCESS) {
		
		Trace("%s:(%i):ERROR(1): CChannel4ModuleSystem::EnterXPortMonitor [ Quittierungsprfantwort - CRC schlug fehl ]\n", __FILE__, __LINE__);
		return m_result;
	}
	
	if ((module == IV_MODULE_SYSTEM) && (function == IV_FUNCTION_ENTER_XPORT_MONITOR_OK)) {
		
		Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::EnterXPortMonitor [ Antwort von CRC OK ]\n", __FILE__, __LINE__);
	}
	
	// evntl. aktiven Fehler auswerten
	if (error != IV_ERROR_SUCCESS) {

		m_result = error;
		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR_OK);
		return m_result;
	}
*/
	if ((m_result = Execute(IV_CONTEXT_PENDING, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR_OK, m_data, count)) != IV_ERROR_SUCCESS ) {

		// Fehler ausgeben
		TraceError(GetDeviceIDRemote(), IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR_OK);
		return m_result;
	}

	// Daten uebernehmen
	for (i=0; i < count; i++) {
		
		data[i] = m_data[i];
	}
	
	return m_result;
}

int CChannel4ModuleSystem::CallEnterXPortMonitor(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallEnterXPortMonitor [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::CommandXPort(short deviceIDRemote, char *buf, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CommandXPort [ %i, 0x%08X, %i ]\n", __FILE__, __LINE__, deviceIDRemote, buf, count);

	int i;
	
	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Daten fuer senden uebernehmen
	for (i=0; i < count; i++) {

		m_data[i] = buf[i];
	}

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_CMD_XPORT, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_CMD_XPORT);
		return m_result;
	}

	// OK

	// Daten von XPort uebernehmen
	for (i=0; i < count; i++) {

		buf[i] = m_data[i];
	}

	return m_result;
}

int CChannel4ModuleSystem::CallCommandXPort(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallCommandXPort [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::ExitXPortMonitor(short deviceIDRemote)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::ExitXPortMonitor [ %i ]\n", __FILE__, __LINE__, deviceIDRemote);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Beendigung einer Uebertragung eines Datanblocks
	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_XPORT_MONITOR, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_XPORT_MONITOR);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::ExitXPortMonitor [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}
	
	// Bei (den einzelnen innenliegenden) Systemstopps,
	// werden die erfolgreich berechneten CRC - Pruefungen zurueckgemeldet.
	//		if (art != SS_UPDATE) {
/*
	short module, function, error = IV_ERROR_SUCCESS;
	// nachdem der CRC geprueft wurde, kommt ein Bestaetigungspaket,
	// dieses musz hier seperat abgegriffen werden.
	if ((m_result = Pending(deviceIDRemote, module, function, error, m_data, count)) != IV_ERROR_SUCCESS) {
		
		Trace("%s:(%i):ERROR(1): CChannel4ModuleSystem::ExitXPortMonitor [ Quittierungsprfantwort - CRC schlug fehl (%i) ]\n", __FILE__, __LINE__, m_result);
		return m_result;
	}
	
	if ((module == IV_MODULE_SYSTEM) && (function == IV_FUNCTION_EXIT_XPORT_MONITOR_OK)) {
		
		Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::ExitXPortMonitor [ Antwort von CRC OK ]\n", __FILE__, __LINE__);
	}
	
	// evntl. aktiven Fehler auswerten
	if (error != IV_ERROR_SUCCESS) {

		m_result = error;
		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_XPORT_MONITOR_OK);
		return m_result;
	}
*/
	if ((m_result = Execute(IV_CONTEXT_PENDING, IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_XPORT_MONITOR_OK, m_data, count)) != IV_ERROR_SUCCESS ) {

		// Fehler ausgeben
		TraceError(GetDeviceIDRemote(), IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_XPORT_MONITOR_OK);
		return m_result;
	}

	return m_result;
}

int CChannel4ModuleSystem::CallExitXPortMonitor(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallExitXPortMonitor [ %i ]\n", __FILE__, __LINE__, data, count);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::OpenComServerMode(short deviceIDRemote, char *comm, unsigned long baudRate, unsigned char byteSize, unsigned char parity, unsigned char stopBits, unsigned short flags, unsigned short timeoutToClose)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::OpenComServerMode [ %i, %s, %i, %i, %i, %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, comm, baudRate, byteSize, parity, stopBits, timeoutToClose);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);

	// Alle Zusatzparameter uebernehmen
	count += sprintf((char*)&m_data[count], "%s", comm) + 1;
	*(unsigned long*)&m_data[count] = baudRate;
	count += sizeof(unsigned long);
	m_data[count] = byteSize;
	count += sizeof(unsigned char);
	m_data[count] = parity;
	count += sizeof(unsigned char);
	m_data[count] = stopBits;
	count += sizeof(unsigned char);
	*(unsigned short*)&m_data[count] = flags;
	count += sizeof(unsigned short);
	*(unsigned short*)&m_data[count] = timeoutToClose;
	count += sizeof(unsigned short);

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_OPEN_COM_SERVER_MODE, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(GetDeviceIDRemote(), IV_MODULE_SYSTEM, IV_FUNCTION_OPEN_COM_SERVER_MODE);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::OpenComServerMode [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	// Aktive Antwort abwarten
	if ((m_result = Execute(IV_CONTEXT_PENDING, IV_MODULE_SYSTEM, IV_FUNCTION_OPEN_COM_SERVER_MODE_OK, m_data, count)) != IV_ERROR_SUCCESS ) {

		// Fehler ausgeben
		TraceError(GetDeviceIDRemote(), IV_MODULE_SYSTEM, IV_FUNCTION_OPEN_COM_SERVER_MODE_OK);
		return m_result;
	}

	return m_result;
}

int CChannel4ModuleSystem::CallOpenComServerMode(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallOpenComServerMode [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	return IV_ERROR_SUCCESS;
}

int CChannel4ModuleSystem::CallOpenComServerModeOK(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallOpenComServerModeOK [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleSystem::CloseComServerMode(short deviceIDRemote)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CloseComServerMode [ %i ]\n", __FILE__, __LINE__, deviceIDRemote);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_CLOSE_COM_SERVER_MODE, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_CLOSE_COM_SERVER_MODE);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::CloseComServerMode [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	return m_result;
}

int CChannel4ModuleSystem::CallCloseComServerMode(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::CallCloseComServerMode [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Ermittelt die Information zu einem Modultyp
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param name Das anzusprechende Modul. (ACCESS.MODULE fuer Zutrittskontrollmodul)
 * @param param Abhaengig vom verwendeten Namen. (Bei ACCESS.MODULE gibt es die TM-Nummer des Moduls )
 * @param info Zeiger auf Zeichenkettenpuffer fuer Rueckgabe.
 * @param infoLength Laenge des Zeichenkettenpuffers fuer Rueckgabe
 */
int CChannel4ModuleSystem::GetInfo(short deviceIDRemote, char *name, char *param, char *info, int &infoLength)

{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::GetInfo [ %i, %s, %s, %s, %i ]\n", __FILE__, __LINE__, deviceIDRemote, name, param, info, infoLength);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);

	count = sprintf((char*)m_data, name) + 1;
	count += sprintf((char*)&m_data[count], param) + 1;

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_GET_INFO, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_GET_INFO);
		return m_result;
	}


	// Antwortdaten erwartet
	if ((unsigned)count < (strlen(name) + 1 + strlen(param) + 1)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::GetInfo [ Antwort ist zu kurz. ]\n", __FILE__, __LINE__);
	}

	int i, offs;
	// Name uebergehen
	offs = strlen((char*)m_data) + 1;
	// Parameter uebergehen
	offs += strlen((char*)&m_data[offs]) + 1;

	// Wert uebernehmen
	for (i=0; (i < (count - offs)) && (i < infoLength); i++) {
	
		info[i] = m_data[offs + i];
	}
	// Uebergebene Laenge mitteilen
	infoLength = i - 1;

	return m_result;
}

// ##################################################################
/**
 * @brief Aktivieren oder Deaktivieren einer Benachrichtigung.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param type Benachrichtigungstyp
 * @param state Zu setzender Zustand.
 */
int CChannel4ModuleSystem::SetNotify(short deviceIDRemote, unsigned char type, unsigned char state)

{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::SetNotify [ %i, %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, type, state);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);

	m_data[count] = type;
	count += sizeof(unsigned char);
	m_data[count] = state;
	count += sizeof(unsigned char);

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_SET_NOTIFY, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_SET_NOTIFY);
		return m_result;
	}


	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::SetNotify [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	return m_result;
}


// ##################################################################
/**
 * @brief Aktivieren oder Deaktivieren einer Benachrichtigung.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param type Benachrichtigungstyp
 * @param state Aktuell gesetzter Zustand.
 */
int CChannel4ModuleSystem::GetNotify(short deviceIDRemote, unsigned char type, unsigned char &state)

{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::GetNotify [ %i, %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, type, state);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);

	m_data[count] = type;
	count += sizeof(unsigned char);

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_GET_NOTIFY, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_GET_NOTIFY);
		return m_result;
	}


	// Antwortdaten erwartet
	if (count != sizeof(unsigned char)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleSystem::GetNotify [ Antwort ist zu lang oder kurz. ]\n", __FILE__, __LINE__);
	} else {

		// Gemeldeten Zustand uebernehmen
		state = m_data[0];
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Abfragen der Softwareinfo der eingespielten Anwendungsfirmware.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param swi Puffer fr die Softwareinfo.
 * @param infoLength Lnge des Puffers und bergebene Anzahl daten.
 */
int CChannel4ModuleSystem::GetSoftwareInfo(short deviceIDRemote, unsigned char *swi, int &infoLength)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleSystem::GetSoftwareInfo [ %i, 0x%08X, %i ]\n", __FILE__, __LINE__, deviceIDRemote, swi, infoLength);

	int i, count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SOFTWARE_INFO, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SOFTWARE_INFO);
		return m_result;
	}

	// Antwortdaten erwartet
	for (i=0; i < count && i < infoLength; i++) {

		swi[i] = m_data[i];
	}
	infoLength = i;

	return m_result;
}


#ifndef DATAFOX_STUDIO_IV

// Funktions-Stumps

// MODULE_HARDWARE
int CChannel4ModuleSystem::SetTypeDisplay(short deviceIDRemote, unsigned char value, int checked)     { return -IV_ERROR_FUNCTION_UNAVAILABLE; }
int CChannel4ModuleSystem::SetTypeKeypad(short deviceIDRemote, unsigned char value, int checked)      { return -IV_ERROR_FUNCTION_UNAVAILABLE; }
int CChannel4ModuleSystem::SetTypeTransponder(short deviceIDRemote, unsigned char value, int checked) { return -IV_ERROR_FUNCTION_UNAVAILABLE; }

int CChannel4ModuleSystem::SetSerial(short deviceIDRemote, unsigned long serial) { return -IV_ERROR_FUNCTION_UNAVAILABLE; }

#endif
