/**
 * @file  DFFileIni.h
 * @brief Schnittstelle fuer die Klasse CDFFileIni.
 */
//////////////////////////////////////////////////////////////////////
#ifndef _DFINIFILE_H_
#define _DFINIFILE_H_

/* Verwaltung einer Initialisierungsdatei mit Sektionen [...] und 
 * Schluesseln mit Werten vom Typ "Zahl" und "Zeichenkette".
 *
 *
 */
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef _WIN32
#define stat _stat
#endif

// Maximale Laenge von Sektionsnamen
#define MAX_SECTION_NAME   64
// Maximale Laenge von Schluesselnamen
#define MAX_KEY_NAME       64
// Maximale Laenge von Aufnehmbaren Werten
#define MAX_VALUE_NAME     258
// Maximale Laenge von Aufnehmbaren Kommentarzeilen
#define MAX_COMMEND_NAME   258

class CDFFileIni
{
public:
	CDFFileIni();
	virtual ~CDFFileIni();

	// Pruefen ob die Datei vorhanden und grundlegend in Ordnung
	int ExistingFile(char *fileName);

	// Lesen eines vorzeichenlosen Integerwertes
	unsigned int GetUInt(char *section, char *key, int defaultVal, char *fileName);
	// Lesen einer Zeichenkette
	int GetString(char *section, char *key, char *value, char *fileName);
	int SetString(char *section, char *key, char *value, char *fileName);

protected:
// FindFlag
#define FF_CREATE 0x00000001 // Erzeugen wenn nicht vorhanden

	int FindSection(FILE *fp, char *section, unsigned long flags);
	int FindKey(FILE *fp, char *key, unsigned long flags, char *line, int length);
	unsigned int GetNum(FILE *fp, int defaultVal, char *line);
	int GetStr(FILE *fp, char *value, char *line);
	//unsigned int SetNum(FILE *fp, int iDefault, char *szLine);
	int SetStr(FILE *fp, char *value, char *line);

public:
// ScanFlag
#define SF_READ   0x00000001 // Werte lesen
#define SF_WRITE  0x00000002 // Werte schreiben
#define SF_CREATE 0x00000010 // Sektion sowie Schluessel erzeugen wenn noch nicht vorhanden
#define SF_DELETE 0x00000020 // Sektion sowie Schluessel loeschen
// der value - Paramter kann auf folgende Werte verweisen
#define SF_VALUE_COMMEND 0x00000100 // Wert ist eine Kommentarzeile
#define SF_VALUE_STRING  0x00000200 // Wert ist eine Zeichenkette
#define SF_VALUE_DIGIT   0x00000400 // Wert ist eine Zahl

	int Scan(char *section, char *key, char *value, unsigned long flags, char *fileName);
};

#endif

