// DatafoxLibraryIV.cpp : Definiert die exportierten Funktionen für die DLL-Anwendung.
//

#include "DatafoxLibraryIV.h"
#include "api/df_api.h"
#include "../Shared/Channel/ChannelBridge.h"


DatafoxLibraryIV *DatafoxLibraryIV::m_this = NULL;

DatafoxLibraryIV::DatafoxLibraryIV()
{
    m_this = this;

#ifdef _MSC_VER
    _hModule = NULL;
    // Initialize the critical section one time only.
    InitializeCriticalSection( &m_mutexQue ); 
    InitializeCriticalSection( &m_mutexChannelRequest );
#else
    pthread_mutex_init( &m_mutexQue, NULL );
    pthread_mutex_init( &m_mutexChannelRequest, NULL );
#endif
    // Members Initialisieren

    // Haupttask 
    m_workerThread[0] = 0;
    m_workerThread[1] = 0;

    // Sockets initialisieren
    m_listenSocket = INVALID_SOCKET;

    // Verarbeitung von Datensatzmeldungen auf "nicht gesetzt" setzen.
    m_notifyRecord = -1;

    // Signal ob das Binden eines Kanals waehrend der Laufzeit erfolgt ist.
    m_bindSignal = 0;

    // Globales Passwort fuer die verschluesselte Kommunikation
    memset( m_globalCommunicationPassword, 0, sizeof( m_globalCommunicationPassword ) );
    m_globalCommunicationPasswordSize = 0;
    m_globalSessionTimeout = 0;
}

DatafoxLibraryIV::~DatafoxLibraryIV()
{
    // Aktive Verbindung ggf. beenden.
    StopActiveConnection();
    //	EndNotify();

#ifdef _MSC_VER
    // Release resources used by the critical section object.
    DeleteCriticalSection( &m_mutexQue );
    DeleteCriticalSection( &m_mutexChannelRequest );
#else
    pthread_mutex_destroy( &m_mutexQue );
    pthread_mutex_destroy( &m_mutexChannelRequest );
#endif
}

// Das einzige DatafoxLibraryIV-Objekt

DatafoxLibraryIV theApp;

#ifdef _MSC_VER
// Lesen der Versionsinformationen aus den Ressourcen
int DatafoxLibraryIV::GetVersion( CDFSoftwarebuildVersion &version )
{
    int res;
    TCHAR szModuleName[MAX_PATH];
    DWORD dwHandle;
    DWORD dwLen;
    LPVOID lpBuffer = NULL;
    unsigned int uiLen;

    unsigned char *pucData = NULL;

    // Modulname der Anwendung
    if ( !GetModuleFileName( theApp._hModule, szModuleName, MAX_PATH ) )
        return 0;

    // Laenge der Version - Resource
    dwLen = GetFileVersionInfoSize( szModuleName, &dwHandle );
    if ( !dwLen )
        return 0;

    pucData = new unsigned char[dwLen + 100]; // unicode
    // Anwendung - Versioninfo ermitteln
    if ( !GetFileVersionInfo( szModuleName, dwHandle, dwLen, reinterpret_cast<LPVOID>( pucData ) ) )
    {
        delete pucData;
        return 0;
    }

    // Pointer auf allgemeine Infostruktur ermitteln
    if ( !VerQueryValue( reinterpret_cast<LPVOID>( pucData ), TEXT( "\\\0" ), &lpBuffer, &uiLen ) )
    {
        delete pucData;
        return 0;
    }
    // Daten zurueckgeben
    VS_FIXEDFILEINFO *pvs_ffi = (VS_FIXEDFILEINFO*)lpBuffer;

    // Ermittelte Version aus der Resource in Versionsobjekt wandeln.
    version.Reset();
    res = version.Parse( static_cast<unsigned int>( ((pvs_ffi->dwFileVersionMS & 0x00FF0000) << 8) | ((pvs_ffi->dwFileVersionMS & 0x000000FF) << 16) | ((pvs_ffi->dwFileVersionLS & 0x00FF0000) >> 8) | (pvs_ffi->dwFileVersionLS & 0x000000FF) ) );

    delete pucData;

    return res;
}
#endif

/**
* @brief Erhalt eines Kanalobjekts oder NULL bei Bereichsueberschreitung sowie Fehler.
*/
ChannelBridge *DatafoxLibraryIV::GetAEChannel( int &channelID )
{
    if ( AEChannelRequest( channelID ) )
    {
        // CallBack fuer Notify uebergeben
        m_channelArray[channelID]->setCallBackNotifyRecord( CBNotifyRecord );
        // Passwort fuer die verschluesselte Kommunikation uebergeben.
        if ( m_channelArray[channelID]->isGlobalCommunicationPassword() )
            m_channelArray[channelID]->setCommunicationPassword( m_globalCommunicationPassword, m_globalCommunicationPasswordSize, m_globalSessionTimeout, 1 );

        return m_channelArray[channelID];
    }

    return NULL;
}

//#ifdef WIN32
/**
* @brief Sicherstellen das die Anfrage und das Kanalobjekt in Ordnung ist.
*/
bool DatafoxLibraryIV::AEChannelRequest( int &channelID )
{
    // Kanalbereich pruefen und evntl. erstellen des nicht vorhandenen Kanalobjektes.
    if ( (channelID != -1) && (channelID < 0 || channelID >= MAX_CHANNEL) )
    {
        // Bereich falsch
        return false;
    }

#ifdef _MSC_VER
    EnterCriticalSection( &m_mutexChannelRequest );
#else
    pthread_mutex_lock( &m_mutexChannelRequest );
#endif

    if ( channelID == -1 )
    {
        // Freies suchen
        for ( channelID = ACTIVE_CHANNEL_LOWER_BOUND; channelID < ACTIVE_CHANNEL_UPPER_BOUND; ++channelID )
        {
            if  ( m_channelArray[channelID] == NULL )
            {
                break;
            }
            else if ( m_channelArray[channelID]->getActiveState() == is_free )
            {
#ifdef _MSC_VER
                LeaveCriticalSection( &m_mutexChannelRequest );
#else
                pthread_mutex_unlock( &m_mutexChannelRequest );
#endif
               return true;
            }
        }

        if ( channelID == ACTIVE_CHANNEL_UPPER_BOUND )
        {
            // Alle Kanalklassen sind vergeben!
            channelID = -1;
#ifdef _MSC_VER
            LeaveCriticalSection( &m_mutexChannelRequest );
#else
            pthread_mutex_unlock( &m_mutexChannelRequest );
#endif
            return false;
        }
    }
    else
    {
        // Kanalobjekt bereits vorhanden?
        if ( m_channelArray[channelID] != NULL )
        {
#ifdef _MSC_VER
            LeaveCriticalSection( &m_mutexChannelRequest );
#else
            pthread_mutex_unlock( &m_mutexChannelRequest );
#endif
            return true;
        }
    }


    ChannelBridge *c = reinterpret_cast<ChannelBridge *>( new ChannelBridge( static_cast<short>( channelID ) ) );
    if ( c == NULL )
    {
#ifdef _MSC_VER
        LeaveCriticalSection( &m_mutexChannelRequest );
#else
        pthread_mutex_unlock( &m_mutexChannelRequest );
#endif
        return false;
    }

    // Kanalobjekt erzeugen
    m_channelArray.set( channelID, c );

#ifdef _MSC_VER
    LeaveCriticalSection( &m_mutexChannelRequest );
#else
    pthread_mutex_unlock( &m_mutexChannelRequest );
#endif

    return true;
}


/**
* @brief Ermitteln an welche Kanalnummer dieser Kanal gebunden werden soll.
*/
int DatafoxLibraryIV::ShouldBoundTo( ChannelBridge *channel )
{
    ChannelBridge *checkChannel;
    int channelID;
    for ( channelID = 1; channelID < ACTIVE_CHANNEL_LOWER_BOUND; ++channelID )
    {
        checkChannel = m_channelArray[channelID];
        if ( checkChannel != NULL && checkChannel->isBind() )
        {
            // Soll der Geraetetyp + serial passen oder passt die IP.
            if ( (channel->_channel4._hwInfo.typeDevice == checkChannel->_bindInfo.typeDevice && channel->_channel4._serial == checkChannel->_bindInfo.serial)
                || (strlen( reinterpret_cast<char *>( checkChannel->_bindInfo.ip ) ) && strlen( reinterpret_cast<char *>( channel->_channel4.m_tcpipInfo.ip ) )
                && strcmp( reinterpret_cast<char *>( checkChannel->_bindInfo.ip ), reinterpret_cast<char *>( channel->_channel4.m_tcpipInfo.ip ) ) == 0 ) )
            {
                break;
            }
        }
    }

    return (channelID == ACTIVE_CHANNEL_LOWER_BOUND) ? 0 : channelID;
}

/**
* @brief Ermitteln an welche Kanalnummer dieser Kanal gebunden werden soll.
*/
bool DatafoxLibraryIV::ChannelSwitch( int channelIDFrom, int channelIDTo )
{
    // Kanalbereiche pruefen.
    if ( channelIDFrom < 0 || channelIDFrom >= MAX_CHANNEL
        || channelIDTo < 1 || channelIDTo > ACTIVE_CHANNEL_LOWER_BOUND
        || channelIDFrom == channelIDTo )
    {
        // Bereich falsch
        return false;
    }

    ChannelBridge *channelFrom = m_channelArray[channelIDFrom];
    ChannelBridge *channelTo = m_channelArray[channelIDTo];
    // Objekte sicherstellen
    if ( channelFrom == NULL )
        return false;

    // Quellkanal aus Array entkoppeln
    m_channelArray[channelIDFrom] = NULL;

    // Zielkanal ggf. schliessen
    if ( channelTo != NULL )
    {
        // Bindinfo uebernehmen
        channelFrom->_bindInfo = channelTo->_bindInfo;
        // Verbindungen Beenden
        channelTo->close();
    }

    // Kanal wechseln
    return SetAEChannel( channelIDTo, channelFrom );
}

/**
* @brief Zuweisen eines Kanalobjektes an das Array.
*/
bool DatafoxLibraryIV::SetAEChannel( int channelID, ChannelBridge *channel )
{
    // Kanalbereich pruefen.
    if ( channelID < 1 || channelID > ACTIVE_CHANNEL_LOWER_BOUND )
    {
        // Bereich falsch
        return false;
    }

    // Kanalobjekt bereits vorhanden?
    if ( m_channelArray[channelID] != NULL )
    {
        // Loeschen.
        delete m_channelArray[channelID];
        m_channelArray[channelID] = NULL;
    }

    // Kanalobjekt erzeugen
    m_channelArray[channelID] = channel;

    // Daten zuweisen
    channel->setChannelId( channelID );

    return true;
}

int DatafoxLibraryIV::StartActiveConnection( char *ip, int port, int deviceID, int timeout, int aliveTO, int maxConnections, int infoFlags, int reserved, int *piError )
{
    (void)reserved;

    // Nicht bereits aktiv?
    if ( m_workerThread[0] != 0 )
    {
        *piError = IV_ERROR_AC_STARTED;
        return 0;
    }

    strcpy( m_listenThreadData.ip, ip );
    m_listenThreadData.port = port;
    m_listenThreadData.deviceID = deviceID;
    m_listenThreadData.timeout = timeout;
    m_listenThreadData.aliveTO = aliveTO;
    m_listenThreadData.maxConnections = maxConnections;
    m_listenThreadData.infoFlags = infoFlags;

    // Hauptthread starten; Dieser besorgt die Verbindunsannahme.
    if ( m_workerThread[0] == 0 )
    {
        int i;
#ifdef _MSC_VER
        DWORD threadID;
#endif
        m_initSignal[0] = -1; // Initialisierungssignal auf undefiniert setzen
#ifdef _MSC_VER
        m_workerThread[0] = ::CreateThread( NULL, 0, ListenThread, this, 0, &threadID );
#else
        pthread_create( &m_workerThread[0], NULL, ListenThread, this );
#endif

        // Hochlauf des Workers pruefen
        for ( i = 0; (m_workerThread[0] != 0) && (m_initSignal[0] == -1) && (i < 100); ++i )
            Sleep( 100 );

        // Wurde ein Fehler ausgeloest.
        if ( m_initSignal[0] != IV_ERROR_SUCCESS )
        {
            // Grundlegender Initialiserungsfehler?
            if ( m_initSignal[0] == -1 )
                *piError = IV_ERROR_AC_LISTENER;
            else // Fehler von Workerthread 1:1 weitergeben.
                *piError = m_initSignal[0];

            // Hartes Beenden, wenn Thread noch exisitiert.
            if ( m_workerThread[0] != 0 )
            {
#ifdef _MSC_VER
                ::CloseHandle( m_workerThread[0] );
#else
                pthread_cancel( m_workerThread[0] );
#endif
                m_workerThread[0] = 0;
            }

            return 0;
        }
    }

    return (m_workerThread[0] != 0);
}

int DatafoxLibraryIV::StopActiveConnection()
{
    // Hauptthread deaktivieren, wenn aktiv
    if ( m_workerThread[0] != 0 )
    {
        // WorkerThread fuer Socket beenden.
        m_endSignal[0] = TRUE;

        // Thread umschalten
        Sleep( 10 );

        // Thread Beendigung abwarten (ListenThread stellt m_endSignal auf FALSE)
        int i;
        for ( i = 0; (i < 10) && m_endSignal[0]; ++i )
            Sleep( 1000 );

#ifdef _MSC_VER
        ::CloseHandle( m_workerThread[0] );
#else
        //pthread_cancel(m_workerThread[0]);
#endif
        m_workerThread[0] = 0;

        // Warteschlange loeschen
        if ( IsNotifyRecord() )
        {
            while ( m_queue.size() )
                m_queue.pop(); // Eintrag verwerfen
        }
#ifdef _MSC_VER
        //TRACE( "Listener end by %i\n", i );
#endif
    }

    return !m_endSignal[0];
}

int DatafoxLibraryIV::SetRecordAvailable( int enable )
{
    // Wert uebernehmen
    m_notifyRecord = (enable == 0) ? 0 : 1;
    // Warteschlange loeschen?
    if ( m_notifyRecord == 0 )
    {
        while ( m_queue.size() )
            m_queue.pop(); // Eintrag verwerfen
    }

    return TRUE;
}

int DatafoxLibraryIV::SetNotifySoftware( int type, char *ip, int port, int retry, int delay, int state )
{
    (void)ip;
    (void)port;
    (void)retry;
    (void)delay;

    switch ( type )
    {
    case IV_VALUE_NOTIFY_RECORD:
        if ( state )
        {
            if ( !StartNotify() )
                return IV_ERROR_FAIL;
        }
        else
        {
            if ( !EndNotify() )
                return IV_ERROR_FAIL;
        }
        break;

    default:
        return IV_ERROR_FAIL;
    }

    return IV_ERROR_SUCCESS;
}

int DatafoxLibraryIV::StartNotify()
{
    // Benachrichtigungsthread starten; Dieser besorgt die Nachrichtenverteilung.
    if ( m_workerThread[1] == 0 )
    {
#ifdef _MSC_VER
        DWORD threadID;
        m_workerThread[1] = ::CreateThread( NULL, 0, NotifyThread, this, 0, &threadID );
#else
        pthread_create( &m_workerThread[1], NULL, NotifyThread, this );
#endif
    }

    return (m_workerThread[1] != 0);
}

int DatafoxLibraryIV::EndNotify()
{
    // Benachrichtigungsthread deaktivieren, wenn aktiv
    if ( m_workerThread[1] != 0 )
    {
        // WorkerThread fuer Benachrichtigung beenden.
        m_endSignal[1] = TRUE;

        // Thread umschalten
        Sleep( 10 );

        // Thread Beendigung abwarten
        int i;
        for ( i = 0; (i < 100) && m_endSignal[1]; ++i )
            Sleep( 50 );

#ifdef _MSC_VER
        CloseHandle( m_workerThread[1] );
#else
        pthread_cancel( m_workerThread[1] );
#endif
        m_workerThread[1] = 0;

#ifdef _MSC_VER
        //TRACE( "Notifyer end by %i\n", i );
#endif
    }

    return !m_endSignal[1];
}

#ifdef _MSC_VER
unsigned long WINAPI DatafoxLibraryIV::ListenThread( LPVOID lpParam )
#else
void *DatafoxLibraryIV::ListenThread( void *lpParam )
#endif
{
    // Muss einen Parameter uebergeben bekommen vom Type DatafoxLibraryIV.
    if ( lpParam == NULL )
        return 0;

    // Referenz auf den uebergebenen Parameter
    DatafoxLibraryIV &own = *reinterpret_cast<DatafoxLibraryIV *>( lpParam );

#ifdef _MSC_VER
#ifndef CONST
#define CONST const
#endif
#else
#define CONST
#endif

    // Hauptschleife ausfuehren?
    int run = FALSE;

    int rc;
    int channelID;

    SOCKADDR_IN sa;
    struct timeval seltime;
    fd_set readfds;

    ChannelBridge *channelConnect = NULL;
    Channel4 *channel4Connect = NULL;

    // Locales Channel-Objekt fuer Trace usw..
    channelID = MAX_CHANNEL-1;
    Channel4 &channel = own.GetAEChannel( channelID )->_channel4;

    // Zustandsstatus fuer Listen - Socket
    own.m_listenThreadState = DOWN;

    // Timeout fuer Verbindungsanfragen. 
    //Achtung: Nicht auf 0 da sonst der Thread blockiert wuerde!
    seltime.tv_sec  = 1;
    seltime.tv_usec = 0;

    // Sockethandle reservieren
    own.m_listenSocket = socket( AF_INET, SOCK_STREAM, 0 );
    if ( own.m_listenSocket != INVALID_SOCKET ) 
    {
        memset( &sa, 0, sizeof( sa ) );

        sa.sin_family = AF_INET;
        sa.sin_port = htons( own.m_listenThreadData.port );
#ifdef _MSC_VER
        sa.sin_addr.S_un.S_addr = inet_addr( own.m_listenThreadData.ip );
#else
        sa.sin_addr.s_addr = inet_addr( own.m_listenThreadData.ip );
#endif

        // Socket mit lokalem Port verbinden.
        if ( ::bind( own.m_listenSocket, reinterpret_cast<SOCKADDR *>( &sa ), sizeof( sa ) ) == SOCKET_ERROR )
        {
#ifdef _MSC_VER
            if ( WSAGetLastError() == WSAEADDRINUSE )
            {
#else
            if ( WSAGetLastError() == EADDRINUSE )
            {
#endif
                channel.trace( "%s(%i):ERROR(1): %s [ bind fail (%i) Die gewaehlte Portnummer ist in Gebrauch (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError(), own.m_listenThreadData.port );
                // Initialisierungsfehler setzten
                own.m_initSignal[0] = IV_ERROR_AC_PORTINUSE;
            }
            else
            {
                channel.trace( "%s(%i):ERROR(1): %s [ bind fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError() );
                // Initialisierungsfehler setzten
                own.m_initSignal[0] = IV_ERROR_AC_LISTENER;
            }

#ifdef _MSC_VER
            if ( closesocket( own.m_listenSocket ) == SOCKET_ERROR )
            {
                channel.trace( "%s(%i):ERROR(1): %s [ closesocket fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError() );
                // Initialisierungsfehler setzten
                own.m_initSignal[0] = IV_ERROR_AC_LISTENER;
            }
#else
            if ( shutdown(own.m_listenSocket, 2 ) == SOCKET_ERROR )
            {
                channel.trace( "%s(%i):ERROR(1): %s [ shutdown fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError() );
                // Initialisierungsfehler setzten
                own.m_initSignal[0] = IV_ERROR_AC_LISTENER;
            }
            if ( close( own.m_listenSocket ) == SOCKET_ERROR )
            {
                channel.trace( "%s(%i):ERROR(1): %s [ close fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError() );
                // Initialisierungsfehler setzten
                own.m_initSignal[0] = IV_ERROR_AC_LISTENER;
            }

#endif
            own.m_listenSocket = INVALID_SOCKET;
        }
        else
        {
            // Mit Reset - Status beginnen
            run = TRUE;
            own.m_listenThreadState = RESET;
            // Initialisierung erfolgreich
            own.m_initSignal[0] = IV_ERROR_SUCCESS;
        }
    }
    else
    {
        channel.trace( "%s(%i):ERROR(1): %s [ socket fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError() );
        // Initialisierungsfehler setzten
        own.m_initSignal[0] = IV_ERROR_AC_LISTENER;
    }

    // Hauptschleife des Worker
    while ( run )
    {
        // Beendigungssignal abfragen
        if ( own.m_endSignal[0] )
        {
            // Listensocket herunterfahren
            if ( own.m_listenSocket != INVALID_SOCKET )
            {
                if ( own.m_listenThreadState != DOWN )
                {
                    // Anstehende Verbindungsgesuche verwerfen
                    if ( listen( own.m_listenSocket, 1 ) == SOCKET_ERROR )
                    { 
                        channel.trace("%s(%i):ERROR(1): %s [ listen fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                    }
                }
#ifdef _MSC_VER
                // Listensocket schliessen
                if ( closesocket( own.m_listenSocket ) == SOCKET_ERROR )
                {
                    channel.trace("%s(%i):ERROR(1): %s [ closesocket fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                }
#else
                if ( shutdown( own.m_listenSocket, 2 ) == SOCKET_ERROR )
                {
                    channel.trace( "%s(%i):ERROR(1): %s [ shutdown fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError() );
                }
                if ( close( own.m_listenSocket ) == SOCKET_ERROR )
                {
                    channel.trace( "%s(%i):ERROR(1): %s [ close fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError() );
                }
#endif
                own.m_listenSocket = INVALID_SOCKET;

                // Eventuelle Verbindungen beenden
                for ( int i = MAX_CHANNEL - 1; i > 0; --i )
                {
                    if ( (own.m_channelArray[i] != NULL) && (own.m_channelArray[i]->isActive()) )
                        own.m_channelArray[i]->directClose();
                }
            }

            // Zustand setzen
            own.m_listenThreadState = DOWN;

            // Hauptschleife verlassen
            run = FALSE;
            continue;
        }

        // Je nach Zustand den Listen-Socket verwalten
        switch ( own.m_listenThreadState )
        {
        case DOWN: 
            // Aufgrund eines Fehlers abgeschaltet
            break;

        case RESET: // Verwirft eventuell anstehende Verbindungsanforderungen,
            // durch (oder) Erzeugung Listensocket.
            if ( listen( own.m_listenSocket, 1 ) == SOCKET_ERROR )
            { 
                channel.trace( "%s(%i):ERROR(1): %s [ listen fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError() );

                own.m_listenThreadState = DOWN;
                channel.trace( "%s(%i):INFO(1): %s [ State change to down... ]\n", __FILE__, __LINE__, __func__ );
            } else {

                own.m_listenThreadState = LISTENING;
                channel.trace( "%s(%i):INFO(1): %s [ State change to listening... ]\n", __FILE__, __LINE__, __func__ );
            }
            break;

        case LISTENING: // Wartet auf connect. Im "listening" - Zustand.

            FD_ZERO( reinterpret_cast<fd_set FAR *>( &readfds ) );
            FD_SET( own.m_listenSocket, reinterpret_cast<fd_set FAR *>( &readfds ) );

            if ( (rc = select( FD_SETSIZE,
                reinterpret_cast<fd_set FAR *>( &readfds ), reinterpret_cast<fd_set FAR *>( 0 ), reinterpret_cast<fd_set FAR *>( 0 ),
                &seltime ) ) == SOCKET_ERROR )
            {
                channel.trace( "%s(%i):ERROR(1): %s [ select fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError() );
            }
            else if ( rc )
            {
                // Verbindungsanforderung auf Listensocket im naechsten
                // Druchlauf verarbeiten.
                own.m_listenThreadState = ACCEPTING;
                channel.trace( "%s(%i):INFO(1): %s [ State change to accepting... ]\n", __FILE__, __LINE__, __func__ );
            } 

            // Bindpruefung wenn noetig durchfuehren
            if ( own.m_bindSignal )
            {
                int i;
                int x;

                // Alle Aktiven Kanaele pruefen
                for ( i = ACTIVE_CHANNEL_LOWER_BOUND; i < ACTIVE_CHANNEL_UPPER_BOUND; ++i )
                {
                    if ( own.m_channelArray[i] != NULL && own.m_channelArray[i]->isActive() )
                    {
                        // Pruefen ob der Kanal an eine andere channelID gebunden werden soll.
                        if ( (x = own.ShouldBoundTo( own.m_channelArray[i] )) )
                        {
                            channel.trace( "%s(%i):INFO(1): %s [ Global channel switch %i to %i ]\n", __FILE__, __LINE__, __func__, own.m_channelArray[i]->_channel4.getId(), x );
                            own.ChannelSwitch( own.m_channelArray[i]->_channel4.getId(), x );
                        }
                    }
                }

                // Zuruecksetzen
                own.m_bindSignal = 0;
            }
            break;

        case ACCEPTING: { // Verbindungsanforderung verarbeiten

            // Anzahl aktiver Verbindungen ermitteln
            int i;
            int x;
            int connected = 0;
            for ( i = 0; i < ACTIVE_CHANNEL_UPPER_BOUND; ++i )
            {
                if ( (own.m_channelArray[i] != NULL) && (own.m_channelArray[i]->isActive()) )
                    connected++;
            }

            // Maximale Anzahl Verbinungen noch nicht erreicht?
            if ( connected < own.m_listenThreadData.maxConnections )
            {
                channelID = -1;
                channelConnect = own.GetAEChannel( channelID );
                if ( channelConnect )
                    channel4Connect = &channelConnect->_channel4;
                else
                    channel4Connect = NULL;
            }
            else
            {
                channelConnect = NULL;
                channel4Connect = NULL;
            }

            channel.trace( "%s(%i):INFO(1): %s [ Try connect to (%i) ]\n", __FILE__, __LINE__, __func__, channelID );

            if ( (channel4Connect == NULL) 
                || !channel4Connect->connect( &own.m_listenSocket, channelID, own.m_listenThreadData.deviceID, own.m_listenThreadData.timeout, own.m_listenThreadData.aliveTO, own.m_listenThreadData.infoFlags ) )
            {
                // Wenn alle Kanalobjekte vergeben wurden
                // oder ein Fehler bei der Erstellung auftrat, dann
                // Verbindungsanforderung verwerfen.
                // if ( closesocket( accept( own.m_listenSocket, NULL, NULL ) ) == SOCKET_ERROR )
                // {
                //      channel.Trace( "%s(%i):INFO(1): %s [ closesocket fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError() );
                // }

                own.m_listenThreadState = RESET;
                channel.trace( "%s(%i):INFO(1): %s [ State change to reset... ]\n", __FILE__, __LINE__, __func__ );
                continue;
            }

            // Zuletzt verbundener Kanal pruefen ob alle benoetigten Daten vorliegen
            for ( i = 0; (channelConnect->getActiveState() < is_usable) && (i < 100); ++i )
                Sleep( 10 );

            // Wenn der Kanal jetzt noch nicht verwendbar ist, dann schliessen
            if ( channelConnect->getActiveState() < is_usable )
            {
                channelConnect->directClose();
                channel.trace( "%s(%i):ERROR(1): %s [ not use able ]\n", __FILE__, __LINE__, __func__ );
            }
            else
            {
                // Neue Verbindung erfolgreich entgegengenommen, Bindpruefung durchfuehren.
                channelConnect->setActiveState( is_binding );
                channel.trace( "%s(%i):INFO(1): %s [ setActiveState(%i) is_binding ]\n", __FILE__, __LINE__, __func__, channelConnect->_channel4.getId() );

                // Pruefen ob der Kanal an eine andere channelID gebunden werden soll.
                if ((x = own.ShouldBoundTo(channelConnect))) {

                    channel.trace( "%s(%i):INFO(1): %s [ Local channel switch %i to %i ]\n", __FILE__, __LINE__, __func__, channelConnect->_channel4.getId(), x );
                    own.ChannelSwitch( channelConnect->_channel4.getId(), x );
                }

                // Pruefung durchlaufen, ggf. umgemappt.
                channelConnect->setActiveState( is_active );
                channel.trace( "%s(%i):INFO(1): %s [ setActiveState(%i) is_active ]\n", __FILE__, __LINE__, __func__, channelConnect->_channel4.getId() );
            }

            // Verbindungsanfrage verarbeitet
            own.m_listenThreadState = LISTENING;
            channel.trace( "%s(%i):INFO(1): %s [ State change to listening... ]\n", __FILE__, __LINE__, __func__ );
            break;
            }
        }

        // Kurze Ausszeit um die Systemlast zu reduzieren.
        Sleep( 10 );
    }

    // Threadzeiger ruecksetzen
    own.m_workerThread[0] = 0;

    // Thread Beendigung ruecksetzen
    own.m_endSignal[0] = FALSE;

#ifndef _MSC_VER
    pthread_exit( NULL );
#endif
    return 0;
}

#ifdef _MSC_VER
unsigned long WINAPI DatafoxLibraryIV::NotifyThread( LPVOID lpParam )
#else
void *DatafoxLibraryIV::NotifyThread( void *lpParam )
#endif
{
    // Muss einen Parameter uebergeben bekommen vom Type DatafoxLibraryIV.
    if ( lpParam == NULL )
        return 0;

    // Referenz auf den uebergebenen Parameter
    DatafoxLibraryIV &own = *reinterpret_cast<DatafoxLibraryIV *>( lpParam );

    // Hauptschleife ausfuehren?
    int run = TRUE;

    // Hauptschleife des Worker
    while ( run )
    {
        // Beendigungssignal abfragen
        if ( own.m_endSignal[1] )
        {
            // Hauptschleife verlassen
            run = FALSE;
            continue;
        }

        int channelID, deviceID;
        if ( own.RecordAvailable( &channelID, &deviceID ) == IV_ERROR_SUCCESS )
        {
#ifdef _MSC_VER
            // Use introspection API to look up the supported methods
            //XmlRpc::XmlRpcClient c(own.m_notifyThreadData.ip, own.m_notifyThreadData.port);
            //XmlRpc::XmlRpcValue params, result;
            //params[0] = channelID;
            //params[1] = deviceID;
            //params[2] = infoString;

            //if (c.execute("DFCRecordAvailable", params, result)) {

            //	TRACE("DFCRecordAvailable: %i\n", int(result));
            //}
#endif
        }

        Sleep( 500 );
    }

    // Threadzeiger ruecksetzen
    own.m_workerThread[1] = 0;

    // Thread Beendigung ruecksetzen
    own.m_endSignal[1] = FALSE;

    return 0;
}
/*! Passwort fuer die verschluesselte Kommunikation setzen.
\param[in] channelID Auf welche Kanalnummer die Vergabe anzwenden ist. 
Wird dieser Parameter mit -1 übergeben bezieht sich das Passwort auf alle Kanäle, 
deviceID wird dann ebenfalls wie -1 behandelt.
\param[in] deviceID 

*/
int DatafoxLibraryIV::SetCommunicationPassword( int channelID, unsigned char *password, int size, int sessionTimeout )
{
    if ( channelID == -1 )
    {
        if ( size == 0 )
        {
            // Versuchen als Zeichenfolge auszuwerten
            for ( ; size < 16 && password[size]; ++size );
        }
        else if ( size < 0 || size > 16 )
        {
            return 0;
        }

        if ( size == 0 )
        {
            // Ggf. Globale Passwortangabe loeschen
            memset( m_globalCommunicationPassword, 0, sizeof( m_globalCommunicationPassword ) );
            m_globalCommunicationPasswordSize = 0;
            m_globalSessionTimeout = 0; // 1 Stunde default.
        }
        else
        {
            memset( m_globalCommunicationPassword, 0, sizeof( m_globalCommunicationPassword ) );
            memcpy( m_globalCommunicationPassword, password, size );
            m_globalCommunicationPasswordSize = size;
            m_globalSessionTimeout = sessionTimeout;
        }

        // Alle verfuegbaren Kanaele durchgehen und Passwort vergeben.
        int i;
        for ( i = 0; i < ACTIVE_CHANNEL_UPPER_BOUND; ++i )
        {
            if ( (m_channelArray[i] != NULL) && (m_channelArray[i]->isGlobalCommunicationPassword()) )
            {
                // Passwort an die Kanalklasse weitergeben
                m_channelArray[i]->setCommunicationPassword( m_globalCommunicationPassword, m_globalCommunicationPasswordSize, m_globalSessionTimeout, 1 );
            }
        }
    }
    else
    {

        ChannelBridge *pChannel = GetAEChannel( channelID );
        if ( pChannel == 0 )
            return 0;

        if ( size == 0 )
        {
            // Versuchen als Zeichenfolge auszuwerten
            for ( ; size < 16 && password[size]; ++size );
        }
        else if ( size < 0 || size > 16 )
        {
            return 0;
        }

        // Es wird ein Passwort direckt fuer diesen Kanal gesetzt. Wenn das Passwort durch size == 0 geloescht
        // werden soll, dann muss es wieder moeglich sein dem Kanal ueber Global ein Passwort zuzuweisen.
        pChannel->setCommunicationPassword( password, size, sessionTimeout, (size == 0) );
    }

    return 1;
}
