#include "../df_system_records.h"
#include "../../DatafoxLibraryIV.h"

DF_API_EXTERN int DF_API_CALL DFCSystemReadRecord(int connectionId, int deviceAddress, char *buffer, int *bufferSize, int *errorNumber)
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return -1;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254))
        || (buffer == NULL) || (bufferSize == NULL) || (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t\t\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, -1);
        return -1;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t\t\t%i\t(%i)", *errorNumber, -1);
        return -1;
    }

    int res = pChannel->systemReadRecord(deviceAddress, buffer, bufferSize, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (res == -1)
    {
        pChannel->resetWithError(*errorNumber);

        pChannel->logAppend("\t\t");
    }
    else if (res == 0)
    {
        pChannel->logAppend("\t\t");
    }
    else
    {
        // Daten fuer Log in Puffer uebergeben
        unsigned char ucTmp[MAX_MB_PAKET_SYSTEM_DATA * 3 + 1];
        int i;
        for (i=0; i < *bufferSize; i++)
        {
            sprintf((char*)&ucTmp[i * 3], "%02X ", (unsigned char)buffer[i]);
        }
        ucTmp[i * 3] = 0;

        pChannel->logAppend("\t%s\t%i", ucTmp, *bufferSize);
    }
    pChannel->logEnd("\t%i\t(%i)", *errorNumber, res);
    return res;
}

DF_API_EXTERN int DF_API_CALL DFCSystemQuitRecord( int connectionId, int deviceAddress, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return -1;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) 
        || (errorNumber == NULL))
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, -1);
        return -1;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, -1);
        return -1;
    }

    int res = pChannel->systemQuitRecord(deviceAddress, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (res == -1) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, res);
    return res;
}

DF_API_EXTERN int DF_API_CALL DFCSystemRestoreRecords( int connectionId, int deviceAddress, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (errorNumber == NULL))
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(0)", IV_ERROR_FUNCTION_PARAMETER);
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(0)", *errorNumber);
        return 0;
    }

    *errorNumber = IV_ERROR_SUCCESS;
    int iRes = pChannel->systemRestoreRecords(deviceAddress, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);
    return iRes;
}
