#ifndef _EXTENDED_FLASH_H_
#define _EXTENDED_FLASH_H_

// ########################################################
// Erweiterter Flash

///////////////////////////////////////
// Struct: StructExtendedFlash

#define EXTENDED_CHECK 0xAAAAAAAE  ///< StructExtendedFlash.ulCheck: Pruefwert fuer Gueltigkeit (Versionskennung)

#define EXTF_EMPTY     0x00000000  ///< StructExtendedFlash.flags: Nicht vorhanden.
#define EXTF_NEW       0x00000001  ///< StructExtendedFlash.flags: Struktur angelegt.
#define EXTF_DOWNLOAD  0x00000010  ///< StructExtendedFlash.flags: Aus Geraet ausgelesene Daten.

/** Diese Struktur enthaelt die Kopfdaten des erweiterten Speichers.

  Diese Struktur enthaelt die Adressen der einzelnen Bloecke. Sie ist die erste und einzige Struktur ihrer art
  im Erweiterten Speicherblock (16kByte).
*/
struct StructExtendedFlash
{
    unsigned int check;                                    ///< muss immer mit EXTENDED_CHECK belegt werden.
    unsigned int size;                                     ///< groesse der Stuktur sizeof(StructExtendedFlash)
    unsigned int offsetIOModule;                           ///< Offset auf IOModul - Struktur
    unsigned int offsetIOModule8DigitalInputs;             ///< Offset auf die 8 StructDigitalInput - Strukturen des IOModul
    unsigned int offsetIntern4DigitalInputs;               ///< Offset auf die 4 StructDigitalInput - Strukturen fuer intern.
    unsigned int offsetEventChains;                        ///< Offset auf PointerArray der Eingabeketten fuer MDE
    unsigned int offsetFreeScope;                          ///< Offset auf naechsten freien Speicherbereich im ExtendedFlashMem
    unsigned int offsetAccessControlTablesV1;              ///< Offset auf die Tabellen der Zutrittskontrolle Version 1.
    unsigned int offsetAccessControlEventChainsV1;         ///< Offset auf OffsetArray der Eingabeketten fuer Zutrittskontrolle Version 1.
    unsigned int offsetGlobalVariableValues;               ///< Offset auf Struktur mit Standardwerten der GVs
    unsigned int offsetAccessControlTablesV2;              ///< Offset auf die Tabellen fuer Zutrittskontrolle Version 2.
    unsigned int offsetAccessControlEventChainsV2;         ///< Offset auf OffsetArray der Eingabeketten fuer Zutrittskontrolle Version 2.
    unsigned int offsetExtendedStruct;                     ///< Offset auf Struktur fuer Erweiterte Funktionalitaeten
    unsigned int offsetIntern4DigitalInputsPlus;           ///< Offset auf die weiteren 4 StructDigitalInput - Strukturen fuer intern.
    unsigned int offsetIntern2AnalogInputs;                ///< Offset auf die 2 StructAnalogInput - Strukturen fuer intern.
    unsigned int offsetEnergieSchemas;                     ///< Offset auf die Energieschemen
    unsigned int offsetMachineDefinitions;                 ///< Offset auf die Maschienendefinitionen
    unsigned int offsetEnergieMemo;                        ///< Offset auf Zeichenfolge für Memo der Energieverwaltung
    unsigned int offsetTransponderConfiguration;           ///< Offset auf die Transponderkonfigurationen
    // Die folgenden Zeiger werden von DatafoxStudioIV-Versionen < 04.03.03.02 ebenfalls gelesen, aber überschrieben.
    unsigned char unused_1[20];                               ///< Reserviert. Auf 100 Byte Strukturgroesse bringen.
    unsigned int flags;                                    ///< Flags fuer Struktur und Erweiterten Speicher
    // Ab der Version 04.03.03.01 auf 200 Byte vergrößert.
    // Die folgenden Zeiger werden von DatafoxStudioIV-Versionen < 04.03.03.02 nicht gelesen.
    unsigned int offsetInternDigitalInputs;                ///< Offset auf StructHeaderOffsetArray für die StructDigitalInput - Strukturen intern (ersetzt pulMDEDigital).
    unsigned int offsetInternDigitalInputsExt;             ///< Offset auf StructHeaderOffsetArray für die StructDigitalInputExt - Strukturen intern (ersetzt pulMDEDigitalExt).
    unsigned int offsetInternAnalogInputs;                 ///< Offset auf StructHeaderOffsetArray für die StructAnalogInput - Strukturen intern (ersetzt pulMDEAnalog).
    unsigned int offsetTimers;                             ///< Offset auf StructHeaderOffsetArray für die StructTimer - Stukturen (ersetzt pulExtended.timer[4]).
    unsigned int offsetVibrationSensor;                    ///< Offset auf eine StructVibrationSensor für den Vibrationssensor.
    unsigned char unused_2[80];
};

#define sizeof_StructExtendedFlash_1    100 // Strukturgroesse der Version 1 ist: 100Byte
#define sizeof_StructExtendedFlash_2    200 // Strukturgroesse der Version 2 ist: 200Byte

///////////////////////////////////////
// Struct: StructHeaderOffsetArray

/** Diese Struktur enthaelt die Kopfdaten für folgende OffsetArray - Strukturdaten.

  Diese Struktur ist eine vereinheitlichte Hilfstruktur für OffsetArray-Strukturdaten,
  wie z. B. für StructDigitalInput, StructDigitalInputExt, StructAnalogInput, StructTimer, etc.
*/
struct StructHeaderOffsetArray
{
    unsigned int size;        ///< Version / Groesse der Stuktur sizeof(StructHeaderOffsetArray)
    unsigned int count;       ///< Anzahl der Pointer in dem Array
    unsigned int offsetArray; ///< Basisoffset auf die Sturkturbasis, zum Pointerarray.
};

///////////////////////////////////////
// Struct: StructAE3IOModule

#define MAX_UNIT_TXT	5 ///< StructAE3IOModule.szUnitU: Maximallaenge des Einheitssuffix mit 0 terminie.

/** Diese Struktur enthaelt die Informationen zum IO-Module.

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructIOModule
{
    // Spannungseingaenge
    unsigned char ucActiveU;       ///< Flags welcher Eingang aktiv.
    char szNameU[8][MAX_NAME];     ///< Bezeichner der Eingaenge.
    float fMinU[8];                ///< Minimalwert fuer Berechnung.
    float fMaxU[8];                ///< Maximalwert fuer Berechnung.
    char szUnitU[8][MAX_UNIT_TXT]; ///< Einheit - suffix der Berechnung.
    unsigned char ucOnScreenU;     ///< Flags ob Ausgabe auf Display erlaubt.

    // Stromeingaenge
    unsigned char ucActiveI;       ///< Flags welcher Eingang aktiv.
    char szNameI[8][MAX_NAME];     ///< Bezeichner der Eingaenge
    float fMinI[8];                ///< Minimalwert fuer Berechnung.
    float fMaxI[8];                ///< Maximalwert fuer Berechnung.
    char szUnitI[8][MAX_UNIT_TXT]; ///< Einheit - suffix der Berechnung.
    unsigned char ucOnScreenI;     ///< Flags ob Ausgabe auf Display erlaubt.
};


///////////////////////////////////////
// Struct: StructDigitalInput

/** Diese Struktur enthaelt die Informationen fuer die Behandlung eines digitalen Einganges.

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructDigitalInput
{
    unsigned char ucUseAs;         ///< Verwendung als Start/Stop, Zaehler etc..
    unsigned char ucStartLowHigh;  ///< Auszufuehrende Eingabekette bei Flankenwechsel low -> high
    unsigned char ucStartHighLow;  ///< Auszufuehrende Eingabekette bei Flankenwechsel high -> low
    unsigned short uiCountDivisor; ///< Bei Verwendung als Zaehler, dessen Teiler
    unsigned short uiCycle;        ///< Bei Verwendung als Zaehler, dessen Zyklus.
    unsigned short uiTimeOut;      ///< Bei Verwendung als Zaehler mit Start/Stop, dessen Timeout.
    unsigned int ulCountDivisor32;///< Bei Verwendung als Zaehler, dessen Teiler als 32bit
    unsigned short uiDebounceHigh; ///< Endprellzeit fuer Flankenwechsel Low-High
    unsigned short uiDebounceLow;  ///< Endprellzeit fuer Flankenwechsel High-Low
    unsigned char ucInvokeInputChainCountDivisor; ///< Auszufuehrende Eingabekette bei erreichtem Zaehler
    unsigned char ucInvokeInputChainCycle;        ///< Auszufuehrende Eingabekette bei erreichtem Zyklus

    unsigned char unused[2];       ///< Reserviert.
}; // 21 Byte

///////////////////////////////////////
// Struct: StructDigitalInputExt

/** Diese Struktur enthaelt zusaetzliche Informationen fuer die Behandlung eines digitalen Einganges.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructDigitalInputExt
{
    unsigned short uiCavity;   ///< Die zu verwendende Kavitaet.
    unsigned short uiStrokes;  ///< Die zu verwendende Hubzahl.

    unsigned char unused[17];       ///< Reserviert.
}; // 21 Byte

///////////////////////////////////////
// Struct: StructAnalogInput

/** Diese Struktur enthaelt die Informationen fuer die Behandlung eines analogen Einganges.

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructAnalogInput
{
    unsigned char useAs;               ///< Eingangsverwendung als ...
    unsigned char show;                ///< Flags ob Ausgabe auf Display erlaubt.
    unsigned char sampleRate;          ///< Abtastrate fuer den analogen Eingang
    char name[MAX_NAME];               ///< Bezeichner der Eingaenge.
    char unit[MAX_UNIT_TXT];           ///< Einheit - suffix der Berechnung.
    float minLimit;                    ///< Minimalwert fuer Berechnung.
    float maxLimit;                    ///< Maximalwert fuer Berechnung.
    float minInput;                    ///< Minimalwert fuer Eingangsspannung.
    float maxInput;                    ///< Maximalwert fuer Eingangsspannung.
    float limitUpperAlarm;             ///< Wert fuer die obere Alarmgrenze
    float limitUpperWarning;           ///< Wert fuer die obere Warngrenze
    float limitLowerWarning;           ///< Wert fuer die untere Warngrenze
    float limitLowerAlarm;             ///< Wert fuer die untere Alarmgrenze
    unsigned char limitTolerance;      ///< Grenzwertabweichung
    unsigned char invokeInputChain[5]; ///< Auszufuehrende Eingabekette bei Statuswechsel

    unsigned char unused[27];             ///< Reserviert.
}; // 92 Byte


///////////////////////////////////////
// Struct: StructVibrationSensor

/** Diese Struktur enthaelt die Informationen fuer die Behandlung eines Vibrationssensors.

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructVibrationSensor
{
    unsigned char useAs;               ///< Eingangsverwendung als ...
    unsigned int  level;               ///< Wert fuer das auslösen Vibration erkannt
    unsigned short timeout;            ///< Erkennungszeitraum in Sekunden
    unsigned char invokeInputChain[2]; ///< Auszufuehrende Ketten eione für start und eine für stopp der Vibration

    unsigned char unused[23];             ///< Reserviert.
}; // 32 Byte


///////////////////////////////////////
// Struct: StructValueGVx

/** Diese Struktur enthaelt die ggf. zu setzenden Standardwerte der globalen Variablen.

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructValueGVx
{
    unsigned char ucFlags;            ///< Bitfeld gibt an welche Variablen gesetzt werden sollen.
    char szValue[8][MAX_LENGTH_FELD]; ///< Zu setzender Standardwert der Variablen.
}; // 169 Byte


///////////////////////////////////////
// Struct: StructFingerprint

/** Diese Struktur enthaelt die Einstellparameter fuer ein Fingerprint-Modul

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructFingerprint
{
    unsigned char method;             ///< Methode des Fingerprints
    unsigned char segment;            ///< Startsegment der abgelegten Fingertemplates auf Ausweis
    unsigned char securityLevel;      ///< Sicherheitslevel; der sich aus Falschrueckweisungrate et Falschakzeptanzrate ergibt.
    float securityFRR;                ///< Falschrueckweisungsrate
    float securityFAR;                ///< Falschakzeptanzrate
    unsigned char flags;              ///< Bit1+Bit2: Templateformat
                                      ///< 0 = DIN V66400 (161 Byte)
                                      ///< 1 = Idencom-Compact (216 Byte)
                                      ///< 2 = Idencom-Standard (561 Byte)
                                      ///< Bit[3 - 8] Frei.
    unsigned char quality;            ///< Mindestbildqualitaet beim scannen
    unsigned char minutiaeCount;      ///< Mindestanzahl der Minutiae beim scannen
    unsigned char passwordGroup;      ///< Zu verwendende Passwortgruppe beim Schreiben des Ausweises
    char unused[5];                      ///< Reserve fuer evntl. Erweiterungen
}; // 20 Byte

// Zusatztypen (161 Byte)
struct StructFingertemplateDIN
{
    unsigned int pidHi;
    unsigned int pidLo;
    unsigned char fid;
    unsigned char quality;
    unsigned char count;
    unsigned char data[50][3];
}; // 161 Byte

// Zusatztypen (216 Byte)
struct StructFingertemplateIdencom
{
    unsigned int pidHi;
    unsigned int pidLo;
    unsigned char fid;
    unsigned char quality;
    unsigned char count;
    unsigned char data[41][5];
}; // 216 Byte

// Zusatztypen (561 Byte)
struct StructFingertemplateIdencomStandard
{
    unsigned int pidHi;
    unsigned int pidLo;
    unsigned char fid;
    unsigned char quality;
    unsigned char count;
    unsigned char data[50][11];
}; // 561 Byte

struct StructFingertemplateId
{
    char pid[21];
    char fid[4];
}; // 25 Byte

///////////////////////////////////////
// Struct: StructTBAdaption

/** Diese Struktur enthaelt die Einstellparameter fuer eine Timeboyadaption an MasterIV-Geraete

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructTimeboyAdaption
{
    unsigned char method;               ///< Methode der Adaption
    unsigned char count;                ///< Anzahl der Steckplaetze bei Mehrfachdocking
    unsigned char records;              ///< Datenablage auf MasterIV oder Timeboy
    unsigned char invokeInputChainPush; ///< Auszufuehrende Eingabekette, wenn Timeboy gesteckt wird
    unsigned char invokeInputChainPull; ///< Auszufuehrende Eingabekette, wenn Timeboy gezogen wird
    char unused[15];                       ///< Reserve fuer evntl. Erweiterungen
}; // 20 Byte

///////////////////////////////////////
// Struct: StructGPRS

/** Diese Struktur enthaelt die Einstellparameter fuer eine GPRS-Kommunikation

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructGPRS
{
    unsigned char method;               ///< Methode der GPRS-Kommunikation
    unsigned char invokeInputChain;     ///< Auszufuehrende Eingabekette, wenn Alive-Timeout abgelaufen
    char unused[8];                        ///< Reserve fuer evntl. Erweiterungen
}; // 10 Byte

///////////////////////////////////////
// Struct: StructEnergieOption

/** Diese Struktur enthaelt die Einstellparameter fuer die EnergieOption zum Modbil-MasterIV.

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructEnergieOption
{
    unsigned char mode;                 ///< Aktivierung der Option
    unsigned char digitalIn;            ///< Eingabekette fuer das An/Aus der Zuendung
    unsigned short timeout;             ///< Timout, nach dem die Module nach einem An, eingeschaltet werden
    unsigned char invokeRelease;        ///< Eingabekette zur Fahrzeugfreigabe.
    unsigned char invokeGPS;            ///< Eingabekette bei gueltigen GPS-Daten.
    unsigned char digitalOut;           ///< Zu schaltendes Relais fuer Selbstabschaltung
    unsigned char flags;                ///< Flags fuer die Energieoption
                                        ///< Bit0: Einzel-(0) oder Dauerfreischaltung(1).
                                        ///< Bit1: GPS-Start/Stopp für An/Aus der Zündung nutzen.
                                        ///< Bit2: Immer Bewegung, Erkennung fuer An/Aus der Zündung deaktiviert.
                                        ///< Bit3-7: unused
    unsigned char powerLow;             ///< Tiefentladeschutz. Wert in Volt. 10.2V wird als 102 also *10 abgelegt
    unsigned char unused[7];               ///< Reserve
}; // 16 Byte

///////////////////////////////////////
// Struct: StructTimer

/** Diese Struktur enthaelt die Einstellparameter fuer die Timer
    Die Struktur ist im extendet Speicherblock abgelegt.
*/
struct StructTimer
{
    unsigned char ucUseAs;          ///< Verwendung als Timer in Eingabeketten etc..
    unsigned short uiCycle;         ///< Bei Verwendung als Timer, dessen Zyklus.
    unsigned short uiTimeOut;       ///< Bei Verwendung als Timer, dessen Timeout.
    unsigned char invokeInputChain; ///< Auszufuehrende Eingabekette, wenn Timeout abgelaufen, oder danach der Zyklus abgelaufen.

    unsigned char unused[10];          ///< Reserve
}; // 16 Byte

///////////////////////////////////////
// Struct: StructGPS

/** Diese Struktur enthaelt die Einstellparameter fuer die GPS-Signalverarbeitung.
    Die Struktur ist im extendet Speicherblock abgelegt.
*/
struct StructGPS
{
    unsigned char startSpeed;             ///< Zu erreichende Geschwindigkeit als Starterkennung. Einheit: km/h
    unsigned short startMove;             ///< Zu erreichende Bewegungsentfernung als alternative Starterkennung. Einheit: Meter
    unsigned char invokeInputChainStart;  ///< Auszuführende Eingabekette für Start Erkennung, 0 = keine Zuordnung
    unsigned short stopTimeout;           ///< Timeoutzeit für die stopSpeed unterschritten sein muss, damit ein Stopperkennung stattfindet. Einheit: Sekunden
    unsigned char stopSpeed;              ///< Geschwindigkeit die für erzeugen einer Stopperkennung unterschritten werden muss. Einheit: km/h
    unsigned char invokeInputChainStop;   ///< Auszuführende Eingabekette für Stopperkennung, 0 = keine Zuordnung
    unsigned short cycle;                 ///< Zykluszeit, in der ein Zyklusdatensatz erzeugt werden soll, wenn keine anderen Fahrdatensätze erzeugt werden. Einheit: Sekunden.
    unsigned char invokeInputChainCycle;  ///< Auszuführende Eingabekette für Zyklusdatensatz, 0 = keine Zuordnung
    unsigned char course;                 ///< Wert für Kursänderung.
    unsigned char invokeInputChainCourse; ///< Auszuführende Eingabekette für Kursdatensatz, 0 = keine Zuordnung
    unsigned short move;                  ///< Wert für Bewegung. Einheit: Meter.
    unsigned char invokeInputChainMove;   ///< Auszuführende Eingabekette für Bewegungsdatensatz, 0 = keine Zuordnung
    unsigned char unused[2];                 ///< Reserve
}; // 18 Byte

///////////////////////////////////////
// Struct: StructMachineDefinition

/** Diese Struktur enthaelt Informationen ueber die Definition einer Maschine.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructMachineDefinition
{
    unsigned char flags;   ///< Gibt Informationsbits an. 
                           ///< Bit 0: 0 = Deaktiviert, 1 = Aktiviert.
    char name[MAX_NAME];   ///< Bezeichnung der Maschine
    unsigned char inputs;  ///< Gibt die zugeordneten digitalen Eingänge an. 
                           ///< Bit 0: E1
                           ///< ...
                           ///< Bit 7: E8
    unsigned char unused[5];///< Reserviert.
}; // 24 Byte

///////////////////////////////////////
// Struct: StructExtended

/** Diese Struktur enthaelt zusaetzliche Informationen.

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
  In dieser Struktur werden alle weiteren Informationen abgelegt.
*/
struct StructExtended
{
    unsigned short doubleInputTime;  ///< Zeit fuer "Doppelte Eingabe" PZE1/PZE2 Modus in Sekunden
    unsigned short confirmationTime; ///< Zeit fuer "Bestätigungsfelder" in Sekunden
    // Sommer/Winterzeitumschaltung
    unsigned char daylightActive;    ///< Aktivieren der Sommer-/Winterzeitumschaltung (daylight saving)
                                     ///< 0->Deaktiviert, 1->Alte Aktivierung, 2->Aktiv und von FW auszuwerten
    unsigned char daylight1Month;    ///< Monat in dem Umgestellt wird (1 - 12)
    unsigned char daylight1Week;     ///< Monatswoche in der Umgestellt wird (1 - 5, wobei 5 letzte im Monat bedeutet)
    unsigned char daylight1Day;      ///< Tag der Umschaltung (1 - 7, wobei 1 = Montag)
    unsigned char daylight1Hour;     ///< Stunde fuer Umschaltung (0 - 23)
    unsigned char daylight2Month;    ///< Monat in dem Umgestellt wird (1 - 12)
    unsigned char daylight2Week;     ///< Monatswoche in der Umgestellt wird (1 - 5, wobei 5 letzte im Monat bedeutet)
    unsigned char daylight2Day;      ///< Tag der Umschaltung (1 - 7, wobei 1 = Montag)
    unsigned char daylight2Hour;     ///< Stunde fuer Umschaltung (0 - 23)
    char transponderPassword32[9];   ///< Passwort fuer Hitag2 (32bit) in ASCII (Hex) abgelegt.
    char transponderPasswordA0[13];  ///< Passwort fuer Mifare (48bit) in ASCII (Hex) abgelegt.
    unsigned char phgAESKey[16];     ///< Schluessel fuer Kommunikationsverschluesselung der PHG Leser
    StructFingerprint fingerprint;        ///< Datenstruktur zum Fingerprint
    StructTimeboyAdaption adaption;            ///< Anbindungsstruktur von Timboys an MasterIV
    StructGPRS gprs;                      ///< Struktur fuer Zusatzangaben bezueglich GPRS, wie z. B. Alive.
    StructEnergieOption energieOption;    ///< Struktur fuer Energieoption bei Mobil-MasterIV
    StructTimer timer[4];				 ///< Struktur fuer Timer in der Signalverarbeitung (vorgesehen sind 4 Timer a 16 Byte)
    unsigned char transponderPasswordFlag; ///< Passwortflag fuer Passwortverwendung von Mifare
                                           ///< Bit 0, 1: Passwortverwendung für Lesen/Schreiben A0/B0; 0=KeyA/KeyA, 1=KeyA/KeyB, 2=KeyB/KeyB
                                           ///< Bit 2, 3: Passwortverwendung für Lesen/Schreiben A1/B1; 0=KeyA/KeyA, 1=KeyA/KeyB, 2=KeyB/KeyB
                                           ///< Bit 4, 5: Passwortverwendung für Lesen/Schreiben A2/B2; 0=KeyA/KeyA, 1=KeyA/KeyB, 2=KeyB/KeyB
                                           ///< Bit 6: Frei
                                           ///< Bit 7: Autologin; Verwendung aller Passwörter für Lesen
    char transponderPasswordB0[13];  ///< Passwort fuer Mifare (48bit) in ASCII (Hex) abgelegt.
    char transponderPasswordA1[13];  ///< Passwort fuer Mifare (48bit) in ASCII (Hex) abgelegt.
    char transponderPasswordB1[13];  ///< Passwort fuer Mifare (48bit) in ASCII (Hex) abgelegt.
    char transponderPasswordA2[13];  ///< Passwort fuer Mifare (48bit) in ASCII (Hex) abgelegt.
    char transponderPasswordB2[13];  ///< Passwort fuer Mifare (48bit) in ASCII (Hex) abgelegt.
    int transponderSectorTrailer;    ///< Zugriffseinstellungen für den globale Sektor.
    unsigned char transponderInitialValue[17]; ///< Startwert von Mifare-Ausweis
    unsigned char memoryUsage;           ///< Speicheraufteilung zwischen Daten/Listen in %. Wert gibt die % für Daten reserviert an.
    unsigned char entranceOnline;        ///< Betriebsmodus der Zutrittskontrolle. 0=Offline, 1=Online, 2=Online/Offline mit Timeout.
    unsigned char entranceOnlineTimeout; ///< Timeout fuer die Umschaltung zwischen Online/Offline Betrieb der ZK.
    StructDigitalInputExt digitalExtE[8];     ///< Erweiterte Strukturen für die Digitalen Eingaenge.
    char fileName[44];                   ///< Dateiname der Setupdatei. Wird vor der Übertragung durch Laderoutine der Setupdatei gesetzt.
    char fileTimeStamp[20];              ///< Modifizierungszeitstempel der Setupdatei. Wird vor der Übertragung durch Laderoutine der Setupdatei gesetzt.
    unsigned char passwordSetup[17];     ///< Passwort für die Kommunikation
    StructGPS gps;							 ///< Struktur für GPS Datensatz Erzeugung
    unsigned char userPasswordEnable;    ///< Benutzerpasswort verwenden, ansonsten wird das Systempasswort fürs Userpasswort verwendet.
    unsigned char escJumpToMainmenu;     ///< Im PZE-Modus wird durch drücken der ESC-Taste das Hauptmenü angezeigt.
    unsigned char ledPerFieldFunction;   ///< AE/BDE-MasterIV: LED steuerung mit Feldfunktion "LED schalten" steuern.
    unsigned short dataWriteOKTime;      ///< Die Anzeigezeit der Meldung "Datenspeicherung erfolgreich". 0=aus, 500ms default,bereich 0,500-5000 in 250ms Schritten; Einheit ms
    unsigned char dataWriteOKFontType;   ///< Der verwendete Zeichensatztyp bei Meldung "Datenspeicherung erfolgreich". 0=
    unsigned int powerActiv;            ///< Gibt an ob die Energieverwaltung des Setups aktiviert ist, inclusive Flags für die aktivierten Module.
                                         ///< siehe PAF_x (P)ower(A)ctiv(F)lag
    unsigned char powerClockTimerCount;  ///< Gibt an wie viele Timer die zu einer Bestimmten Uhrzeit auslösen existieren dürfen.
    unsigned char spsQuery;              ///< Abfrage einer angeschlossenen SPS. Feldfunktion SPS-Abfrage.
    unsigned char dataOnCard;                 ///< Datenarrays auf Transponderkarten schreiben.
    unsigned char zkExtension;                ///< Erweiterte Zutrittskontrolle verwenden.
    unsigned short zkTagTimeToClose;          ///< Die Zeit wie lange der Ausweis vor einen ZK Leser gehalten werden muss bis eine Schliessung erfolgt.
    unsigned short zkAlarmTimeout;            ///< Auszeit nachdem ein Alarmdatensatz generiert wird, weil ein Digitaler Eingang zu lange offen war.
    unsigned char akkuLoadOptimization;       ///< Ladeerhaltung voller Akkus optimieren.
    unsigned char showSmallWatch;             ///< Zusaetzlich zur Anzeige der großen Uhr wird eine kleine Uhr mit Sekundenanzeige eingeblendet.
    unsigned char dontDimBacklightForCharge;  ///< Die Hintergrundbeleuchtung soll nicht gedimmt werden um die Aufladung der Akku's zu beschleunigen.
    unsigned char flagsEnergie;               ///< Zusaetzliche Flags fuer bestimmtes Geraeteverhalten in der Energieverwaltung.
                                            ///< Bit1: TimeboyIV soll GPS abschalten, wenn er sich in der Dockingstation befindet.
                                            ///< Bit2: TimeboyIV soll Mobilfunk abschalten, wenn er sich in der Dockingstation befindet.
                                            ///< Bit3-8: Reserve.
	unsigned short displayOffAfter;			///< Display ausschalten nach [Wert] Minuten ohne Bedienung. 0 = niemals aus bis 990 = 16,5 Stunden  
	unsigned char flagsShowSetupNote;		///< Flag ob ein Hinweis zu Änderungen im Setup angezeigtwerden soll 
											///< Bit1: Energiesparoption "Display aus nach n Minuten ohne Bedienung."
											///< Bit2: Standardwert wurde auf 120 Minuten gesetzt
    unsigned char  unused[461];             ///< Frei fuer Erweiterungen
}; // 1024 Byte

// Defines und Strukturen fuer die Transponderverfahren.
#include "transponder.h"

// Defines und Strukturen fuer die Energieschemen
#include "Power.h"

#endif
