#ifndef TRANSPONDER_H
#define TRANSPONDER_H

///////////////////////////////////////
// Struct: StructTransponder

// Flags fuer TP StructTransponder.tbModeX
#define BDTP_ACTIVE          0x80 ///< StructTransponder.tpMode: Einstellung fuer Transponder aktiviert.
//  lesen / schreiben
#define BDTP_WRITE           0x40 ///< StructTransponder.tpMode: Der Wert wird ins Segment geschrieben, ansonsten gelesen.

#define BDTP_DEC             0x00 ///< StructTransponder.tpMode: Behandlung der Daten als Dezimalwert.
#define BDTP_HEX             0x10 ///< StructTransponder.tpMode: Behandlung der Daten als Hexadezimalwert.
#define BDTP_ASCII           0x20 ///< StructTransponder.tpMode: Behandlung der Daten als ASCII - Zeichenfolge.
#define BDTP_HEX_TURNED      0x30 ///< StructTransponder.tpMode: Behandlung der Daten als Hexadezimalwert aber umgekehrte Bytefolge.

// Untere 4bit sind Anzahl feste stellen

/** Diese Struktur dient bei Angabe eines Hitagtransponders, der zu verwendenden Segmente, Ablageformate sowie Länge.

Diese Struktur kommt als Erweiterung in der Struktur StructExNumFeld zur Anwendung. Sie wird auch verwendet
um die globale Angabe sowie die Angabe bei der Zutrittskontrolle im StructConfig abzulegen.
*/
struct StructTransponder
{
  unsigned char tpSegment1;  ///< Gibt das erste Segment an, aus dem der Wert gelesen / oder in den der Wert geschieben werden soll.
  unsigned char tpMode1;     ///< Gibt den Modus des Wertes an. Oberes Nibble beinhaltet oben definierte Flags, unteres die feste Laengenangabe. 
  unsigned char tpSegment2;  ///< siehe Beschreibung zu tpSegment1, hier fuer ein zweites Segment.
  unsigned char tpMode2;     ///< siehe Beschreibung zu tpMode1, hier fuer einen zweiten Mode.
  unsigned char tpSegment3;  ///< siehe Beschreibung zu tpSegment1, hier fuer ein drittes Segment.
  unsigned char tpMode3;     ///< siehe Beschreibung zu tpMode1, hier fuer einen dritten Mode.
}; // 6 Byte

/** Diese Struktur dient bei Angabe eines Mifaretransponders, der zu verwendende Sektor, Ablageformat sowie Auswahl.

Diese Struktur kommt als Erweiterung in der Struktur StructExNumFeld zur Anwendung. Sie wird auch verwendet
um die globale Angabe sowie die Angabe bei der Zutrittskontrolle im StructConfig abzulegen.
*/
struct StructTransponderMifare
{
  unsigned char sectorBlock;      ///< Gibt im oberen Nibble den Sektor, im unteren den Block an, 
                                  ///< aus dem der Wert gelesen / oder in den der Wert geschieben werden soll.
  unsigned char mode;             ///< Oberes Nibble beinhaltet oben definierte Flags fuer das Datenformat, 
                                  ///< unteres die feste Laengenangabe. Bei Hex-Format ist der Wert mal 2 zu nehmen.
  unsigned char cut;              ///< Gibt im oberen Nibble die Startposition des ersten zu lesenden Bytes der 16 moeglichen an.
                                  ///< Das untere Nibble ist frei.
  unsigned char passwordMode;     ///< Gibt den zu verwendenden Passwortbereich an.
  unsigned char securityLevel;    ///< Gibt den zu verwendenden Security-Level des Mifare-Plus ausweises an.
  unsigned char unused;             ///< Frei.
}; // 6 Byte

/** Diese Struktur dient bei Angabe eines HID iClass Transponders, der zu verwendende Ausweistyp, Ablageformat sowie Auswahl.

Diese Struktur kommt als Erweiterung in der Struktur StructExNumFeld zur Anwendung. Sie wird auch verwendet
um die globale Angabe sowie die Angabe bei der Zutrittskontrolle im StructConfig abzulegen.
*/
struct StructTransponderHIDiClass
{
  unsigned char type;         ///< Gibt den zu verwendenden Ausweistyp an.
                              ///< 0 -> H10301
                              ///< 1 -> H10302
                              ///< 2 -> H10304
                              ///< 3 -> Freies Datenformat
  unsigned char format;       ///< Ablageformat.
                              ///< 0 -> Dezimal
                              ///< 1 -> Hexadezimal
                              ///< 2 -> Reserviert für ASCII nicht unterstützt!
                              ///< 3 -> Hexadezimal gedreht
  unsigned char length;       ///< Ablage feste Laengenangabe.
                              ///< 0 -> Keine feste Länge
                              ///< n -> Anzahl Zeichen ggf. mit Vornullen auf die angegebene Länge bringen.
  unsigned char unused[3];    ///< Frei.
}; // 6 Byte


///////////////////////////////////////
// Struct: StructTransponderLegicAdvantOld

/** Diese Struktur dient bei Angabe eines Legic Advant Transponders in den Globalen Einstellungen, 
das zu verwendende Segment, Ablageformat sowie Auswahl.

Diese Struktur wird verwendet um globale Angaben fuer Legic Advant Ausweise in der Firmwareversionen
04.02.02 aufzunehmen. Ab Version 04.02.03 wird auf die StructTransponderLegicAdvant ausgewichen.
*/
struct StructTransponderLegicAdvantOld
{
  unsigned char segment;      ///< Gibt das zu verwendende Segment an. Segment 0 bedeutet, dass der Ausweis nicht segmentiert ist.
  unsigned short pos;         ///< Position des zuerst gelesenen Datenbyte.
  unsigned char length;       ///< Anzahl zu lesender Datenbyte.
  unsigned char format;       ///< Gibt das abzulegende Werteformat an.
                              ///< 0 -> Dezimal
                              ///< 1 -> Hexadezimal
                              ///< 2 -> ASCII
                              ///< 3 -> Hexadezimal gedreht
  unsigned char formatLength; ///< Gibt die abzulegende Werteformatlaenge an.
                              ///< 0 -> Keine feste Laengenangabe
                              ///< n -> Anzahl der abzulegenden Zeichen ggf. mit Vornullen auf die gewuenschte Laenge bringen.
  unsigned char typeFlags;    ///< Enthält die gewuenschten Ausweistypen und zusaetzliche Flags.
                              ///< Bit 8 und 7 (Maske 0xC0) beinhaltet den Ausweistyp
                              ///<    0 -> Nur Advant Ausweise
                              ///<    1 -> Nur Prime Ausweise
                              ///<    2 -> Advant und Prime Ausweise
                              ///<    3 -> Frei
                              ///< Bit 6 und 5 (Maske 0x30) CRC Pruefung
                              ///<    0 -> Keine Pruefung
                              ///<    1 -> 8Bit
                              ///<    2 -> 16Bit
                              ///<    3 -> Frei
                              ///< Bit 4. Bei gesetztem Bit ist Access Segment Definition aktiv.
                              ///< Bit 3. Bei gesetztem Bit sind ISO14443 Ausweise zugelassen.
                              ///< Bit 2. Bei gesetztem Bit sind ISO15693 Ausweise zugelassen.
                              ///< Bit 1. Bei gesetztem Bit sind Seriennummern von Prime Ausweisen nach dem Advant Format zu liefern.
  unsigned char valType;      ///< Zu Lesender Wertetyp.
                              ///< 0 -> Seriennummer
                              ///< 1 -> Freie Datenauswahl
                              ///< 2 -> Kaba group Header von Prime Ausweisen lesen.
                              ///< 3 -> Segment mittels Suche finden.
                              ///< ... -> Frei
  unsigned short crcAddress;  ///< Position des CRC
  unsigned char kghSearch[13];///< Zu suchende Bytefolge fuer den Kaba group header
  unsigned char kghSearchLength; ///< Laenge der zu suchenden Bytefolge des Kaba group header
  unsigned char flags;        ///< Enthaelt zusaetzliche Flags
                              ///< Bit1: Bei gesetztem Bit wird nach KGH gesucht. Nur bei Ausweisen vom Typ Legic Prime.
                              ///< Bit2 - 8: Frei.
  unsigned char unused[5];       ///< Reserve für Erweiterungen
}; // 30 Byte


///////////////////////////////////////
// Struct: StructTransponderLegicAdvantHeader

/** Diese Struktur enthaelt zusaetzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen fuer die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von count unsigned short Offsets, die die Offsets auf die
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhaengig.
*/
struct StructTransponderLegicAdvantHeader
{
  unsigned char structSize;       ///< Groesse dieser Struktur (ist gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray)
  unsigned char structFlags;      ///< Flags fuer die Strukturuebergreifende Verarbeitung.
                                  ///< Bit1: Passwoerter der Strukturen werden mittels RC4 und DatafoxKey verschluesselt.
                                  ///< Bit2-8: Unused
  unsigned char transponderType;  ///< Typ des Leseverfahrens.
  unsigned char count;            ///< Anzahl der Eintraege in dem Offsetarray, was auf diese Struktur folgt.
  unsigned char mode;             ///< Enthaelt die gewuenschten Ausweistypen.
                                  ///< 0 -> Nur Advant Ausweise
                                  ///< 1 -> Nur Prime Ausweise
                                  ///< 2 -> Advant und Prime Ausweise
  unsigned char flags;            ///< Enthaelt zusaetzliche Flags.
                                  ///< Bit1: Bei gesetztem Bit sind ISO14443 Ausweise zugelassen.
                                  ///< Bit2: Bei gesetztem Bit sind ISO15693 Ausweise zugelassen.
                                  ///< Bit3: Bei gesetztem Bit ist Access Segment Definition aktiv.
                                  ///< Bit4: Bei gesetztem Bit wird nach KGH gesucht. Nur bei Ausweisen vom Typ Legic Prime.
}; // 6 Byte

/** Diese Struktur dient bei Angabe eines Legic Advant Transponders in den Globalen Einstellungen, 
das zu verwendende Segment, Ablageformat sowie Auswahl.

Diese Struktur kommt als Erweiterung in der Struktur StructExNumFeld zur Anwendung. Sie wird auch verwendet
um die globale Angabe sowie die Angabe bei der Zutrittskontrolle im StructConfig abzulegen.
*/
struct StructTransponderLegicAdvant
{
  unsigned char rwType;         ///< Zu Lesender Wertetyp.
                                ///< 0 -> Seriennummer
                                ///< 1 -> Freie Datenauswahl
                                ///< 2 -> Kaba group Header mittels Suche finden.
                                ///< ... -> Frei
  unsigned char rwFlags;        ///< Enthaelt zusaetzliche Flags.
                                ///< Bit1: Bei gesetztem Bit sind Seriennummern von Prime Ausweisen nach dem Advant Format zu liefern.
  unsigned char rwSegment;      ///< Gibt das zu verwendende Segment an. Segment 0 bedeutet, dass der Ausweis nicht segmentiert ist.
  unsigned short rwOffset;      ///< Position des zuerst gelesenen Datenbyte.
  unsigned short rwLength;      ///< Anzahl zu lesender Datenbyte.
  unsigned char kghSearch[13];  ///< Zu suchende Bytefolge fuer den Kaba group header
  unsigned char kghSearchLength;///< Laenge der zu suchenden Bytefolge des Kaba group header
  unsigned char crcCheck;       ///< Art der CRC Pruefung
                                ///< 0 -> Keine Pruefung
                                ///< 1 -> 8Bit
                                ///< 2 -> 16Bit
  unsigned short crcAddress;    ///< Position des CRC
  unsigned char outputFormat;   ///< Gibt das abzulegende Werteformat an.
                                ///< 0 -> Dezimal
                                ///< 1 -> Hexadezimal
                                ///< 2 -> ASCII
                                ///< 3 -> Hexadezimal gedreht
                                ///< 4 -> Dezimal gedreht
  unsigned short outputLength;  ///< Gibt die abzulegende Werteformatlaenge an.
                                ///< 0 -> Keine feste Laengenangabe
                                ///< n -> Anzahl der abzulegenden Zeichen ggf. mit Vornullen auf die gewuenschte Laenge bringen.
  unsigned char unused[3];         ///< Reserve für Erweiterungen
}; // 30 Byte


///////////////////////////////////////
// Struct: StructTransponderMifareDesfireHeader

/** Diese Struktur enthaelt zusaetzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen fuer die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von count unsigned short Offsets, die die Offsets auf die
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhaengig.
*/
struct StructTransponderMifareDesfireHeader
{
  unsigned char structSize;       ///< Groesse dieser Struktur (ist gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray)
  unsigned char structFlags;      ///< Flags fuer die Strukturuebergreifende Verarbeitung.
                                  ///< Bit1: Passwoerter der Strukturen werden mittels RC4 und DatafoxKey verschluesselt.
                                  ///< Bit2-8: Unused
  unsigned char transponderType;  ///< Typ des Leseverfahrens.
  unsigned char count;            ///< Anzahl der Eintraege in dem Offsetarray, was auf diese Struktur folgt.
}; // 4 Byte

/** Diese Struktur dient bei Angabe eines Mifare Desfire Transponders in den Globalen Einstellungen, 
das zu verwendende Segment, Ablageformat sowie Auswahl.

*/
struct StructTransponderMifareDesfire
{
  unsigned char rwFlags;            ///< Gibt durch Flags folgendes an.
                                    ///< Bit1: Gesetzt bedeutet, UID lesen. (Seriennummer)
                                    ///< Bit2: Gesetzt bedeutet, Echte UID bei Random UID Ausweisen lesen.
                                    ///< Bit3: Gesetzt bedeutet, UID auch von anderen Mifare Ausweisen lesen.
                                    ///< Bit4: Gesetzt bedeutet, UID mit einer Laenge von 4 Byte liefern.
                                    ///< Bit5-8: Unused
  unsigned int applicationNumber;   ///< Nummer der zu verwendenden Applikation.
  unsigned char fileNumber;         ///< Nummer der zu verwendenden Datei.
  unsigned short rwOffset;          ///< Offset in die Daten der Datei.
  unsigned short rwLength;          ///< Anzahl zu lesender Datenbyte aus der Datei.
  unsigned char fileType;           ///< Type der Datei.
                                    ///< 0 -> Standard
                                    ///< 1 -> Backup
  unsigned char communicationMode;  ///< Modus der Kommunikation zwischen Ausweis und Leser.
                                    ///< 0 -> Plain
                                    ///< 1 -> MACed
                                    ///< 2 -> Crypt
  unsigned char keyNumber;          ///< Nummer des zu verwendenden Schluessels.
  unsigned char keyFlags;           ///< Passwortflags
                                    ///< Bit1: Passwort liegt im Verschluesselten Format vor.
                                    ///< Bit2-8: Unused
  unsigned char applicationKey[32]; ///< Zu verwendender Schluessel.
  unsigned char cryptType;          ///< Typ der Verschluesselung.
                                    ///< 0 -> DF4    (16 Byte Schluessel)
                                    ///< 1 -> TDES   (16 Byte Schluessel)
                                    ///< 2 -> AES    (16 Byte Schluessel)
                                    ///< 3 -> 3KTDES (24 Byte Schluessel)
  unsigned char outputFormat;       ///< Gibt das abzulegende Werteformat an.
                                    ///< 0 -> Dezimal
                                    ///< 1 -> Hexadezimal
                                    ///< 2 -> ASCII
                                    ///< 3 -> Hexadezimal gedreht
                                    ///< 4 -> Dezimal gedreht
  unsigned short outputLength;      ///< Gibt die abzulegende Werteformatlaenge an.
                                    ///< 0 -> Keine feste Laengenangabe
                                    ///< n -> Anzahl der abzulegenden Zeichen ggf. mit Vornullen auf die gewuenschte Laenge bringen.
  unsigned char unused[10];            ///< Reserve fuer Erweiterungen
}; // 60 Byte


///////////////////////////////////////
// Struct: StructTransponderMifarePlusHeader

/** Diese Struktur enthaelt zusaetzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen fuer die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von count unsigned short Offsets, die die Offsets auf die
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhaengig.
*/
struct StructTransponderMifarePlusHeader
{
  unsigned char structSize;       ///< Groesse dieser Struktur (ist gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray)
  unsigned char structFlags;      ///< Flags fuer die Strukturuebergreifende Verarbeitung.
                                  ///< Bit1: Passwoerter der Strukturen werden mittels RC4 und DatafoxKey verschluesselt.
                                  ///< Bit2-8: Unused
  unsigned char transponderType;  ///< Typ des Leseverfahrens.
  unsigned char count;            ///< Anzahl der Eintraege in dem Offsetarray, was auf diese Struktur folgt.
  unsigned char securityLevel;    ///< Anzuwendender Sicherheitslevel.
                                  ///< 0 -> Security Level 0 (Ausweis ist blanko)
                                  ///< 1 -> Security Level 1
                                  ///< 2 -> Security Level 2
                                  ///< 3 -> Security Level 3
  unsigned char iso14443A;        ///< Gibt im Security Level 1/2 an welche Ausweistypen von ISO14443A akzeptiert werden.
                                  ///< Bit1: ISO14443A Mini
                                  ///< Bit2: ISO14443A Classic 1k
                                  ///< Bit3: ISO14443A Classic 4k
                                  ///< Bit4: ISO14443A Plus S
                                  ///< Bit5: ISO14443A Plus X
}; // 6 Byte

/** Diese Struktur dient bei Angabe eines Mifare Plus Transponders in den Globalen Einstellungen, 
das zu verwendende Segment, Ablageformat sowie Auswahl.

*/
struct StructTransponderMifarePlus
{
  unsigned char rwFlags;              ///< Gibt durch Flags folgendes an.
                                      ///< Bit1: Gesetzt bedeutet, UID lesen. (Seriennummer)
                                      ///< Bit2: Gesetzt bedeutet, Echte UID bei Random UID Ausweisen lesen. Nur im Security level 3.
                                      ///< Bit3: Gesetzt bedeutet, UID auch von anderen Mifare Ausweisen lesen.
                                      ///< Bit4: Gesetzt bedeutet, UID mit einer Laenge von 4 Byte liefern.
                                      ///< Bit5-8: Unused
  unsigned char rwSector;             ///< Nummer des zu verwendenden Sektors.
  unsigned char rwBlock;              ///< Nummer des zu verwendenden Blocks.
  unsigned char rwOffset;             ///< Offset in den Datenblock.
  unsigned short rwLength;            ///< Anzahl zu lesender Datenbyte aus dem Sektor/Block.
  unsigned char passwordUsage;        ///< Passwortverwendung fuer Lesen / Schreiben.
                                      ///< 0 -> KeyA / KeyA
                                      ///< 1 -> KeyA / KeyB
                                      ///< 2 -> KeyB / KeyB
  unsigned char passwordFlags;        ///< Passwortflags
                                      ///< Bit1: Fuer Lesen auch die Standardpasswoerter verwenden.
                                      ///< Bit2: passwordA_Crypto1 liegt im Verschluesselten Format vor.
                                      ///< Bit3: passwordB_Crypto1
                                      ///< Bit4: passwordA_AES liegt im Verschluesselten Format vor.
                                      ///< Bit5: passwordB_AES liegt im Verschluesselten Format vor.
                                      ///< Bit6: instID liegt im Verschluesselten Format vor.
                                      ///< Bit7-8: Unused
  unsigned char passwordA_Crypto1[6]; ///< Zu verwendendes Passwort A fuer Crypto 1 (nur im Security Level 1/2)
  unsigned char passwordB_Crypto1[6]; ///< Zu verwendendes Passwort B fuer Crypto 1 (nur im Security Level 1/2)
  unsigned char passwordA_AES[16];    ///< Zu verwendendes Passwort A im Security level 2 / 3
                                      ///< Zu verwendender Virtual Card Polling Encryption key im SL 3 bei Random UID
  unsigned char passwordB_AES[16];    ///< Zu verwendendes Passwort B
                                      ///< Zu verwendender Virtual Card Polling MAC key im SL 3 bei Random UID
  unsigned char instID[16];           ///< Zu verwendender Installation Identifier im SL 3 bei Random UID
  unsigned char outputFormat;         ///< Gibt das abzulegende Werteformat an.
                                      ///< 0 -> Dezimal
                                      ///< 1 -> Hexadezimal
                                      ///< 2 -> ASCII
                                      ///< 3 -> Hexadezimal gedreht
                                      ///< 4 -> Dezimal gedreht
  unsigned short outputLength;        ///< Gibt die abzulegende Werteformatlaenge an.
                                      ///< 0 -> Keine feste Laengenangabe
                                      ///< n -> Anzahl der abzulegenden Zeichen ggf. mit Vornullen auf die gewuenschte Laenge bringen.
  unsigned char unused[9];               ///< Reserve fuer Erweiterungen
}; // 80 Byte


///////////////////////////////////////
// Struct: StructTransponderMifareUltralightHeader

/** Diese Struktur enthaelt zusaetzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen fuer die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von count unsigned short Offsets, die die Offsets auf die
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhaengig.
*/
struct StructTransponderMifareUltralightHeader
{
  unsigned char structSize;       ///< Groesse dieser Struktur (ist gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray)
  unsigned char structFlags;      ///< Flags fuer die Strukturuebergreifende Verarbeitung.
                                  ///< Bit1: Passwoerter der Strukturen werden mittels RC4 und DatafoxKey verschluesselt.
                                  ///< Bit2-8: Unused
  unsigned char transponderType;  ///< Typ des Leseverfahrens.
  unsigned char count;            ///< Anzahl der Eintraege in dem Offsetarray, was auf diese Struktur folgt.

}; // 4 Byte

/** Diese Struktur dient bei Angabe eines Mifare Ultralight Transponders in den Globalen Einstellungen, 
das zu verwendende Segment, Ablageformat sowie Auswahl.

*/
struct StructTransponderMifareUltralight
{
  unsigned char rwFlags;            ///< Gibt durch Flags folgendes an.
                                    ///< Bit1: Gesetzt bedeutet, UID lesen. (Seriennummer)
                                    ///< Bit2-8: Unused.
  unsigned char rwBlock;            ///< Nummer des zu verwendenden Blocks.
  unsigned short rwOffset;          ///< Offset in den Block.
  unsigned short rwLength;          ///< Anzahl zu lesender Datenbyte.
  unsigned char keyUsage;           ///< Verwenddung des Schluessels.
                                    ///< 0 -> Kein Passwort.
                                    ///< 1 -> Ausweise ohne und mit diesem Passwort akzeptieren.
                                    ///< 2 -> Nur Ausweise mit diesem Passwort akzeptieren.
  unsigned char keyFlags;           ///< Passwortflags
                                    ///< Bit1: Passwort liegt im Verschluesselten Format vor.
                                    ///< Bit2-8: Unused
  unsigned char key[16];            ///< Zu verwendender Schluessel.
  unsigned char outputFormat;       ///< Gibt das abzulegende Werteformat an.
                                    ///< 0 -> Dezimal
                                    ///< 1 -> Hexadezimal
                                    ///< 2 -> ASCII
                                    ///< 3 -> Hexadezimal gedreht
                                    ///< 4 -> Dezimal gedreht
  unsigned short outputLength;      ///< Gibt die abzulegende Werteformatlaenge an.
                                    ///< 0 -> Keine feste Laengenangabe
                                    ///< n -> Anzahl der abzulegenden Zeichen ggf. mit Vornullen auf die gewuenschte Laenge bringen.
  unsigned char unused[3];             ///< Reserve fuer Erweiterungen
}; // 30 Byte


///////////////////////////////////////
// Struct: StructTransponderiCodeHeader

/** Diese Struktur enthaelt zusaetzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen fuer die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von count unsigned short Offsets, die die Offsets auf die
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhaengig.
*/
struct StructTransponderiCodeHeader
{
  unsigned char structSize;       ///< Groesse dieser Struktur (ist gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray)
  unsigned char structFlags;      ///< Flags fuer die Strukturuebergreifende Verarbeitung.
                                  ///< Bit1: Passwoerter der Strukturen werden mittels RC4 und DatafoxKey verschluesselt.
                                  ///< Bit2-8: Unused
  unsigned char transponderType;  ///< Typ des Leseverfahrens.
  unsigned char count;            ///< Anzahl der Eintraege in dem Offsetarray, was auf diese Struktur folgt.
}; // 4 Byte

/** Diese Struktur dient bei Angabe eines iCode Transponders in den Globalen Einstellungen, 
das zu verwendende Segment, Ablageformat sowie Auswahl.

*/
struct StructTransponderiCode
{
  unsigned char rwFlags;            ///< Gibt durch Flags folgendes an.
                                    ///< Bit1: Gesetzt bedeutet, UID lesen. (Seriennummer)
                                    ///< Bit2-8: Unused.
  unsigned char rwBlock;            ///< Nummer des zu verwendenden Blocks.
  unsigned short rwOffset;          ///< Offset in den Block.
  unsigned short rwLength;          ///< Anzahl zu lesender Datenbyte.
  unsigned char outputFormat;       ///< Gibt das abzulegende Werteformat an.
                                    ///< 0 -> Dezimal
                                    ///< 1 -> Hexadezimal
                                    ///< 2 -> ASCII
                                    ///< 3 -> Hexadezimal gedreht
                                    ///< 4 -> Dezimal gedreht
  unsigned short outputLength;      ///< Gibt die abzulegende Werteformatlaenge an.
                                    ///< 0 -> Keine feste Laengenangabe
                                    ///< n -> Anzahl der abzulegenden Zeichen ggf. mit Vornullen auf die gewuenschte Laenge bringen.
  unsigned char unused[6];             ///< Reserve fuer Erweiterungen
}; // 15 Byte


///////////////////////////////////////
// Struct: StructTransponderJavaCardHeader

/** Diese Struktur enthaelt zusaetzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen fuer die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von count unsigned short Offsets, die die Offsets auf die
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhaengig.
*/
struct StructTransponderJavaCardHeader
{
	unsigned char structSize;       ///< Groesse dieser Struktur (ist gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray)
	unsigned char structFlags;      ///< Flags fuer die Strukturuebergreifende Verarbeitung.
	///< Bit1: Passwoerter der Strukturen werden mittels RC4 und DatafoxKey verschluesselt.
	///< Bit2-8: Unused
	unsigned char transponderType;  ///< Typ des Leseverfahrens.
	unsigned char count;            ///< Anzahl der Eintraege in dem Offsetarray, was auf diese Struktur folgt.
}; // 4 Byte

/** Diese Struktur dient bei Angabe eines JavaCard Transponders in den Globalen Einstellungen, 
das zu verwendende Segment, Ablageformat sowie Auswahl.

*/
struct StructTransponderJavaCard
{
	unsigned char rwFlags;            ///< Gibt durch Flags folgendes an.
	///< Bit1: Gesetzt bedeutet, UID lesen. (Seriennummer)
	///< Bit2-8: Unused.
	unsigned char rwBlock;            ///< Nummer des zu verwendenden Blocks.
	unsigned short rwOffset;          ///< Offset in den Block.
	unsigned short rwLength;          ///< Anzahl zu lesender Datenbyte.
	unsigned char outputFormat;       ///< Gibt das abzulegende Werteformat an.
	///< 0 -> Dezimal
	///< 1 -> Hexadezimal
	///< 2 -> ASCII
	///< 3 -> Hexadezimal gedreht
	///< 4 -> Dezimal gedreht
	unsigned short outputLength;      ///< Gibt die abzulegende Werteformatlaenge an.
	///< 0 -> Keine feste Laengenangabe
	///< n -> Anzahl der abzulegenden Zeichen ggf. mit Vornullen auf die gewuenschte Laenge bringen.
	unsigned char unused[6];             ///< Reserve fuer Erweiterungen
}; // 15 Byte


///////////////////////////////////////
// Struct: StructTransponderMyDHeader

/** Diese Struktur enthaelt zusaetzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen fuer die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von count unsigned short Offsets, die die Offsets auf die
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhaengig.
*/
struct StructTransponderMyDHeader
{
  unsigned char structSize;       ///< Groesse dieser Struktur (ist gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray)
  unsigned char structFlags;      ///< Flags fuer die Strukturuebergreifende Verarbeitung.
                                  ///< Bit1: Passwoerter der Strukturen werden mittels RC4 und DatafoxKey verschluesselt.
                                  ///< Bit2-8: Unused
  unsigned char transponderType;  ///< Typ des Leseverfahrens.
  unsigned char count;            ///< Anzahl der Eintraege in dem Offsetarray, was auf diese Struktur folgt.
  unsigned char isoFlags;         ///< Enthaelt die zugelassenen Ausweistypen.
                                  ///< Bit1: Bei gesetztem Bit sind ISO14443 Ausweise zugelassen.
                                  ///< Bit2: Bei gesetztem Bit sind ISO15693 Ausweise zugelassen.
                                  ///< Bit3-8: Unused
}; // 5 Byte

/** Diese Struktur dient bei Angabe eines MyD Transponders in den Globalen Einstellungen, 
das zu verwendende Segment, Ablageformat sowie Auswahl.

*/
struct StructTransponderMyD
{
  unsigned char rwFlags;            ///< Gibt durch Flags folgendes an.
                                    ///< Bit1: Gesetzt bedeutet, UID lesen. (Seriennummer)
                                    ///< Bit2-8: Unused.
  unsigned short rwBlock;           ///< Nummer des zu verwendenden Blocks.
  unsigned short rwOffset;          ///< Offset in den Block.
  unsigned short rwLength;          ///< Anzahl zu lesender Datenbyte.
  unsigned char keyUsage;           ///< Verwenddung des Schluessels.
                                    ///< 0 -> Kein Passwort.
                                    ///< 1 -> Ausweise ohne und mit diesem Passwort akzeptieren.
                                    ///< 2 -> Nur Ausweise mit diesem Passwort akzeptieren.
  unsigned char keyFlags;           ///< Passwortflags
                                    ///< Bit1: keyA liegt im Verschluesselten Format vor.
                                    ///< Bit2: keyB liegt im Verschluesselten Format vor.
                                    ///< Bit3-8: Unused
  unsigned char keyA[8];            ///< Zu verwendender Schluessel A.
  unsigned char keyB[8];            ///< Zu verwendender Schluessel B.
  unsigned char outputFormat;       ///< Gibt das abzulegende Werteformat an.
                                    ///< 0 -> Dezimal
                                    ///< 1 -> Hexadezimal
                                    ///< 2 -> ASCII
                                    ///< 3 -> Hexadezimal gedreht
                                    ///< 4 -> Dezimal gedreht
  unsigned short outputLength;      ///< Gibt die abzulegende Werteformatlaenge an.
                                    ///< 0 -> Keine feste Laengenangabe
                                    ///< n -> Anzahl der abzulegenden Zeichen ggf. mit Vornullen auf die gewuenschte Laenge bringen.
  unsigned char unused[2];             ///< Reserve fuer Erweiterungen
}; // 30 Byte


///////////////////////////////////////
// Struct: StructTransponderISOTagsHeader

/** Diese Struktur enthaelt zusaetzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen fuer die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von count unsigned short Offsets, die die Offsets auf die
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhaengig.
*/
struct StructTransponderISOTagsHeader
{
  unsigned char structSize;       ///< Groesse dieser Struktur (ist gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray)
  unsigned char structFlags;      ///< Flags fuer die Strukturuebergreifende Verarbeitung.
                                  ///< Bit1: Passwoerter der Strukturen werden mittels RC4 und DatafoxKey verschluesselt.
                                  ///< Bit2-8: Unused
  unsigned char transponderType;  ///< Typ des Leseverfahrens.
  unsigned char count;            ///< Anzahl der Eintraege in dem Offsetarray, was auf diese Struktur folgt.
}; // 4 Byte

/** Diese Struktur dient bei Angabe eines ISO Tags Transponders in den Globalen Einstellungen, 
das zu verwendende Segment, Ablageformat sowie Auswahl.

*/
struct StructTransponderISOTags
{
  unsigned char isoType;          ///< Gibt durch Flags an, welche ISO Ausweise gelesen werden sollen.
                                  ///< Bit1: ISO14443A
                                  ///< Bit2: ISO14443B
                                  ///< Bit3: ISO14443-2
                                  ///< Bit4: ISO14443-3
                                  ///< Bit5: ISO15693
                                  ///< Bit6-8: Unused
  unsigned char iso14443A;        ///< Gibt an welche Ausweistypen von ISO14443A gelesen werden sollen.
                                  ///< Bit1: ISO14443A Mini
                                  ///< Bit2: ISO14443A Classic 1k
                                  ///< Bit3: ISO14443A Classic 4k
                                  ///< Bit4: ISO14443A Plus S
                                  ///< Bit5: ISO14443A Plus X
                                  ///< Bit6: ISO14443A Ultralight
                                  ///< Bit7: ISO14443A DESFire
                                  ///< Bit8: ISO14443A LegicAdvant
  unsigned char iso14443A_2;      ///< Gibt an welche Ausweistypen von ISO14443A gelesen werden sollen.
                                  ///< Bit1: ISO14443A Andere (SmartMX, usw.)
                                  ///< Bit2-8: Unused
  unsigned char iso15693;         ///< Gibt an welche Ausweistypen von ISO15693 gelesen werden sollen.
                                  ///< Bit1: ISO15693i Code SL
                                  ///< Bit2: ISO15693 Tag-it HF-I
                                  ///< Bit3: ISO15693 My-D vicinity
                                  ///< Bit4: ISO15693 Andere (LegicAdvant usw.)
                                  ///< Bit5: ISO15693 24LR16 / 24LR64 von ST
                                  ///< Bit6: ISO15693 Fujitsu MB89R118 / MB89R119
                                  ///< Bit7: ISO15693 LRI2k / LRI64k von ST
                                  ///< Bit8: Unused
  unsigned char outputFormat;     ///< Gibt das abzulegende Werteformat an.
                                  ///< 0 -> Dezimal
                                  ///< 1 -> Hexadezimal
                                  ///< 2 -> unbenutzt (ASCII)
                                  ///< 3 -> Hexadezimal gedreht
                                  ///< 4 -> Dezimal gedreht
  unsigned char outputLength;     ///< Gibt die abzulegende Werteformatlaenge an.
                                  ///< 0 -> Keine feste Laengenangabe
                                  ///< n -> Anzahl der abzulegenden Zeichen ggf. mit Vornullen auf die gewuenschte Laenge bringen.
  unsigned char unused[4];           ///< Reserve fuer Erweiterungen
}; // 10 Byte


///////////////////////////////////////
// Struct: StructTransponderETSITagsHeader

/** Diese Struktur enthaelt zusaetzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen fuer die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von count unsigned short Offsets, die die Offsets auf die
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhaengig.
*/
struct StructTransponderETSITagsHeader
{
    unsigned char structSize;       ///< Groesse dieser Struktur (ist gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray)
    unsigned char structFlags;      ///< Flags fuer die Strukturuebergreifende Verarbeitung.
                                    ///< Bit1: Passwoerter der Strukturen werden mittels RC4 und DatafoxKey verschluesselt.
                                    ///< Bit2-8: Unused
    unsigned char transponderType;  ///< Typ des Leseverfahrens.
    unsigned char count;            ///< Anzahl der Eintraege in dem Offsetarray, was auf diese Struktur folgt.
}; // 4 Byte

/** Diese Struktur dient bei Angabe eines ETSI Tag Transponders in den Globalen Einstellungen, 
das zu verwendende Segment, Ablageformat sowie Auswahl.

*/
struct StructTransponderETSITags
{
    unsigned char strength;         ///< Gibt die Feldstaerke in % an.
    unsigned char rwFlags;          ///< Gibt durch Flags folgendes an.
                                    ///< Bit1: Gesetzt bedeutet, EPC lesen. Ansonsten soll der angegebene Bank gelesen werden.
                                    ///< Bit2: Gesetzt bedeutet, Mehrachbuchung bei EPC. Es wird für bis zu 10 im Feld befindliche Ausweise eine separate Buchung durchgeführt.
                                    ///< Bit3-8: Unused
    unsigned char rwBank;           ///< Nummer der zu verwendenden Bank (0 - 3).
    unsigned short rwOffset;        ///< Offset in den Datenblock.
    unsigned short rwLength;        ///< Anzahl zu lesender Datenbyte aus dem Sektor/Block.
    unsigned int password;          ///< Passwortverwendung fuer Lesen / Schreiben.
    unsigned char outputFormat;     ///< Gibt das abzulegende Werteformat an.
                                    ///< 0 -> Dezimal
                                    ///< 1 -> Hexadezimal
                                    ///< 2 -> unbenutzt (ASCII)
                                    ///< 3 -> Hexadezimal gedreht
                                    ///< 4 -> Dezimal gedreht
    unsigned short outputLength;    ///< Gibt die abzulegende Werteformatlaenge an.
                                    ///< 0 -> Keine feste Laengenangabe
                                    ///< n -> Anzahl der abzulegenden Zeichen ggf. mit Vornullen auf die gewuenschte Laenge bringen.
    unsigned char unused[4];           ///< Reserve fuer Erweiterungen
}; // 18 Byte


///////////////////////////////////////
// Struct: StructTransponderQ5Header

/** Diese Struktur enthaelt zusaetzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen fuer die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von count unsigned short Offsets, die die Offsets auf die
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhaengig.
*/
struct StructTransponderQ5Header
{
    unsigned char structSize;       ///< Groesse dieser Struktur (ist gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray)
    unsigned char structFlags;      ///< Flags fuer die Strukturuebergreifende Verarbeitung.
                                    ///< Bit1: Passwoerter der Strukturen werden mittels RC4 und DatafoxKey verschluesselt.
                                    ///< Bit2-8: Unused
    unsigned char transponderType;  ///< Typ des Leseverfahrens.
    unsigned char count;            ///< Anzahl der Eintraege in dem Offsetarray, was auf diese Struktur folgt.
    unsigned char isoFlags;         ///< Enthaelt die zugelassenen Ausweistypen.
                                    ///< Bit1: Bei gesetztem Bit sind ISO14443 Ausweise zugelassen.
                                    ///< Bit2: Bei gesetztem Bit sind ISO15693 Ausweise zugelassen.
                                    ///< Bit3-8: Unused
}; // 5 Byte

/** Diese Struktur dient bei Angabe eines Q5 Transponders in den Globalen Einstellungen, 
das zu verwendende Segment, Ablageformat sowie Auswahl.

*/
struct StructTransponderQ5
{
    unsigned char rwFlags;          ///< Gibt durch Flags folgendes an.
                                    ///< Bit1: Gesetzt bedeutet, UID lesen. (Seriennummer)
                                    ///< Bit2-8: Unused.
    unsigned short rwBlock;         ///< Nummer des zu verwendenden Blocks.
    unsigned short rwOffset;        ///< Offset in den Block.
    unsigned short rwLength;        ///< Anzahl zu lesender Datenbyte.
    unsigned char keyUsage;         ///< Verwenddung des Schluessels.
                                    ///< 0 -> Kein Passwort.
                                    ///< 1 -> Ausweise ohne und mit diesem Passwort akzeptieren.
                                    ///< 2 -> Nur Ausweise mit diesem Passwort akzeptieren.
    unsigned char keyFlags;         ///< Passwortflags
                                    ///< Bit1: keyA liegt im Verschluesselten Format vor.
                                    ///< Bit2: keyB liegt im Verschluesselten Format vor.
                                    ///< Bit3-8: Unused
    unsigned char keyA[8];          ///< Zu verwendender Schluessel A.
    unsigned char keyB[8];          ///< Zu verwendender Schluessel B.
    unsigned char outputFormat;     ///< Gibt das abzulegende Werteformat an.
                                    ///< 0 -> Dezimal
                                    ///< 1 -> Hexadezimal
                                    ///< 2 -> ASCII
                                    ///< 3 -> Hexadezimal gedreht
                                    ///< 4 -> Dezimal gedreht
    unsigned short outputLength;    ///< Gibt die abzulegende Werteformatlaenge an.
                                    ///< 0 -> Keine feste Laengenangabe
                                    ///< n -> Anzahl der abzulegenden Zeichen ggf. mit Vornullen auf die gewuenschte Laenge bringen.
    unsigned char unused[2];           ///< Reserve fuer Erweiterungen
}; // 30 Byte

#endif
