// DFFileStudio.h: Schnittstelle für die Klasse CDFFileStudio.
//
//////////////////////////////////////////////////////////////////////

#ifndef FILESTUDIO_H
#define FILESTUDIO_H

#include "../Common/aes.h"
#include "../Common/crc.h"
#include "../Common/md5.h"
#include "../Common/DFSoftwareVersion.h"
#include <stdio.h>

// Angabe über die benötigten Mindestversionen für die Setupdatei.
#define NEED_STUDIO_VERSION   "04.02.02.08"
#define NEED_LIBRARY_VERSION  "04.02.02.08"

// Angabe ueber die maximal bekannte Firmware-Version.
#define MAX_FIRMWARE1_VERSION "04.01.09.99"
#define MAX_FIRMWARE2_VERSION "04.02.05.99"
#define MAX_FIRMWARE3_VERSION "04.03.06.99"

#define FILESTUDIOHEADER_CHECK 0xDFDFDFDF ///< StructFileStudioHeader.check: Pruefwert fuer Gueltigkeit (Versionskennung) 
// Einen Dezimalwert 99 in Hexwert 0x99 wandeln.
//#define AESHEADER_DEC2VERSION(x) (x + (x / 10) * 6)
// Einen Hexwert 0x99 in Dezimalwert 99 wandeln.
//#define AESHEADER_VERSION2DEC(x) (x - (x / 16) * 6)
#define FILESTUDIOHEADER_VERSION(master, major, minor, build) ((master & 0xFF) << 24 | (major & 0xFF) << 16 | (minor & 0xFF) << 8 | (build & 0xFF))
#define FILESTUDIOHEADER_VERSION_UNSUPPORTED                  FILESTUDIOHEADER_VERSION(99, 99, 99, 99)

// Flags fuer den Dateiheader
#define FILESTUDIOHEADER_FLAG_IGNORE_SHOULD_STUDIO_VERSION        0x00000001
#define FILESTUDIOHEADER_FLAG_IGNORE_SHOULD_LIBRARY_VERSION       0x00000002
#define FILESTUDIOHEADER_FLAG_IGNORE_SHOULD_FIRMWARE1_VERSION     0x00000004
#define FILESTUDIOHEADER_FLAG_IGNORE_SHOULD_FIRMWARE2_VERSION     0x00000008
#define FILESTUDIOHEADER_FLAG_IGNORE_SHOULD_FIRMWARE3_VERSION     0x00000010

// Flags fuer die Loadroutine Pruefung auf Versionen
#define FILESTUDIOHEADER_IGNORE_MIN_STUDIO_VERSION            0x00000001
#define FILESTUDIOHEADER_IGNORE_SHOULD_STUDIO_VERSION         0x00000002
#define FILESTUDIOHEADER_IGNORE_MIN_LIBRARY_VERSION           0x00000004
#define FILESTUDIOHEADER_IGNORE_SHOULD_LIBRARY_VERSION        0x00000008

// Wird der Header verwendet, wird eine Firmware 04.00.xx nicht mehr unterstuetzt!

struct StructFileStudioHeader
{
    unsigned int check;                      ///< Muss immer mit FILESTUDIOHEADER_CHECK belegt werden. 
    ///< Wird herangezogen um zu erkennen ob eine Headerstruktur in der Datei vorliegt.
    unsigned int bugfixSize;                 ///< Groesse der Struktur sizeof_StructFileStudioHeader_1, Version 1. Naeheres siehe Erleuterung zur bugfixSize weiter unten.
    ///< Ist auch gleichzeitig eine Art Datei-Version. Die Struktur kann derzeit maximal (EEP_SIZE - EEP_SIZE_SU) gross werden.
    unsigned int flags;                      ///< Flags fuer z. B. "Gelesen mit DFComDLL, jedoch noch nicht bearbeitet"
    unsigned int studioVersion;              ///< Studioversion mit der die Datei zuletzt abgespeichert wurde.
    ///< Wurde die Datei ueber die DLL ausgelesen liegt hier keine Angabe vor!
    // Mindestversionen
    unsigned int minStudioVersion;           ///< Mindestversion des einzusetzenden Studios, um alle enthaltenen Daten zu unterstuetzen.
    unsigned int shouldStudioVersion;        ///< Sollteversion des einzusetzenden Studios, um alle enthaltenen Daten zu unterstuetzen.
    unsigned int minLibraryVersion;          ///< Mindestversion der zu verwendenden DFComDLL fuer die korrekte Uebertragung der Setupdaten.
    unsigned int shouldLibraryVersion;       ///< Sollteversion der zu verwendenden DFComDLL fuer die korrekte Uebertragung der Setupdaten.
    unsigned int minFirmware1Version;        ///< Mindestversion der zu verwendenden Firmware 04.01.xx.xx, um alle gewaehlten Einstellungen zu unterstuetzen.
    unsigned int shouldFirmware1Version;     ///< Sollteversion der zu verwendenden Firmware 04.01.xx.xx, um alle gewaehlten Einstellungen zu unterstuetzen.
    unsigned int minFirmware2Version;        ///< Mindestversion der zu verwendenden Firmware 04.02.xx.xx, um alle gewaehlten Einstellungen zu unterstuetzen.
    unsigned int shouldFirmware2Version;     ///< Sollteversion der zu verwendenden Firmware 04.02.xx.xx, um alle gewaehlten Einstellungen zu unterstuetzen.
    // CRC der Dateibereiche
    unsigned int crcEep;                     ///< CRC ueber Eep-Bereich.
    unsigned int crcEepExtended;             ///< CRC ueber EepExtended-Bereich (dort liegt auch Header jedoch mit eigenem CRC)
    unsigned int crcFlash;                   ///< CRC ueber Flash-Bereich.
    unsigned int crcListFile;                ///< CRC ueber die Listendateiangaben.
    unsigned int crcConfig;                  ///< CRC ueber die Config-Struktur.
    unsigned int crcFlashExtended;           ///< CRC ueber FlashExtended-Bereich.
    unsigned int crcListEntranceFile;        ///< CRC ueber die Listendateiangaben der ZK1.
    unsigned int crcListEntrance2File;       ///< CRC ueber die Listendateiangaben der ZK2.
    unsigned int crcListExtendedFile;        ///< CRC ueber die Zusaetzlichen Listendateiangaben.
    // Mindestversionen (Hinzugefuegt ab Version 04.03.xx)
    unsigned int bugfixCRC;                  ///< CRC der Headerstruktur Version 1. Naeheres siehe Erleuterung zum bugfixCRC weiter unten.
    ///< Folgende Angaben sind ab VERSION 2 verfuegbar.
    unsigned int size;                       ///< Groesse der Struktur sizeof(StructFileStudioHeader) ab Version 2.
    ///< ACHTUNG: Bei Verwendung wird eine Mindestversion 04.03.xx des Studio und der Library vorausgesetzt.
    unsigned int minFirmware3Version;        ///< Mindestversion der zu verwendenden Firmware 04.03.xx.xx, um alle gewaehlten Einstellungen zu unterstuetzen.
    unsigned int shouldFirmware3Version;     ///< Sollteversion der zu verwendenden Firmware 04.03.xx.xx, um alle gewaehlten Einstellungen zu unterstuetzen.
    //...
    unsigned int crc;                        ///< CRC ueber alle Kopfdaten
};

#define sizeof_StructFileStudioHeader_1     88 // Strukturgroesse der Version 1 ist: 88Byte
#define sizeof_StructFileStudioHeader_2    104 // Strukturgroesse der Version 2 ist: 104Byte

/*
Erlaeuterung zum bugfixCRC.
Vorgaengerversionen der Software pruefen zuerst den CRC ueber ihren direkten Strukturzugriff (Version 1). Da sich der CRC bei aktuelleren Versionen jedoch weiter nach hinten
verschiebt, fuehrt dieses dazu, dass es zu einem Head-CRC-Fehler kommt bevor die Pruefung auf die Mindestversion der Library oder des Studios kommt.
Ab dieser Version wird der CRC in den letzten 4 Byte der Struktur auf Grundlage ihrer size Angabe erwartet. Zukuenftig hinzugefuegte Werte in der Struktur werden
unangetastet mitverarbeitet. Muss eine dieser Variablen beachtet werden, muss die Version der Library und des Studios auf die entsprechende Mindestversion gesetzt werden. 
Wird zum Beispiel der Parameter minFirmware3Version verwendet, muss die Library und Studio Version auf mindestens 4.3.0.0 gestellt werden, damit dieser Parameter auch
beachtet und verarbeitet wird.
*/

class FileStudio  
{
public:
    FileStudio();
    virtual ~FileStudio();

    void SetDefault( int flags = 0 );
    void SetDefaultHeader();

    int Load( const char *fileName, int *error, int flags );
    int Save( const char *fileName, int *error );
    unsigned int GetFlashSetupSize();
    unsigned int GetExtendedFlashSetupSize();
    unsigned int GetTablesSize( bool lists );  ///< Ermittelt die Groesse der Listen- oder Datensazbeschreibungen.
    int NeededVersion();
    void ChangeOrderLong( unsigned char *Buf );

    int buildHeader();
    int checkHeader();

    inline int IsHeader()
    {
        return (m_fileHeader && m_fileHeader->check == FILESTUDIOHEADER_CHECK);
    }

    inline StructFileStudioHeader& GetHeader()
    { 
        return (StructFileStudioHeader&)*m_fileHeader; 
    }

    void SetStudioVersion( const char *version )
    {
        m_studioVersion.Parse( version );
        m_fileHeader->studioVersion = FILESTUDIOHEADER_VERSION( m_studioVersion.GetMaster(), m_studioVersion.GetMajor(), m_studioVersion.GetMinor(), m_studioVersion.GetBuild() );
    }

    // Anpassung auf eventuell benoetigte Mindestversion.
    void NeedStudioVersion( const char *version )
    {
        CDFSoftwarebuildVersion minStudioVersion( m_fileHeader->minStudioVersion );
        CDFSoftwarebuildVersion minVersion( version );
        if ( minStudioVersion < minVersion )
            m_fileHeader->minStudioVersion = FILESTUDIOHEADER_VERSION( minVersion.GetMaster(), minVersion.GetMajor(), minVersion.GetMinor(), minVersion.GetBuild() );
    }

    void ShouldStudioVersion( const char *version )
    {
        CDFSoftwarebuildVersion studioVersion( m_fileHeader->shouldStudioVersion );
        CDFSoftwarebuildVersion shouldVersion( version );
        if ( studioVersion < shouldVersion )
            m_fileHeader->shouldStudioVersion = FILESTUDIOHEADER_VERSION( shouldVersion.GetMaster(), shouldVersion.GetMajor(), shouldVersion.GetMinor(), shouldVersion.GetBuild() );
    }

    void SetLibraryVersion( const char *version )
    {
        m_libraryVersion.Parse( version );
    }

    void NeedLibraryVersion( const char *version )
    {
        CDFSoftwarebuildVersion minLibraryVersion( m_fileHeader->minLibraryVersion );
        CDFSoftwarebuildVersion minVersion( version );
        if ( minLibraryVersion < minVersion )
            m_fileHeader->minLibraryVersion = FILESTUDIOHEADER_VERSION( minVersion.GetMaster(), minVersion.GetMajor(), minVersion.GetMinor(), minVersion.GetBuild() );
    }

    void ShouldLibraryVersion( const char *version )
    {
        CDFSoftwarebuildVersion libraryVersion( m_fileHeader->shouldLibraryVersion );
        CDFSoftwarebuildVersion shouldVersion( version );
        if ( libraryVersion < shouldVersion )
            m_fileHeader->shouldLibraryVersion = FILESTUDIOHEADER_VERSION( shouldVersion.GetMaster(), shouldVersion.GetMajor(), shouldVersion.GetMinor(), shouldVersion.GetBuild() );
    }

    void ShouldFirmware1Version( const char *version )
    {
        CDFSoftwarebuildVersion firmwareVersion( m_fileHeader->shouldFirmware1Version );
        CDFSoftwarebuildVersion shouldVersion( version );
        if ( firmwareVersion < shouldVersion )
        {
            m_fileHeader->shouldFirmware1Version = FILESTUDIOHEADER_VERSION( shouldVersion.GetMaster(), shouldVersion.GetMajor(), shouldVersion.GetMinor(), shouldVersion.GetBuild() );
            NeedLibraryVersion( "04.02.02.08" );
            NeedStudioVersion( "04.02.02.08" );
        }
    }

    void NeedFirmware1Version( const char *version )
    {
        CDFSoftwarebuildVersion minFirmwareVersion( m_fileHeader->minFirmware1Version );
        CDFSoftwarebuildVersion minVersion( version );
        if ( minFirmwareVersion < minVersion )
        {
            m_fileHeader->minFirmware1Version = FILESTUDIOHEADER_VERSION( minVersion.GetMaster(), minVersion.GetMajor(), minVersion.GetMinor(), minVersion.GetBuild() );
            NeedLibraryVersion( "04.02.02.08" );
            NeedStudioVersion( "04.02.02.08" );
        }
    }

    void UnsupportFirmware1Version()
    {
        m_fileHeader->minFirmware1Version = FILESTUDIOHEADER_VERSION_UNSUPPORTED;
    }

    bool supportFirmware1Version()
    {
        return (m_fileHeader->minFirmware1Version != FILESTUDIOHEADER_VERSION_UNSUPPORTED);
    }

    void ShouldFirmware2Version( const char *version )
    {
        CDFSoftwarebuildVersion firmwareVersion( m_fileHeader->shouldFirmware2Version );
        CDFSoftwarebuildVersion shouldVersion( version );
        if ( firmwareVersion < shouldVersion )
        {
            m_fileHeader->shouldFirmware2Version = FILESTUDIOHEADER_VERSION( shouldVersion.GetMaster(), shouldVersion.GetMajor(), shouldVersion.GetMinor(), shouldVersion.GetBuild() );
            NeedLibraryVersion( "04.02.02.08" );
            NeedStudioVersion( "04.02.02.08" );
        }
    }

    void NeedFirmware2Version( const char *version )
    {
        CDFSoftwarebuildVersion minFirmwareVersion( m_fileHeader->minFirmware2Version );
        CDFSoftwarebuildVersion minVersion( version );
        if ( minFirmwareVersion < minVersion )
        {
            m_fileHeader->minFirmware2Version = FILESTUDIOHEADER_VERSION( minVersion.GetMaster(), minVersion.GetMajor(), minVersion.GetMinor(), minVersion.GetBuild() );
            NeedLibraryVersion( "04.02.02.08" );
            NeedStudioVersion( "04.02.02.08" );
        }
    }

    void UnsupportFirmware2Version()
    {
        m_fileHeader->minFirmware2Version = FILESTUDIOHEADER_VERSION_UNSUPPORTED;
    }

    bool supportFirmware2Version()
    {
        return (m_fileHeader->minFirmware2Version != FILESTUDIOHEADER_VERSION_UNSUPPORTED);
    }

    void ShouldFirmware3Version( const char *version )
    {
        CDFSoftwarebuildVersion firmwareVersion( m_fileHeader->shouldFirmware3Version );
        CDFSoftwarebuildVersion shouldVersion( version );
        if ( firmwareVersion < shouldVersion )
        {
            m_fileHeader->shouldFirmware3Version = FILESTUDIOHEADER_VERSION( shouldVersion.GetMaster(), shouldVersion.GetMajor(), shouldVersion.GetMinor(), shouldVersion.GetBuild() );
            NeedLibraryVersion( "04.03.00.08" );
            NeedStudioVersion( "04.03.00.08" );
        }
    }

    void NeedFirmware3Version( const char *version )
    {
        CDFSoftwarebuildVersion minFirmwareVersion( m_fileHeader->minFirmware3Version );
        CDFSoftwarebuildVersion minVersion( version );
        if ( minFirmwareVersion < minVersion )
        {
            m_fileHeader->minFirmware3Version = FILESTUDIOHEADER_VERSION( minVersion.GetMaster(), minVersion.GetMajor(), minVersion.GetMinor(), minVersion.GetBuild() );
            NeedLibraryVersion( "04.03.00.08" );
            NeedStudioVersion( "04.03.00.08" );
        }
    }

    void UnsupportFirmware3Version()
    {
        m_fileHeader->minFirmware3Version = FILESTUDIOHEADER_VERSION_UNSUPPORTED;
        NeedLibraryVersion( "04.03.00.08" );
        NeedStudioVersion( "04.03.00.08" );
    }

    bool supportFirmware3Version()
    {
        return (m_fileHeader->minFirmware3Version != FILESTUDIOHEADER_VERSION_UNSUPPORTED);
    }

    void NeedFirmwareVersion( const char *version )
    {
        CDFSoftwarebuildVersion fwVersion( version );
        // Pruefung auf 04.01.xx
        if ( fwVersion >= "04.01.01.00" && fwVersion < "04.02.00.00" )
        {
            // Mindestversion der Firmware 04.01.xx.yy setzen.
            NeedFirmware1Version( version );
        }
        else if ( fwVersion >= "04.02.00.00" && fwVersion < "04.03.00.00" )
        {
            // Mindestversion der Firmware 04.02.xx.yy setzen.
            NeedFirmware2Version( version );
        }
        else if ( fwVersion >= "04.03.00.00" && fwVersion < "04.04.00.00" )
        {
            // Mindestversion der Firmware 04.03.xx.yy setzen.
            NeedFirmware3Version( version );
        }
    }

    void ShouldFirmwareVersion( const char *version )
    {
        CDFSoftwarebuildVersion fwVersion( version );
        // Pruefung auf 04.01.xx
        if ( fwVersion >= "04.01.01.00" && fwVersion < "04.02.00.00" )
        {
            // Empfohlene Firmware 04.01.xx.yy eintragen.
            ShouldFirmware1Version( version );
        }
        else if ( fwVersion >= "04.02.00.00" && fwVersion < "04.03.00.00" )
        {
            // Empfohlene Firmware 04.02.xx.yy pruefen?
            ShouldFirmware2Version( version );
        }
        else if ( fwVersion >= "04.03.00.00" && fwVersion < "04.04.00.00" )
        {
            // Empfohlene Firmware 04.03.xx.yy pruefen?
            ShouldFirmware3Version( version );
        }
    }

    int IsSupportedFirmwareVersion( const char *version )
    {
        CDFSoftwarebuildVersion fwVersion( version );
        // Pruefung auf 04.xx.yy
        if ( (fwVersion >= "04.01.01.00" && fwVersion <= MAX_FIRMWARE1_VERSION) 
            || (fwVersion >= "04.02.00.00" && fwVersion <= MAX_FIRMWARE2_VERSION) 
            || (fwVersion >= "04.03.00.00" && fwVersion <= MAX_FIRMWARE3_VERSION) )
        {
                return 1;
        }

        return 0;
    }
    int ValidFirmwareVersion( CDFSoftwarebuildVersion &fwVersion );

    // CRC ueber Dateibereiche
    unsigned int GetCrcEep();                ///< CRC ueber Eep-Bereich.
    unsigned int GetCrcEepExtended();        ///< CRC ueber Eep-Bereich.
    unsigned int GetCrcFlash();              ///< CRC ueber Flash-Bereich.
    unsigned int GetCrcListFile();           ///< CRC ueber die Listendateiangaben.
    unsigned int GetCrcConfig();             ///< CRC ueber die Config-Struktur.
    unsigned int GetCrcFlashExtended();      ///< CRC ueber FlashExtended-Bereich.
    unsigned int GetCrcListEntranceFile();   ///< CRC ueber die Listendateiangaben der ZK1.
    unsigned int GetCrcListEntrance2File();  ///< CRC ueber die Listendateiangaben der ZK2.
    unsigned int GetCrcListExtendedFile();   ///< CRC ueber die Zusaetzlichen Listendateiangaben.
    // MD5 ueber Dateibereiche.
    char *GetMD5Header();                    ///< MD5 ueber die Struktur fuer den Dateikopf.
    char *GetMD5Tables( bool lists );        ///< MD5 ueber die Listenbeschreibungen.

public:
    CDFSoftwarebuildVersion m_studioVersion;
    CDFSoftwarebuildVersion m_libraryVersion;
    StructFileStudioHeader *m_fileHeader;
    StructConfig m_config;
    unsigned char *m_configMem;
    unsigned char *m_eep;
    unsigned char *m_header;
    unsigned char *m_flash;
    unsigned char *m_flashExtended;
    unsigned int m_flashSizeSU2;
    unsigned int m_flashMemOffset;

public:
    // Pfadangaben der Listen / Zusatzpfade
    char m_listFileName[MAX_DATENSATZ_BESCHREIBUNG][260];
    char m_listEntranceFileName[4][260];
    char m_listEntrance2FileName[MAX_DATENSATZ_BESCHREIBUNG][260];
    char m_strExtendedFilePath[MAX_DATENSATZ_BESCHREIBUNG][260];
};

#endif
